@@ -1,7 +1,7 @@
 /*
  * abcm2ps: a program to typeset tunes written in abc format using PostScript
  *
- * Copyright (C) 1998-2005 Jean-Franois Moine
+ * Copyright (C) 1998-2006 Jean-Franois Moine
  *
  * Adapted from abc2ps-1.2.5:
  *  Copyright (C) 1996,1997  Michael Methfessel
@@ -20,9 +20,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * Jean-Franois Moine
- *	mailto:moinejf@free.fr
- *
  * Original page:
  *	http://moinejf.free.fr/
  *
@@ -79,30 +76,9 @@ int nwhistle;
 
 /* -- local variables -- */
 
-static int include_xrefs = -1;	/* to include xref numbers in title */
-static int one_per_page = -1;	/* new page for each tune */
 static int splittune = -1;	/* tune may be splitted */
-static int write_history = -1;	/* write history and notes */
-static float alfa_c = -1.0;	/* max compression allowed */
-static int bars_per_line = -1;	/* bars for auto linebreaking */
-static int encoding = -1;	/* latin encoding number */
-static int continue_lines = -1;	/* flag to continue all lines */
 static int deco_old;		/* abc2ps decorations */
-static int landscape = -1;	/* flag for landscape output */
-static float lmargin = -1.0;	/* left margin */
-static float indent = -1.0;	/* 1st line indentation */
-static int music_only = -1;	/* no vocals if 1 */
-static int flatbeams = -1;	/* flat beams when bagpipe */
-static int graceslurs = -1;	/* slurs in grace notes */
-static float scalefac = 0;	/* scale factor for symbol size */
-static float staffsep = 0;	/* staff separation */
 static int def_fmt_done = 0;	/* default format read */
-static float swidth = 0;	/* staff width */
-static int measurenb = -1;	/* measure numbering (-1: none, 0: on the left, or every n bars) */
-static int measurebox = -1;	/* display measure numbers in a box */
-static int measurefirst = -1;	/* first measure number */
-static int printtempo = -1;	/* print the tempo indications */
-static char *sel;		/* current input file selector */
 static struct SYMBOL notitle;
 
 /* memory arena (for clrarena, lvlarena & getarena) */
@@ -122,7 +98,7 @@ static void do_filter(struct abctune *t,
 static void do_select(struct abctune *t,
 		      int first_tune,
 		      int last_tune);
-static void output_file(void);
+static void output_file(char *sel);
 static void read_def_format(void);
 static char *read_file(void);
 static void set_page_format(void);
@@ -138,7 +114,7 @@ int main(int argc,
 	 char **argv)
 {
 	int j;
-	char *p;
+	char *p, *sel;
 	char c, *aaa;
 
 	fprintf(stderr, "abcm2ps-" VERSION " (" VDATE ")\n");
@@ -168,6 +144,7 @@ int main(int argc,
 #endif
 
 	/* parse the arguments - finding a new file, treat the previous one */
+	sel = 0;
 	while (--argc > 0) {
 		argv++;
 		p = *argv;
@@ -176,25 +153,55 @@ int main(int argc,
 		if (*p == '+') {	/* switch off flags with '+' */
 			while (*++p != '\0') {
 				switch (*p) {
-				case 'B': bars_per_line = 0; break;
-				case 'c': continue_lines = 0; break;
+				case 'B':
+					cfmt.barsperstaff = 0;
+					lock_fmt(&cfmt.barsperstaff);
+					break;
+				case 'c':
+					cfmt.continueall = 0;
+					lock_fmt(&cfmt.continueall);
+					break;
 				case 'E': epsf = 0; break;
 				case 'F': def_fmt_done = 1; break;
-				case 'G': graceslurs = 1; break;
+				case 'G':
+					cfmt.graceslurs = 1;
+					lock_fmt(&cfmt.graceslurs);
+					break;
 				case 'j':
-				case 'k': measurenb = -1; break;
-				case 'l': landscape = 0; break;
-				case 'M': music_only = 0; break;
+				case 'k':
+					cfmt.measurenb = -1;
+					lock_fmt(&cfmt.measurenb);
+					break;
+				case 'l':
+					cfmt.landscape = 0;
+					lock_fmt(&cfmt.landscape);
+					break;
+				case 'M':
+					cfmt.musiconly = 0;
+					lock_fmt(&cfmt.musiconly);
+					break;
 				case 'N': pagenumbers = 0; break;
-				case 'n': write_history = 0; break;
+				case 'n':
+					cfmt.writehistory = 0;
+					lock_fmt(&cfmt.writehistory);
+					break;
 				case 'O':
 					outf[0] = '\0';
 					break;
-				case 'Q': printtempo = 0; break;
-				case 'x': include_xrefs = 0; break;
+				case 'Q':
+					cfmt.printtempo = 0;
+					lock_fmt(&cfmt.printtempo);
+					break;
+				case 'x':
+					cfmt.withxrefs = 0;
+					lock_fmt(&cfmt.withxrefs);
+					break;
 				case 'W': nwhistle = 0; break;
 				case '0': splittune = 0; break;
-				case '1': one_per_page = 0; break;
+				case '1':
+					cfmt.oneperpage = 0;
+					lock_fmt(&cfmt.oneperpage);
+					break;
 				default:
 					fprintf(stderr,
 						"++++ Cannot switch off flag: +%c\n",
@@ -208,8 +215,10 @@ int main(int argc,
 
 		if (*p == '-') {	     /* interpret a flag with '-'*/
 			if (p[1] == '\0') {
-				if (in_fname != 0)
-					output_file();
+				if (in_fname != 0) {
+					output_file(sel);
+					sel = 0;
+				}
 				in_fname = "";		/* read from stdin */
 				continue;
 			}
@@ -221,35 +230,61 @@ int main(int argc,
 					return 2;
 				}
 				argv++;
-				interpret_format_line(p, *argv);
+				interpret_fmt_line(p, *argv, 1);
 				continue;
 			}
 			while (*++p != '\0') {
 				switch (c = *p) {
 
 					/* simple flags */
-				case 'c': continue_lines = 1; break;
+				case 'c':
+					cfmt.continueall = 1;
+					lock_fmt(&cfmt.continueall);
+					break;
 				case 'E': epsf = 1; break;
-				case 'f': flatbeams = 1; break;
-				case 'G': graceslurs = 0; break;
+				case 'f':
+					cfmt.flatbeams = 1;
+					lock_fmt(&cfmt.flatbeams);
+					break;
+				case 'G':
+					cfmt.graceslurs = 0;
+					lock_fmt(&cfmt.graceslurs);
+					break;
 				case 'H': 
 					if (fout == 0)
 						set_page_format();
 					print_format();
 					return 0;
 				case 'h': usage(); break;
-				case 'l': landscape = 1; break;
-				case 'M': music_only = 1; break;
-				case 'n': write_history = 1; break;
-				case 'Q': printtempo = 1; break;
+				case 'l':
+					cfmt.landscape = 1;
+					lock_fmt(&cfmt.landscape);
+					break;
+				case 'M':
+					cfmt.musiconly = 1;
+					lock_fmt(&cfmt.musiconly);
+					break;
+				case 'n':
+					cfmt.writehistory = 1;
+					lock_fmt(&cfmt.writehistory);
+					break;
+				case 'Q':
+					cfmt.printtempo = 1;
+					lock_fmt(&cfmt.printtempo);
+					break;
 				case 'u': deco_old = 1; break;
 				case 'V':
 					write_version();
 					return 0;
-				case 'x': include_xrefs = 1; break;
+				case 'x':
+					cfmt.withxrefs = 1;
+					lock_fmt(&cfmt.withxrefs);
+					break;
 				case '0': splittune = 1; break;
-				case '1': one_per_page = 1; break;
-
+				case '1':
+					cfmt.oneperpage = 1;
+					lock_fmt(&cfmt.oneperpage);
+					break;
 				case 'e':	/* filtering */
 					if (sel != 0) {
 						fprintf(stderr,
@@ -327,27 +362,34 @@ int main(int argc,
 					}
 
 					switch (c) {
-					case 'a':
+					case 'a': {
+						float alfa_c;
+
 						sscanf(aaa, "%f", &alfa_c);
-						if (alfa_c > 1. || alfa_c < 0) {
+						if (alfa_c > 1 || alfa_c < 0) {
 							fprintf(stderr,
 								"++++ Bad parameter for flag -a: %s\n",
 								aaa);
 							return 2;
 						}
+						cfmt.maxshrink = alfa_c;
+						lock_fmt(&cfmt.maxshrink);
 						break;
+					    }
 					case 'B':
-						sscanf(aaa, "%d", &bars_per_line);
-						continue_lines = 0;
+						sscanf(aaa, "%d", &cfmt.barsperstaff);
+						lock_fmt(&cfmt.barsperstaff);
 						break;
 					case 'b':
-						sscanf(aaa, "%d", &measurefirst);
+						sscanf(aaa, "%d", &cfmt.measurefirst);
+						lock_fmt(&cfmt.measurefirst);
 						break;
 					case 'D':
 						styd = aaa;
 						break;
 					case 'd':
-						staffsep = scan_u(aaa);
+						cfmt.staffsep = scan_u(aaa);
+						lock_fmt(&cfmt.staffsep);
 						break;
 					case 'F':
 						read_def_format();
@@ -359,26 +401,31 @@ int main(int argc,
 						}
 						break;
 					case 'I':
-						indent = scan_u(aaa);
+						cfmt.indent = scan_u(aaa);
+						lock_fmt(&cfmt.indent);
 						break;
 					case 'j':
 					case 'k':
-						sscanf(aaa, "%d", &measurenb);
+						sscanf(aaa, "%d", &cfmt.measurenb);
+						lock_fmt(&cfmt.measurenb);
 						if (aaa[strlen(aaa) - 1] == 'b')
-							measurebox = 1;
-						else	measurebox = 0;
+							cfmt.measurebox = 1;
+						else	cfmt.measurebox = 0;
+						lock_fmt(&cfmt.measurebox);
 						break;
 					case 'L':
-						sscanf(aaa, "%d", &encoding);
-						if ((unsigned) encoding > MAXENC) {
+						sscanf(aaa, "%d", &cfmt.encoding);
+						if ((unsigned) cfmt.encoding > MAXENC) {
 							fprintf(stderr,
 								"++++ Bad encoding value %s - reset to 0\n",
 								aaa);
-							encoding = 0;
+							cfmt.encoding = 0;
 						}
+						lock_fmt(&cfmt.encoding);
 						break;
 					case 'm':
-						lmargin = scan_u(aaa);
+						cfmt.leftmargin = scan_u(aaa);
+						lock_fmt(&cfmt.leftmargin);
 						break;
 					case 'N':
 						sscanf(aaa, "%d", &pagenumbers);
@@ -393,11 +440,19 @@ int main(int argc,
 						strcpy(outf, aaa);
 						break;
 					case 's':
-						sscanf(aaa, "%f", &scalefac);
+						sscanf(aaa, "%f", &cfmt.scale);
+						lock_fmt(&cfmt.scale);
 						break;
-					case 'w':
-						swidth = scan_u(aaa);
+					case 'w': {
+						float w;
+
+						w = scan_u(aaa);
+						cfmt.rightmargin = cfmt.pagewidth - w - cfmt.leftmargin;
+						if (cfmt.rightmargin < 0)
+							fprintf(stderr, "Warning: staffwidth too big\n");
+						lock_fmt(&cfmt.rightmargin);
 						break;
+					    }
 					case 'W':
 						whistle_parse(aaa);
 						break;
@@ -424,26 +479,25 @@ int main(int argc,
 			continue;
 		}
 
-		if (in_fname != 0)
-			output_file();
+		if (in_fname != 0) {
+			output_file(sel);
+			sel = 0;
+		}
 		in_fname = p;
 	}
 
 	if (in_fname != 0)
-		output_file();
-
+		output_file(sel);
 	if (!epsf && fout == 0) {
 		fprintf(stderr, "No input file specified\n");
 		return 1;
 	}
-
 	close_output_file();
-
 	return severity;
 }
 
 /* -- generate the current ABC file -- */
-static void output_file(void)
+static void output_file(char *sel)
 {
 	struct abctune *t;
 	char *file;
@@ -462,7 +516,6 @@ static void output_file(void)
 	/* initialize if not already done */
 	if (fout == 0)
 		set_page_format();
-
 	memset(&default_info, 0, sizeof default_info);
 	default_info.title = &notitle;
 	notitle.as.text = "T:(notitle)";
@@ -477,10 +530,9 @@ static void output_file(void)
 	t = abc_parse(file);
 	free(file);
 
-	if (sel) {
+	if (sel)
 		do_filter(t, sel);
-		sel = 0;
-	} else	do_select(t,
+	else	do_select(t,
 			  -32000,
 			  (int) ((unsigned) (~0) >> 1));
 	/*abc_free(t);	(useless) */
@@ -489,17 +541,17 @@ static void output_file(void)
 /* -- do filtering on an input file -- */
 static void do_filter(struct abctune *t, char *sel)
 {
-	int cur_sel;
-	int end_sel;
-	int n;
+	int cur_sel, end_sel, n;
 
 	for (;;) {
-		sscanf(sel, "%d%n", &cur_sel, &n);
+		if (sscanf(sel, "%d%n", &cur_sel, &n) != 1)
+			break;
 		sel += n;
 		if (*sel == '-') {
 			sel++;
 			end_sel = (int) ((unsigned) (~0) >> 1);
-			sscanf(sel, "%d%n", &end_sel, &n);
+			if (sscanf(sel, "%d%n", &end_sel, &n) != 1)
+				break;
 			sel += n;
 		} else	end_sel = cur_sel;
 		do_select(t, cur_sel, end_sel);
@@ -514,98 +566,31 @@ static void do_select(struct abctune *t,
 		      int first_tune,
 		      int last_tune)
 {
-	while (t != 0) {
-		struct abcsym *s;
-		int print_tune;
+	struct abcsym *s;
+	int print_tune, i;
 
-		print_tune = 0;
+	while (t != 0) {
+		i = first_tune - 1;
 		for (s = t->first_sym; s != 0; s = s->next) {
 			if (s->type == ABC_T_INFO
 			    && s->text[0] == 'X') {
-				int i;
-
-				if (sscanf(s->text, "X:%d", &i) == 1
-				    && i >= first_tune
-				    && i <= last_tune)
-					print_tune = 1;
+				sscanf(s->text, "X:%d", &i);
 				break;
 			}
 		}
+		print_tune = i >= first_tune && i <= last_tune;
 		if (print_tune
 		    || !t->client_data) {	/* (parse the global symbols) */
 			clrarena(2);
 			do_tune(t, !print_tune);
-			t->client_data = 1;	/* treated */
+			t->client_data = (void *) 1;	/* treated */
 		}
+		if (i >= last_tune)
+			break;
 		t = t->next;
 	}
 }
 
-/* -- return the file extension -- */
-static char *getext(char *fid)
-{
-	char *p;
-
-	if ((p = strrchr(fid, DIRSEP)) == 0)
-		p = fid;
-	if ((p = strrchr(p, '.')) != 0)
-		return p + 1;
-	return 0;
-}
-
-/* -- copy the command line options to the current format -- */
-void ops_into_fmt(void)
-{
-	struct FORMAT *fmt;
-
-	fmt = &cfmt;
-	if (landscape >= 0)
-		fmt->landscape = landscape;
-	if (scalefac > 0)
-		fmt->scale = scalefac;
-	if (lmargin >= 0)
-		fmt->leftmargin = lmargin;
-	if (indent >= 0)
-		fmt->indent = indent;
-	if (swidth > 0) {
-		fmt->rightmargin = fmt->pagewidth - swidth - fmt->leftmargin;
-		if (fmt->rightmargin < 0)
-			fprintf(stderr, "Warning: staffwidth too big\n");
-	}
-	if (continue_lines >= 0)
-		fmt->continueall = continue_lines;
-	if (write_history >= 0)
-		fmt->writehistory = write_history;
-	if (bars_per_line >= 0)
-		fmt->barsperstaff = bars_per_line;
-	if (encoding >= 0)
-		fmt->encoding = encoding;
-	if (include_xrefs >= 0)
-		fmt->withxrefs = include_xrefs;
-	if (staffsep > 0)
-		fmt->staffsep = staffsep;
-	if (one_per_page >= 0)
-		fmt->oneperpage = one_per_page;
-	if (splittune >= 0)
-		fmt->splittune = splittune;
-	if (music_only >= 0)
-		fmt->musiconly = music_only;
-	if (graceslurs >= 0)
-		fmt->graceslurs = graceslurs;
-	if (flatbeams >= 0)
-		fmt->flatbeams = flatbeams;
-	if (measurenb >= 0)
-		fmt->measurenb = measurenb;
-	if (measurebox >= 0)
-		fmt->measurebox = measurebox;
-	if (measurefirst >= 0)
-		fmt->measurefirst = measurefirst;
-	if (printtempo >= 0)
-		fmt->printtempo = printtempo;
-	if (alfa_c >= 0)
-		fmt->maxshrink = alfa_c;
-}
-
 /* -- read the default format -- */
 static void read_def_format(void)
 {
@@ -621,6 +606,7 @@ static char *read_file(void)
 	int fsize;
 	FILE *fin;
 	char *file;
+static char new_file[256];
 
 	if (in_fname[0] == '\0') {
 		int offset;
@@ -645,13 +631,12 @@ static char *read_file(void)
 		struct stat sbuf;
 
 		if ((fin = fopen(in_fname, "rb")) == 0) {
-			char new_file[256];
-
-			if (getext(in_fname) != 0)
+			if (strlen(in_fname) >= sizeof new_file - 4)
 				return 0;
 			sprintf(new_file, "%s.abc", in_fname);
 			if ((fin = fopen(new_file, "rb")) == 0)
 				return 0;
+			in_fname = new_file;
 		}
 		if (fseek(fin, 0L, SEEK_END) < 0) {
 			fclose(fin);
@@ -681,7 +666,6 @@ static char *read_file(void)
 static void set_page_format(void)
 {
 	read_def_format();
-	ops_into_fmt();
 	make_font_list();
 }
 
