@@ -30,13 +30,13 @@
 #define STEM_XOFF	3.5
 #define STEM		20	/* default stem height */
 #define STEM_MIN	16	/* min stem height under beams */
-#define STEM_MIN2	12	/* ... for notes with two beams */
-#define STEM_MIN3	10	/* ... for notes with three beams */
+#define STEM_MIN2	14	/* ... for notes with two beams */
+#define STEM_MIN3	12	/* ... for notes with three beams */
 #define STEM_MIN4	10	/* ... for notes with four beams */
-#define STEM_CH_MIN	12	/* min stem height for chords under beams */
-#define STEM_CH_MIN2	 8	/* ... for notes with two beams */
-#define STEM_CH_MIN3	 7	/* ... for notes with three beams */
-#define STEM_CH_MIN4	 7	/* ... for notes with four beams */
+#define STEM_CH_MIN	14	/* min stem height for chords under beams */
+#define STEM_CH_MIN2	10	/* ... for notes with two beams */
+#define STEM_CH_MIN3	 9	/* ... for notes with three beams */
+#define STEM_CH_MIN4	 8	/* ... for notes with four beams */
 #define BEAM_DEPTH	3.2	/* width of a beam stroke (was 2.6) */
 #define BEAM_OFFSET	0.25	/* pos of flat beam relative to staff line */
 #define BEAM_SHIFT	5.0	/* shift of second and third beams (was 5.3) */
@@ -147,14 +147,15 @@ struct SYMBOL { 		/* struct for a drawable symbol */
 	signed char pits[MAXHD]; /* pitches for notes */
 	struct SYMBOL *ts_next, *ts_prev; /* time linkage */
 	int time;		/* starting time */
-	short sflags;		/* symbol flags */
+	unsigned short sflags;	/* symbol flags */
 #define S_EOLN		0x0001		/* end of line */
 #define S_WORD_ST	0x0002		/* word starts here */
-#define S_BEAM_BREAK	0x0004		/* 2nd beam must restart here */
+#define S_BEAM_BR1	0x0004		/* 2nd beam must restart here */
 #define S_NO_HEAD	0x0008		/* don't draw the highest/lowest note head */
 #define S_OTHER_HEAD	0x0010		/* don't draw any note head */
 #define S_IN_TUPLET	0x0020		/* in a tuplet */
-#define S_RRBAR		0x0040		/* right repeat bar */
+#define S_RRBAR		0x0040		/* right repeat bar (when bar) */
+#define S_XSTEM		0x0040		/* cross-staff stem (when note) */
 #define S_BEAM_ON	0x0080		/* continue beaming */
 #define S_SL1		0x0100		/* some chord slur start */
 #define S_SL2		0x0200		/* some chord slur end */
@@ -162,7 +163,8 @@ struct SYMBOL { 		/* struct for a drawable symbol */
 #define S_DYNUP		0x0800		/* dynamic marks above the staff */
 #define S_DYNDN		0x1000		/* dynamic marks below the staff */
 #define S_TREM		0x2000		/* tremolo */
-#define S_XSTEM		0x4000		/* cross-staff stem */
+#define S_RBSTOP	0x4000		/* repeat bracket stop */
+#define S_BEAM_BR2	0x8000		/* 3rd beam must restart here */
 	unsigned char nhd;	/* number of notes in chord - 1 */
 	signed char stem;	/* 1 / -1 for stem up / down */
 	signed char nflags;	/* number of note flags when > 0 */
@@ -214,6 +216,8 @@ struct SYMBOL { 		/* struct for a drawable symbol */
 #define B_DREP 0x44		/* ::	double repeat bar */
 #define B_DASH 0x04		/* :	dashed bar */
 
+extern unsigned short *micro_tb; /* ptr to the microtone table of the tune */
+
 struct FORMAT { 		/* struct for page layout */
 	float pageheight, pagewidth;
 	float topmargin, botmargin, leftmargin, rightmargin;
@@ -222,17 +226,17 @@ struct FORMAT { 		/* struct for page layout */
 	float scale, maxshrink, lineskipfac, parskipfac, sysstaffsep;
 	float indent, infospace, slurheight, notespacingfactor;
 	float maxstaffsep, maxsysstaffsep, stemheight;
-	int aligncomposer, autoclef, barsperstaff, breathlow, bstemdown;
-	int combinevoices, contbarnb, continueall, dynalign;
+	int alignbars, aligncomposer, autoclef, barsperstaff, breathlow;
+	int bstemdown, combinevoices, contbarnb, continueall, dynalign;
 	int encoding, exprabove, exprbelow, flatbeams, freegchord;
-	int infoline, gchordbox, graceslurs, halfbeam, hyphencont;
+	int infoline, gchordbox, graceslurs, comball, hyphencont;
 	int landscape, measurenb, measurefirst, measurebox, musiconly;
 	int oneperpage, partsbox, printparts, printtempo, pslevel;
 	int setdefl, shifthnote, splittune, squarebreve;
 	int straightflags, stretchstaff, stretchlast;
-	int textoption, titlecaps, titleleft, tuplets;
+	int textoption, titlecaps, titleleft, timewarn, tuplets;
 	int vocalabove, withxrefs, writehistory;
-	char *dateformat, *header, *footer;
+	char *dateformat, *header, *footer, *titleformat;
 	struct FONTSPEC annotationfont, composerfont, footerfont;
 	struct FONTSPEC gchordfont, headerfont, historyfont, infofont;
 	struct FONTSPEC measurefont, partsfont, repeatfont, subtitlefont;
@@ -246,7 +250,8 @@ struct FORMAT { 		/* struct for page layout */
 				* guitar chords, annotations and lyrics */
 };
 
-extern struct FORMAT cfmt;	/* current format for output */
+extern struct FORMAT cfmt;	/* current local format for output */
+extern struct FORMAT dfmt;	/* current global format for output */
 
 struct ISTRUCT {		/* information fields */
 	struct SYMBOL *area;
@@ -385,7 +390,6 @@ extern float space_tb[NFLAGS_SZ], dot_space; /* note spacing */
 void clrarena(int level);
 void lvlarena(int level);
 char *getarena(int len);
-void ops_into_fmt(void);
 void strext(char *fid, char *ext);
 /* buffer.c */
 void a2b(void);
@@ -439,7 +443,8 @@ void putx(float x);
 void puty(float y);
 void putxy(float x, float y);
 /* format.c */
-int interpret_format_line(char *w, char *p);
+void interpret_fmt_line(char *w, char *p, int lock);
+void lock_fmt(void *fmt);
 void define_fonts(void);
 int get_textopt(char *p);
 void make_font_list(void);
