@@ -1,10 +1,9 @@
 /*++
  * Generic ABC parser.
  *
- * Copyright (C) 1998-2005 Jean-Franois Moine
+ * Copyright (C) 1998-2006 Jean-Franois Moine
  * Adapted from abc2ps, Copyright (C) 1996, 1997  Michael Methfessel
  *
- * Contact: mailto:moinejf@free.fr
  * Original site: http://moinejf.free.fr/
  *      
  * This program is free software; you can redistribute it and/or modify
@@ -40,8 +39,6 @@ static int keep_comment;
 
 /* global values */
 char *deco_tb[128];		/* decoration names */
-#define MAXMICRO 32
-unsigned short micro_tb[MAXMICRO]; /* microtone values [ (n-1) | (d-1) ] */
 int severity;			/* error severity */
 
 static short abc_state;		/* parse state */
@@ -54,6 +51,7 @@ static int lyric_started;	/* lyric started */
 static struct abcsym *deco_start; /* 1st note of the line for d: / s: */
 static struct abcsym *deco_cont; /* current symbol when d: / s: continuation */
 static int vover;		/* voice overlay (1: single bar, -1: multi-bar */
+static unsigned short *p_micro;	/* ptr to the microtone table of the tune */
 
 #define VOICE_NAME_SZ 64	/* max size of a voice name */
 
@@ -362,6 +360,7 @@ struct abctune *abc_parse(char *file_api)
 			}
 			last_tune = t;
 			ulen = gulen;
+			p_micro = t->micro_tb;
 		}
 
 		/* parse the music line */
@@ -858,6 +857,17 @@ static void parse_key(char *p,
 				s->u.key.minor = 1;
 			} else	goto unk;
 			break;
+		case 'n':
+		case 'N':
+			if (strncasecmp(p, "none", 4) == 0) {
+				p += 4;
+				while (!isspace((unsigned char) *p)
+				       && *p != '\0')
+					p++;
+				s->u.key.nacc = -1;	/* explicit no accidental */
+				continue;
+			}
+			goto unk;
 		case 'o':
 		case 'O':
 			if (strncasecmp(p, "octave", 6) == 0) {	/* (abcMIDI) */
@@ -1041,6 +1051,8 @@ static char *parse_meter(char *p,
 			 struct abcsym *s)
 {
 	int m1, m2, d, wmeasure, nm, i, in_parenth;
+	char *q;
+static char *top_err = "Cannot identify meter top";
 
 	if (*p == '\0')
 		return "Empty meter string";
@@ -1056,27 +1068,37 @@ static char *parse_meter(char *p,
 		wmeasure = 4 * BASE_LEN / 4;
 		nm = 1;
 	} else while (*p != '\0') {
+		if (*p == '=')
+			break;
+		if (nm >= MAX_MEASURE)
+			return "Too many values in M:";
+		if (*p == '(') {
+			q = p + 1;
+			while (*q != '\0') {
+				if (*q == ')' || *q == '/')
+					break;
+				q++;
+			}
+			if (*q == ')')
+				p++;	/* stay on top */
+		}
 		if (*p == '(' || *p == ')') {
-			if (*p == '(')
-				in_parenth = 1;
-			else 	in_parenth = 0;
+			in_parenth = *p == '(';
 			s->u.meter.meter[nm].top[0] = *p++;
 			nm++;
 			continue;
 		}
-		if (*p == '=')
-			break;
 		if (sscanf(p, "%d", &m1) != 1
 		    || m1 <= 0)
-			return "Cannot identify meter top";
-		if (nm >= MAX_MEASURE)
-			return "Too many values in M:";
+			return top_err;
 		i = 0;
 		m2 = 2;			/* default when no bottom value */
 		for (;;) {
 			while (isdigit((unsigned) *p)
 			       && i < sizeof s->u.meter.meter[0].top)
 				s->u.meter.meter[nm].top[i++] = *p++;
+			if (*p == ')')
+				p++;
 			if (*p == '/') {
 				p++;
 				if (sscanf(p, "%d", &m2) != 1
@@ -1093,7 +1115,7 @@ static char *parse_meter(char *p,
 			s->u.meter.meter[nm].top[i++] = *p++;
 			if (sscanf(p, "%d", &d) != 1
 			    || d <= 0)
-				return "Cannot identify meter top";
+				return top_err;
 			if (p[-1] == ' ') {
 				if (d > m1)
 					m1 = d;
@@ -1102,7 +1124,9 @@ static char *parse_meter(char *p,
 		if (!in_parenth)
 			wmeasure += m1 * BASE_LEN / m2;
 		nm++;
-		if (*p == ' ' || *p == '+') {
+		if (*p == ' ')
+			p++;
+		else if (*p == '+') {
 			s->u.meter.meter[nm].top[0] = *p++;
 			nm++;
 		}
@@ -1134,56 +1158,48 @@ static char *parse_meter(char *p,
 static void parse_staves(char *p,
 			 struct abcsym *s)
 {
-	int voice;
+	int voice, flags_st;
 	char flags, flags2;
-	struct staff_s *staff;
+	struct staff_s *p_staff;
 
 	/* define the voices */
-	flags = 0;
-	staff = 0;
+	flags = flags2 = 0;
+	flags_st = 0;
 	voice = 0;
+
 	while (*p != '\0') {
 		switch (*p) {
 		case ' ':
 		case '\t':
 			break;
 		case '[':
-			if (flags & (OPEN_BRACKET | OPEN_BRACE | OPEN_PARENTH))
+#if 1
+/*fixme: should limit at 2 open bracket*/
+			if (flags2 & OPEN_PARENTH)
+#else
+			if (flags2 & (OPEN_BRACKET | OPEN_PARENTH))
+#endif
 				goto err;
+			flags2 |= OPEN_BRACKET;
 			flags |= OPEN_BRACKET;
-			staff = 0;
-			break;
-		case ']':
-			if (staff == 0)
-				goto err;
-			staff->flags |= CLOSE_BRACKET;
+			flags_st <<= 8;
+			flags_st |= OPEN_BRACKET;
 			break;
 		case '{':
-			if (flags & (OPEN_BRACE | OPEN_PARENTH))
+			if (flags2 & (OPEN_BRACE | OPEN_PARENTH))
 				goto err;
+			flags2 |= OPEN_BRACE;
 			flags |= OPEN_BRACE;
-			staff = 0;
-			break;
-		case '}':
-			if (staff == 0)
-				goto err;
-			staff->flags |= CLOSE_BRACE;
+			flags_st <<= 8;
+			flags_st |= OPEN_BRACE;
 			break;
 		case '(':
-			if (flags & OPEN_PARENTH)
+			if (flags2 & OPEN_PARENTH)
 				goto err;
+			flags2 |= OPEN_PARENTH;
 			flags |= OPEN_PARENTH;
-			staff = 0;
-			break;
-		case ')':
-			if (staff == 0)
-				goto err;
-			staff->flags |= CLOSE_PARENTH;
-			break;
-		case '|':
-			if (staff == 0)
-				goto err;
-			staff->flags |= STOP_BAR;
+			flags_st <<= 8;
+			flags_st |= OPEN_PARENTH;
 			break;
 		default:
 			if (!isalnum((unsigned) *p) && *p != '_')
@@ -1196,57 +1212,60 @@ static void parse_staves(char *p,
 				int v;
 
 				p = def_voice(p, &v);
-				staff = &s->u.staves[voice++];
-				staff->voice = v;
-				staff->name = alloc_f(strlen(voice_tb[v].name) + 1);
-				strcpy(staff->name, voice_tb[v].name);
+				p_staff = &s->u.staves[voice++];
+				p_staff->voice = v;
+				p_staff->name = alloc_f(strlen(voice_tb[v].name) + 1);
+				strcpy(p_staff->name, voice_tb[v].name);
 			}
-			staff->flags = flags;
+			for (;;) {
+				switch (*p) {
+				case ' ':
+				case '\t':
+					p++;
+					continue;
+				case ']':
+					if (!(flags_st & OPEN_BRACKET))
+						goto err;
+					flags_st >>= 8;
+					flags2 &= ~OPEN_BRACKET;
+					flags |= CLOSE_BRACKET;
+					p++;
+					continue;
+				case '}':
+					if (!(flags_st & OPEN_BRACE))
+						goto err;
+					flags_st >>= 8;
+					flags2 &= ~OPEN_BRACE;
+					flags |= CLOSE_BRACE;
+					p++;
+					continue;
+				case ')':
+					if (!(flags_st & OPEN_PARENTH))
+						goto err;
+					flags_st >>= 8;
+					flags2 &= ~OPEN_PARENTH;
+					flags |= CLOSE_PARENTH;
+					p++;
+					continue;
+				case '|':
+					flags |= STOP_BAR;
+					p++;
+					continue;
+				}
+				break;
+			}
+			p_staff->flags = flags;
 			flags = 0;
+			if (*p == '\0')
+				break;
 			continue;
 		}
+		if (*p == '\0')
+			break;
 		p++;
 	}
-
-	/* check for errors */
-	if (flags != 0)
-		goto err;
-
-	flags = CLOSE_BRACKET | CLOSE_BRACE | CLOSE_PARENTH;	/* bad flags */
-	flags2 = flags;
-	for (voice = 0, staff = s->u.staves;
-	     voice <= MAXVOICE && staff->name;
-	     voice++, staff++) {
-		if ((staff->flags & (OPEN_PARENTH | CLOSE_PARENTH))
-				== (OPEN_PARENTH | CLOSE_PARENTH))
-			;			/* voice alone in a staff */
-		else {
-			if (staff->flags & flags)
-				goto err;
-			if (staff->flags & CLOSE_PARENTH)
-				flags = flags2;
-		}
-		if (staff->flags & OPEN_BRACKET) {
-			flags &= ~CLOSE_BRACKET;
-			flags |= OPEN_BRACKET;
-		} else if (staff->flags & CLOSE_BRACKET) {
-			flags &= ~OPEN_BRACKET;
-			flags |= CLOSE_BRACKET;
-		} else if (staff->flags & OPEN_BRACE) {
-			flags &= ~CLOSE_BRACE;
-			flags |= OPEN_BRACKET | OPEN_BRACE;
-		} else if (staff->flags & CLOSE_BRACE) {
-			flags &= ~(OPEN_BRACKET | OPEN_BRACE);
-			flags |= CLOSE_BRACE;
-		}
-		if (staff->flags & OPEN_PARENTH
-		    && !(staff->flags & CLOSE_PARENTH)) {
-			flags2 = flags;
-			flags &= ~CLOSE_PARENTH;
-			flags |= OPEN_PARENTH;
-		}
-	}
-	return;
+	if (flags2 == 0)
+		return;
 
 err:
 	syntax("%%staves error", p);
@@ -1637,7 +1656,6 @@ static char *parse_basic_note(char *p,
 			      int *stemless)
 {
 	int pit, len, acc, nostem;
-	char *q;
 
 	acc = pit = nostem = 0;
 
@@ -1667,6 +1685,7 @@ static char *parse_basic_note(char *p,
 	if (acc != 0
 	    && (isdigit((unsigned) *p) || *p == '/')) {
 		int n, d;
+		char *q;
 
 		n = d = 1;
 		if (*p != '/') {
@@ -1685,10 +1704,10 @@ static char *parse_basic_note(char *p,
 		d--;
 		d += (n - 1) << 8;	/* short [ (n-1) | (d-1) ] */
 		for (n = 1; n < MAXMICRO; n++) {
-			if (micro_tb[n] == d)
+			if (p_micro[n] == d)
 				break;
-			if (micro_tb[n] == 0) {
-				micro_tb[n] = d;
+			if (p_micro[n] == 0) {
+				p_micro[n] = d;
 				break;
 			}
 		}
@@ -1843,9 +1862,11 @@ static char *parse_decoline(char *p)
 			if (is->text != 0) {
 				char *gch;
 
-				gch = alloc_f(strlen(is->text) + strlen(gchord) + 1);
+				n = strlen(is->text);
+				gch = alloc_f(n + strlen(gchord) + 2);
 				strcpy(gch, is->text);
-				strcpy(gch + strlen(is->text), gchord);
+				gch[n] = '\n';
+				strcpy(gch + n + 1, gchord);
 				if (free_f) {
 					free_f(gchord);
 					free_f(is->text);
