@@ -3,7 +3,7 @@
  *
  * This file is part of abcm2ps.
  *
- * Copyright (C) 1998-2005 Jean-Franois Moine
+ * Copyright (C) 1998-2006 Jean-Franois Moine
  * Adapted from abc2ps, Copyright (C) 1996,1997 Michael Methfessel
  *
  * This program is free software; you can redistribute it and/or modify
@@ -45,6 +45,7 @@ static float bposy;		/* current position in buffered data */
 static int nepsf;		/* counter for epsf output files */
 static int nbpages;		/* number of pages in the output file */
 static char outfnam[STRL1];	/* internal file name for open/close */
+static struct FORMAT *p_fmt;	/* current format while treating a new page */
 
 char *mbf;			/* where to PUTx() */
 int nbuf;			/* number of bytes buffered */
@@ -55,7 +56,7 @@ static void cnv_date(time_t *ltime)
 {
 	char buf[TEX_BUF_SZ];
 
-	tex_str(cfmt.dateformat);
+	tex_str(p_fmt->dateformat);
 	strcpy(buf, tex_buf);
 	strftime(tex_buf, TEX_BUF_SZ, buf, localtime(ltime));
 }
@@ -70,12 +71,12 @@ static void init_ps(char *str, int is_epsf)
 /*fixme: no landscape for EPS?*/
 		fprintf(fout, "%%!PS-Adobe-3.0 EPSF-3.0\n"
 			"%%%%BoundingBox: 0 0 %.0f %.0f\n",
-			cfmt.pagewidth - cfmt.leftmargin
-				- cfmt.rightmargin + 20,
+			p_fmt->pagewidth - p_fmt->leftmargin
+				- p_fmt->rightmargin + 20,
 			-bposy);
-		cur_lmarg = cfmt.leftmargin - 10;
+		cur_lmarg = p_fmt->leftmargin - 10;
 	} else	fprintf(fout, "%%!PS-Adobe-%d.0\n",
-			cfmt.pslevel);
+			p_fmt->pslevel);
 
 	fprintf(fout, "%%%%Title: %s\n", str);
 
@@ -89,7 +90,7 @@ static void init_ps(char *str, int is_epsf)
 	fprintf(fout, "%%%%LanguageLevel: %d\n"
 		"%%%%EndComments\n"
 		"%%CommandLine:",
-		cfmt.pslevel);
+		p_fmt->pslevel);
 	for (i = 1; i < s_argc; i++) {
 		fprintf(fout,
 			strchr(s_argv[i], ' ') != 0 ? " \'%s\'" : " %s",
@@ -113,7 +114,7 @@ static void init_ps(char *str, int is_epsf)
 		"/SLW/setlinewidth load def\n"
 		"/defl 0 def\n"	/* decoration flags - see deco.c for values */
 		"/dlw{0.7 SLW}!\n");
-	if (cfmt.pslevel < 2)
+	if (p_fmt->pslevel < 2)
 		/* (simple!) level2 emulation */
 	    fprintf(fout, 
 		"/cshow{exch dup{fh fh 4 index exec}forall pop pop}!\n"
@@ -147,6 +148,10 @@ void close_output_file(void)
 
 	if (fout == 0)
 		return;
+	if (multicol_start != 0) {	/* if no '%%multocol end' or filtering */
+		multicol_start = 0;
+		write_buffer();
+	}
 	if (tunenum == 0)
 		error(0, 0, "No tunes written to output file");
 	close_page();
@@ -282,7 +287,7 @@ static float headfooter(int header,
 		p = cfmt.footer;
 		f = &cfmt.footerfont;
 		size = f->size;
-		y = - (pheight - cfmt.topmargin - cfmt.botmargin)
+		y = - (pheight - p_fmt->topmargin - p_fmt->botmargin)
 			- size + 2.;
 	}
 	wsize = 0;
@@ -308,7 +313,7 @@ static float headfooter(int header,
 			if (q != p) {
 				*q = '\0';
 				fprintf(fout, "%.1f %.1f M(",
-					cfmt.leftmargin, y);
+					p_fmt->leftmargin, y);
 				format_hf(p);
 				fprintf(fout, ")show\n");
 			}
@@ -330,7 +335,7 @@ static float headfooter(int header,
 			p = q + 1;
 			if (*p != '\0') {
 				fprintf(fout, "%.1f %.1f M(",
-					pwidth - cfmt.rightmargin, y);
+					pwidth - p_fmt->rightmargin, y);
 				format_hf(p);
 				fprintf(fout, ")showr\n");
 			}
@@ -357,6 +362,8 @@ static void init_page(void)
 	if (in_page)
 		return;
 
+	p_fmt = info.xref == 0 ? &cfmt : &dfmt;	/* global format */
+
 	if (!file_initialized)
 		init_ps(in_fname, 0);
 	in_page = 1;
@@ -364,20 +371,20 @@ static void init_page(void)
 
 	fprintf(fout, "%%%%Page: %d %d\n",
 		nbpages, nbpages);
-	if (cfmt.landscape) {
-		pheight = cfmt.pagewidth;
-		pwidth = cfmt.pageheight;
+	if (p_fmt->landscape) {
+		pheight = p_fmt->pagewidth;
+		pwidth = p_fmt->pageheight;
 		fprintf(fout, "%%%%PageOrientation: Landscape\n"
 			"gsave 90 rotate 0 %.1f T\n",
-			-cfmt.topmargin);
+			-p_fmt->topmargin);
 	} else {
-		pheight = cfmt.pageheight;
-		pwidth = cfmt.pagewidth;
+		pheight = p_fmt->pageheight;
+		pwidth = p_fmt->pagewidth;
 		fprintf(fout, "gsave 0 %.1f T\n",
-			pheight - cfmt.topmargin);
+			pheight - p_fmt->topmargin);
 	}
 
-	posy = pheight - cfmt.topmargin - cfmt.botmargin;
+	posy = pheight - p_fmt->topmargin - p_fmt->botmargin;
 
 	/* output the header and footer */
 	if (cfmt.header == 0) {
@@ -482,6 +489,7 @@ void write_eps(void)
 	long m;
 	char *p, fnm[STRL1], finf[STRL1];
 
+	p_fmt = info.xref == 0 ? &cfmt : &dfmt;	/* global format */
 	close_output_file();
 	strcpy(fnm, outf);
 	if (fnm[0] == '\0')
@@ -596,7 +604,34 @@ void write_buffer(void)
 			fprintf(fout, "0 %.1f T\n", -cfmt.topspace);
 			posy -= cfmt.topspace * cfmt.scale;
 		}
-		fwrite(&buf[i], 1, b2 - i, fout);
+		if (buf[i] != '\001')
+			fwrite(&buf[i], 1, b2 - i, fout);
+		else {				/* %%EPS - see parse.c */
+			FILE *f;
+			char line[BSIZE], *p;
+			float x1, y1;
+
+			i++;
+			p = strchr(&buf[i], '\n');
+			fwrite(&buf[i], 1, p - &buf[i] + 1, fout);
+			sscanf(p + 1, "%f %f %s\n", &x1, &y1, line);
+			if ((f = fopen(line, "r")) == 0) {
+				error(1, 0, "Cannot open EPS file '%s'", line);
+			} else {
+				fprintf(fout,
+					"%% ----- EPS file '%s' -----\n"
+					"save\n"
+					"/showpage{}def/setpagedevice{pop}def\n"
+					"%.2f %.2f T\n", 
+					line, -x1, -y1);
+				while (fgets(line, sizeof line, f))	/* copy the file */
+					fwrite(line, 1, strlen(line), fout);
+				fclose(f);
+				strcpy(line, "restore\n"
+						"% ----- end EPS -----\n");
+				fwrite(line, 1, strlen(line), fout);
+			}
+		}
 		i = b2;
 		posy += dp;
 		p1 = ln_pos[l];
@@ -612,6 +647,8 @@ void buffer_eob(void)
 {
 	if (bposy == 0 && nbuf == 0)
 		return;
+	if (ln_num > 0 && ln_buf[ln_num - 1] == nbuf)
+		return;
 	if (ln_num >= BUFFLN) {
 		error(1, 0, "max number of buffer lines exceeded"
 			" -- check BUFFLN");
