@@ -3,7 +3,7 @@
  *
  * This file is part of abcm2ps.
  *
- * Copyright (C) 2000-2005, Jean-Franois Moine.
+ * Copyright (C) 2000-2006, Jean-Franois Moine.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -54,8 +54,7 @@ static struct deco_elt {
 } *deco_head, *deco_tail;
 
 typedef void draw_f(struct deco_elt *de);
-static draw_f d_arp, d_cresc, d_near, d_slide, d_upstaff,
-	d_pf, d_trill;
+static draw_f d_arp, d_cresc, d_near, d_slide, d_upstaff, d_pf, d_trill;
 
 /* decoration table */
 /* !! don't change the order of the numbered items !! */
@@ -70,77 +69,26 @@ static struct deco_def_s {
 } deco_def_tb[128] = {
 	{0,		0, 0, 0},	/* 0: unknown */
 	{"dot",		0, 0, 2},	/* 1 */
-	{"roll",	3, 10, 10},	/* 2 */
-	{"fermata",	3, 6, 12},	/* 3 */
-	{"emphasis",	3, 21, 8},	/* 4 */
-	{"lowermordent", 3, 5, 10},	/* 5 */
-	{"coda",	3, 16, 24},	/* 6 */
-	{"uppermordent", 3, 4, 10},	/* 7 */
-	{"segno",	3, 17, 20},	/* 8 */
-	{"trill",	3, 7, 11},	/* 9 */
-	{"upbow",	3, 8, 10},	/* 10 */
-	{"downbow",	3, 9, 9},	/* 11 */
-	{"gmark",	3, 3, 6},	/* 12 */
-	{"slide",	1, 2, 3, 7},	/* 13 */
-	{"tenuto",	0, 1, 2},	/* 14 */
-	{"breath",	3, 18, 0},	/* 15 */
-	{"longphrase",	3, 24, 0},	/* 16 */
-	{"mediumphrase", 3, 25, 0},	/* 17 */
-	{"shortphrase", 3, 26, 0},	/* 18 */
-	{"invertedfermata", 3, 6, 12},	/* 19 */
-	{"invertedturn", 3, 22, 10},	/* 20 */
-	{"invertedturnx", 3, 23, 10},	/* 21 */
-	{"0",		3, 11, 8, 0, 0, 255},
-	{"1",		3, 11, 8, 0, 0, 255},
-	{"2",		3, 11, 8, 0, 0, 255},
-	{"3",		3, 11, 8, 0, 0, 255},
-	{"4",		3, 11, 8, 0, 0, 255},
-	{"5",		3, 11, 8, 0, 0, 255},
-	{"+",		3, 20, 7},
-	{"plus",	3, 20, 7},
-	{"accent",	3, 21, 8},
-	{"D.C.",	3, 12, 16, 0, 0, 255},
-	{"D.S.",	3, 12, 16, 0, 0, 255},
-	{"emphasis",	3, 21, 8},
-	{"fine",	3, 12, 16, 0, 0, 1},
-	{"f",		6, 13, 18, 4, 4},
-	{"ff",		6, 13, 18, 5, 11},
-	{"fff",		6, 13, 18, 7, 17},
-	{"ffff",	6, 13, 18, 8, 24},
-	{"mf",		6, 13, 18, 5, 11},
-	{"mp",		6, 13, 18, 5, 11},
-	{"mordent",	3,  5, 10},
-	{"open",	3, 30, 10},
-	{"p",		6, 13, 18, 4, 4},
-	{"pp",		6, 13, 18, 5, 11},
-	{"ppp",		6, 13, 18, 7, 17},
-	{"pppp",	6, 13, 18, 8, 24},
-	{"pralltriller", 3, 4, 10},
-	{"sfz",		6, 14, 18, 7, 17},
-	{"turn",	3, 22, 10},
-	{"wedge",	3, 29, 8},
-	{"turnx",	3, 23, 10},
-	{"trill(",	5, -1, 8},
-	{"trill)",	5, 28, 8},
-	{"snap",	3, 31, 14},
-	{"thumb",	3, 15, 14},
-	{"arpeggio",	2, 27, 12, 10},
-	{"crescendo(",	7, -1, 18},
-	{"crescendo)",	7, 19, 18},
-	{"<(",		7, -1, 18},
-	{"<)",		7, 19, 18},
-	{"diminuendo(", 7, -1, 18},
-	{"diminuendo)", 7, 32, 18},
-	{">(",		7, -1, 18},
-	{">)",		7, 32, 18},
-	{">",		3, 21, 8},
-	{"invisible",	32},
-	{"beamon",	33},
-	{"trem1",	34},
-	{"trem2",	34},
-	{"trem3",	34},
-	{"trem4",	34},
-	{"xstem",	35},
+	{"roll",	3, 1, 10},	/* 2 */
+	{"fermata",	3, 2, 12},	/* 3 */
+	{"emphasis",	3, 3, 8},	/* 4 */
+	{"lowermordent", 3, 4, 10},	/* 5 */
+	{"coda",	3, 5, 24},	/* 6 */
+	{"uppermordent", 3, 6, 10},	/* 7 */
+	{"segno",	3, 7, 20},	/* 8 */
+	{"trill",	3, 8, 11},	/* 9 */
+	{"upbow",	3, 9, 10},	/* 10 */
+	{"downbow",	3, 10, 9},	/* 11 */
+	{"gmark",	3, 11, 6},	/* 12 */
+	{"slide",	1, 12, 3, 7},	/* 13 */
+	{"tenuto",	0, 13, 2},	/* 14 */
+	{"breath",	3, 14, 0},	/* 15 */
+	{"longphrase",	3, 15, 0},	/* 16 */
+	{"mediumphrase", 3, 16, 0},	/* 17 */
+	{"shortphrase", 3, 17, 0},	/* 18 */
+	{"invertedfermata", 3, 2, 12},	/* 19 */
+	{"invertedturn", 3, 18, 10},	/* 20 */
+	{"invertedturnx", 3, 19, 10},	/* 21 */
 };
 
 /* c function table */
@@ -158,43 +106,84 @@ static draw_f *func_tb[] = {
 /* postscript function table */
 static char *ps_func_tb[128] = {
 	"stc",		/* 0: dot */
-	"emb",		/* 1: tenuto */
-	"sld",		/* 2: slide */
-	"grm",		/* 3: gracing mark */
-	"umrd",		/* 4: uppermordent */
-	"lmrd",		/* 5: lowermordent */
-	"hld",		/* 6: fermata */
-	"trl",		/* 7: trill */
-	"upb",		/* 8: upbow */
-	"dnb",		/* 9: downbow */
-	"cpu",		/* 10: roll */
-	"fng",		/* 11: fingers */
-	"dacs",		/* 12: D.C./ D.S. */
-	"pf",		/* 13: p, f, pp, .. */
-	"sfz",		/* 14: sfz */
-	"thumb",	/* 15: thumb */
-	"coda",		/* 16: coda */
-	"sgno",		/* 17: segno */
-	"brth",		/* 18: breath */
-	"cresc",	/* 19: crescendo */
-	"dplus",	/* 20: plus */
-	"accent",	/* 21: accent */
-	"turn",		/* 22: turn */
-	"turnx",	/* 23: turn with bar */
-	"lphr",		/* 24: longphrase */
-	"mphr",		/* 25: mediumphrase */
-	"sphr",		/* 26: shortphrase */
-	"arp",		/* 27: arpeggio */
-	"ltr",		/* 28: long trill */
-	"wedge",	/* 29: wedge */
-	"opend",	/* 30: open */
-	"snap",		/* 31: snap */
-	"dim",		/* 32: diminuendo */
+	"cpu",		/* 1: roll */
+	"hld",		/* 2: fermata */
+	"accent",	/* 3: accent */
+	"lmrd",		/* 4: lowermordent */
+	"coda",		/* 5: coda */
+	"umrd",		/* 6: uppermordent */
+	"sgno",		/* 7: segno */
+	"trl",		/* 8: trill */
+	"upb",		/* 9: upbow */
+	"dnb",		/* 10: downbow */
+	"grm",		/* 11: gracing mark */
+	"sld",		/* 12: slide */
+	"emb",		/* 13: tenuto */
+	"brth",		/* 14: breath */
+	"lphr",		/* 15: longphrase */
+	"mphr",		/* 16: mediumphrase */
+	"sphr",		/* 17: shortphrase */
+	"turn",		/* 18: turn */
+	"turnx",	/* 19: turn with bar */
 };
 
-static char *str_tb[32] = {
-	0,
-	"FINE",		/* 1 */
+static char *str_tb[32];
+
+static char *std_deco_tb[] = {
+	"0 3 fng 8 0 0 0",
+	"1 3 fng 8 0 0 1",
+	"2 3 fng 8 0 0 2",
+	"3 3 fng 8 0 0 3",
+	"4 3 fng 8 0 0 4",
+	"5 3 fng 8 0 0 5",
+	"plus 3 dplus 7 0 0",
+	"+ 3 dplus 7 0 0",
+	"accent 3 accent 8 0 0",
+	"> 3 accent 8 0 0",
+	"D.C. 3 dacs 16 0 0 D.C.",
+	"D.S. 3 dacs 16 0 0 D.S.",
+	"fine 3 dacs 16 0 0 FINE",
+	"f 6 pf 18 4 4",
+	"ff 6 pf 18 5 11",
+	"fff 6 pf 18 7 17",
+	"ffff 6 pf 18 8 24",
+	"mf 6 pf 18 5 11",
+	"mp 6 pf 18 5 11",
+	"mordent 3 lmrd 10 0 0",
+	"open 3 opend 10 0 0",
+	"p 6 pf 18 4 4",
+	"pp 6 pf 18 5 11",
+	"ppp 6 pf 18 7 17",
+	"pppp 6 pf 18 8 24",
+	"pralltriller 3 umrd 10 0 0",
+	"sfz 6 sfz 18 7 17",
+	"turn 3 turn 10 0 0",
+	"wedge 3 wedge 8 0 0",
+	"turnx 3 turnx 10 0 0",
+	"trill( 5 - 8 0 0",
+	"trill) 5 ltr 8 0 0",
+	"snap 3 snap 14 0 0",
+	"thumb 3 thumb 14 0 0",
+	"arpeggio 2 arp 12 10 0",
+	"crescendo( 7 - 18 0 0",
+	"crescendo) 7 cresc 18 0 0",
+	"<( 7 - 18 0 0",
+	"<) 7 cresc 18 0 0",
+	"diminuendo( 7 - 18 0 0",
+	"diminuendo) 7 dim 18 0 0",
+	">( 7 - 18 0 0",
+	">) 7 dim 18 0 0",
+	"invisible 32 0 0 0 0",
+	"beamon 33 0 0 0 0",
+	"trem1 34 0 0 0 0",
+	"trem2 34 0 0 0 0",
+	"trem3 34 0 0 0 0",
+	"trem4 34 0 0 0 0",
+	"xstem 35 0 0 0 0",
+	"beambr1 36 0 0 0 0",
+	"beambr2 36 0 0 0 0",
+	"rbstop 37 0 0 0 0",
+	0
 };
 
 static struct SYMBOL *first_note;	/* first note/rest of the line */
@@ -800,7 +789,8 @@ void deco_add(char *text)
 		error(1, 0, "Invalid deco %s", text);
 		return;
 	}
-	if (c_func < 0 || c_func >= sizeof func_tb / sizeof func_tb[0]) {
+	if ((c_func < 0 || c_func >= sizeof func_tb / sizeof func_tb[0])
+	    && (c_func < 32 || c_func > 37)) {
 		error(1, 0, "%%%%deco: bad C function index (%s)", text);
 		return;
 	}
@@ -940,6 +930,19 @@ void deco_cnv(struct deco *dc,
 			}
 			s->sflags |= S_XSTEM;
 			break;
+		case 36:		/* 36 = beambr1 / beambr2 */
+			if (s->type != NOTE) {
+				error(1, s,
+				      "Cannot have +%s+ on a rest or a bar",
+				       dd->name);
+				break;
+			}
+			s->sflags |= dd->name[6] == '1' ?
+				S_BEAM_BR1 : S_BEAM_BR2;
+			break;
+		case 37:		/* 37 = rbstop */
+			s->sflags |= S_RBSTOP;
+			break;
 		}
 		dc->t[i] = 0;
 	}
@@ -948,6 +951,60 @@ void deco_cnv(struct deco *dc,
 		s->sflags |= p_voice->dyn > 0 ? S_DYNUP : S_DYNDN;
 }
 
+/* -- define a standard decoration -- */
+static unsigned char deco_define(char *name)
+{
+	unsigned char deco;
+	int l;
+
+	l = strlen(name);
+	for (deco = 0; ; deco++) {
+		if (std_deco_tb[deco] == 0)
+			return 128;
+		if (strncmp(std_deco_tb[deco], name, l) == 0
+		    && std_deco_tb[deco][l] == ' ') {
+			deco_add(std_deco_tb[deco]);
+			switch (name[l - 1]) {
+			case '(': deco_add(std_deco_tb[deco + 1]); break;
+			case ')': deco_add(std_deco_tb[deco - 1]); break;
+			}
+			for (deco = 1; deco < 128; deco++) {
+				if (deco_def_tb[deco].name == 0) {
+					deco = 128;
+					break;
+				}
+				if (strcmp(deco_def_tb[deco].name, name) == 0)
+					break;
+			}
+			break;
+		}
+	}
+	return deco;
+}
+
+/* -- convert the external deco number to the internal one -- */
+unsigned char deco_intern(unsigned char deco)
+{
+	char *name;
+
+	if (deco == 0)
+		return deco;
+	name = deco_tb[deco - 128];
+	for (deco = 1; deco < 128; deco++) {
+		if (deco_def_tb[deco].name == 0) {
+			deco = deco_define(name);	/* try a standard decoration */
+			break;
+		}
+		if (strcmp(deco_def_tb[deco].name, name) == 0)
+			break;
+	}
+	if (deco == 128) {
+		error(1, 0, "Decoration %s not treated", name);
+		deco = 0;
+	}
+	return deco;
+}
+
 /* -- update the x position of a decoration -- */
 void deco_update(struct SYMBOL *s, float dx)
 {
@@ -964,27 +1021,6 @@ void deco_update(struct SYMBOL *s, float dx)
 	}
 }
 
-/* -- convert the external deco number to the internal one -- */
-unsigned char deco_intern(unsigned char deco)
-{
-	char *name;
-
-	if (deco == 0)
-		return deco;
-	name = deco_tb[deco - 128];
-	for (deco = 1; deco < 128; deco++) {
-		if (deco_def_tb[deco].name == 0)
-			deco = 127;
-		else if (strcmp(deco_def_tb[deco].name, name) == 0)
-			break;
-	}
-	if (deco == 128) {
-		error(1, 0, "Decoration %s not treated", name);
-		deco = 0;
-	}
-	return deco;
-}
-
 /* -- adjust the symbol width -- */
 float deco_width(struct SYMBOL *s)
 {
@@ -1001,8 +1037,14 @@ float deco_width(struct SYMBOL *s)
 
 		dd =  &deco_def_tb[dc->t[i]];
 		switch (dd->func) {
-		case 1: wl += 7; break;		/* slide */
-		case 2: wl += 10; break;	/* arpeggio */
+		case 1:			/* slide */
+			if (wl < 7)
+				wl = 7;
+			break;
+		case 2:			/* arpeggio */
+			if (wl < 10)
+				wl = 10;
+			break;
 		}
 	}
 	return wl;
@@ -1065,7 +1107,7 @@ void draw_all_deco(void)
 			}
 		}
 
-		set_scale(de->s->staff);
+		set_scale(de->s->voice);
 		set_defl(de->defl);
 /*fixme: scaled or not?*/
 		if (de->flags & DE_VAL)
@@ -1447,19 +1489,34 @@ void draw_deco_staff(void)
 				first_repeat = s;
 			}
 			s1 = s;
-			i = 4;
+			i = s1->as.text ? 4 : 2;
 			for (;;) {
 				if (s->next == 0)
 					break;
 				s = s->next;
-				if (s->type == BAR) {
-					if (((s->as.u.bar.type & 0xf0)	/* if complex bar */
-					     && s->as.u.bar.type != (B_OBRA << 4) + B_CBRA)
-					    || s->as.u.bar.type == B_CBRA
-					    || s->as.u.bar.repeat_bar)
-						break;
-					if (--i < 0)	/*fixme*/
-						break;
+				if (s->sflags & S_RBSTOP)
+					break;
+				if (s->type != BAR)
+					continue;
+				if (((s->as.u.bar.type & 0xf0)	/* if complex bar */
+				     && s->as.u.bar.type != (B_OBRA << 4) + B_CBRA)
+				    || s->as.u.bar.type == B_CBRA
+				    || s->as.u.bar.repeat_bar)
+					break;
+				if (--i <= 0) {
+
+					/* have a shorter repeat bracket */
+					s = s1;
+					i = 2;
+					for (;;) {
+						s = s->next;
+						if (s->type != BAR)
+							continue;
+						if (--i <= 0)
+							break;
+					}
+					s->sflags |= S_RBSTOP;
+					break;
 				}
 			}
 			y2 = y_get(s1, 1, s1->x, s->x - s1->x, 0);
@@ -1467,7 +1524,15 @@ void draw_deco_staff(void)
 				y = y2;
 
 			/* have room for the repeat numbers */
-			if (s1->as.text != 0)
+			if (s1->as.text != 0) {
+#ifdef YSTEP
+				w = cwid('W') * cfmt.repeatfont.swfac
+					* strlen(s1->as.text);
+				y2 = y_get(s1, 1, s1->x + 4, w, 0);
+				y2 += cfmt.repeatfont.size + 2;
+				if (y < y2)
+					y = y2;
+#else
 			    for (s2 = s1->next; s2 != s; s2 = s2->next) {
 				switch (s2->type) {
 				case PART:
@@ -1476,22 +1541,22 @@ void draw_deco_staff(void)
 				case TUPLET:
 				case WHISTLE:
 					continue;
+				case GRACE:
 				case NOTE:
 				case REST:
-#ifdef YSTEP
-					y2 = y_get(s2, 1, s2->x, 3, 0);
-#else
 					y2 = s2->dc_top;
-#endif
 					if (s2->as.text != 0)	/* if guitar chord */
 						y2 -= cfmt.font_tb[s->gcf].size; /* got already */
-					y2 += cfmt.repeatfont.size;
+					y2 += cfmt.repeatfont.size + 2;
 					if (y < y2)
 						y = y2;
+					break;
 				default:
 					break;
 				}
 				break;
+			    }
+#endif
 			}
 			if (s->as.u.bar.repeat_bar)
 				s = s->prev;
@@ -1506,30 +1571,21 @@ void draw_deco_staff(void)
 			    || !s->as.u.bar.repeat_bar)
 				continue;
 			s1 = s;
-			i = 4;
-			s2 = 0;
 			for (;;) {
-				if (s->next == 0) {
-					s2 = s;
+				if (s->next == 0)
 					break;
-				}
 				s = s->next;
-				if (s->type == BAR) {
-					if (((s->as.u.bar.type & 0xf0)	/* if complex bar */
-					     && s->as.u.bar.type != (B_OBRA << 4) + B_CBRA)
-					    || s->as.u.bar.type == B_CBRA
-					    || s->as.u.bar.repeat_bar) {
-						s2 = s;
-						break;
-					}
-					if (i == 4)
-						s2 = s;
-					if (--i < 0)	/*fixme*/
-						break;
-				}
+				if (s->sflags & S_RBSTOP)
+					break;
+				if (s->type != BAR)
+					continue;
+				if (((s->as.u.bar.type & 0xf0)	/* if complex bar */
+				     && s->as.u.bar.type != (B_OBRA << 4) + B_CBRA)
+				    || s->as.u.bar.type == B_CBRA
+				    || s->as.u.bar.repeat_bar)
+					break;
 			}
-			if (s2 == 0)
-				s2 = s;
+			s2 = s;
 /*fixme*/
 			if (s1 == s2)
 				break;
@@ -1538,7 +1594,9 @@ void draw_deco_staff(void)
 				x -= 4;
 			i = 0;			/* no bracket end */
 			w = s2->x - x - 8;
-			if (s2->type != BAR)
+			if (s2->sflags & S_RBSTOP)
+				;
+			else if (s2->type != BAR)
 				w = realwidth - x - 4;
 			else if (((s2->as.u.bar.type & 0xf0)	/* if complex bar */
 				   && s2->as.u.bar.type != (B_OBRA << 4) + B_CBRA)
@@ -1559,17 +1617,19 @@ void draw_deco_staff(void)
 				i--;		/* no bracket start (1) or not draw */
 				p = "";
 			}
-			if (i == 0 && s2->next == 0) {	/* 2nd ending at end of line */
+			if (i == 0 && s2->next == 0	/* 2nd ending at end of line */
+			    && !(s2->sflags & S_RBSTOP)) {
 				if (p_voice->bar_start != 0)
 					i = 2;		/* bracket start and stop */
 				else	repnl = 1;	/* continue on next line */
 			}
 			if (i >= 0) {
-				PUT2("(%s)%d ", p, i);
+				PUT3("(%s)-%.1f %d ",
+				     p, cfmt.repeatfont.size * 0.8 + 1, i);
 				putf(w);
 				putxy(x, y);
 				PUT1("y%d repbra\n", s1->staff);
-				y_set(s1, 1, x, w, y + 2.);
+				y_set(s1, 1, x, w, y + 2);
 			}
 			if (s->as.u.bar.repeat_bar)
 				s = s->prev;
@@ -1841,8 +1901,6 @@ void draw_measnb(void)
 	for ( ; s->next != 0; s = s->next) {
 		switch (s->type) {
 		case TIMESIG:
-			wmeasure = s->as.u.meter.wmeasure;
-			continue;
 		case CLEF:
 		case KEYSIG:
 		case PART:
