@@ -3,7 +3,7 @@
  *
  * This file is part of abcm2ps.
  *
- * Copyright (C) 1998-2005 Jean-Franois Moine
+ * Copyright (C) 1998-2006 Jean-Franois Moine
  * Adapted from abc2ps, Copyright (C) 1996,1997 Michael Methfessel
  *
  * This program is free software; you can redistribute it and/or modify
@@ -46,6 +46,7 @@ static void draw_note(float x,
 		      struct SYMBOL *s,
 		      int fl);
 static void set_sscale(int staff);
+static void set_tie_room(void);
 
 /* -- up/down shift needed to get k*6 -- */
 static float rnd6(float y)
@@ -92,7 +93,7 @@ static int calculate_beam(struct BEAM *bm,
 			  struct SYMBOL *s1)
 {
 	struct SYMBOL *s, *s2;
-	int notes, nflags, staff, voice, two_staves;
+	int notes, nflags, staff, voice, two_staves, two_dir;
 	float x, y, ys, a, b, max_stem_err;
 	float sx, sy, sxx, sxy, syy, a0, stem_xoff, scale;
 	static float min_tb[2][6] = {
@@ -108,26 +109,29 @@ static int calculate_beam(struct BEAM *bm,
   
 	/* find first and last note in beam */
 	notes = nflags = 0;	/* set x positions, count notes and flags */
-	two_staves = 0;
+	two_staves = two_dir = 0;
 	staff = s1->staff;
 	voice = s1->voice;
 	stem_xoff = s1->as.u.note.grace ? GSTEM_XOFF : STEM_XOFF;
-	for (s = s1; ; s = s->next) {
-		if (s->type != NOTE)
+	for (s2 = s1; ; s2 = s2->next) {
+#if 0
+		if (s2 == 0) {
+			error(1, s1, "No beam end!");
+			return 0;
+		}
+#endif
+		if (s2->type != NOTE)
 			continue;
-		if (s->nflags > nflags)
-			nflags = s->nflags;
+		if (s2->nflags > nflags)
+			nflags = s2->nflags;
 		notes++;
-		if (s->staff != staff)
+		if (s2->staff != staff)
 			two_staves = 1;
-		if (s->as.u.note.word_end)
+		if (s2->stem != s1->stem)
+			two_dir = 1;
+		if (s2->as.u.note.word_end)
 			break;
 	}
-	if ((s2 = s) == 0) {
-		error(1, s1, "No beam end!");
-		return 0;
-	}
-
 	bm->s2 = s2;			/* (don't display the flags) */
 	if (staff_tb[staff].y == 0) {	/* staves not defined */
 		if (two_staves)
@@ -143,24 +147,6 @@ static int calculate_beam(struct BEAM *bm,
 		}
 	}
 
-#if 0
-	/* make the beam to follow the note pitches */
-	if (s1->stem * s2->stem > 0) {
-#if 1
-		a = s1->y - s2->y;
-#else
-		a = s1->yav - s2->yav;
-#endif
-		b = s1->ys - s2->ys;
-		if (b != a) {
-			if ((s1->stem > 0 && b < a)
-			    || (s1->stem < 0 && b > a))
-				s1->ys = s2->ys + a;
-			else	s2->ys = s1->ys - a;
-		}
-	}
-#endif
-
 	sx = sy = sxx = sxy = syy = 0;	/* linear fit through stem ends */
 	for (s = s1; ; s = s->next) {
 		if (s->type != NOTE)
@@ -227,7 +213,7 @@ static int calculate_beam(struct BEAM *bm,
 	/* have room for the symbols in the staff */
 	max_stem_err = 0;		/* check stem lengths */
 	s = s1;
-	if (two_staves) {
+	if (two_dir) {
 /*fixme: more to do*/
 		if (!s1->as.u.note.grace)
 			ys = BEAM_SHIFT;
@@ -281,22 +267,20 @@ static int calculate_beam(struct BEAM *bm,
 /*fixme: KO when the voice numbers are inverted */
 						if (s->voice < voice)
 							continue;
-						if (s->y - 4 > ys)
+						if (s->y > ys + 4)
 							continue;
-						stem_err = s->ys - ys;
-					} else	stem_err = s->ymx - ys;
-				} else {
-					if (s->stem > 0)
-						stem_err = ys - s->ymn;
-					else {
-						if (s->voice < voice)
-							continue;
-						if (s->y + 4 < ys)
-							continue;
-						stem_err = ys - s->ys;
 					}
+					stem_err = s->ymx - ys;
+				} else {
+					if (s->stem < 0) {
+						if (s->voice > voice)
+							continue;
+						if (s->y < ys - 4)
+							continue;
+					}
+					stem_err = ys - s->ymn;
 				}
-				stem_err += 5 + BEAM_DEPTH + BEAM_SHIFT * (s->nflags - 1);
+				stem_err += 2 + BEAM_DEPTH + BEAM_SHIFT * (s1->nflags - 1);
 			}
 			if (stem_err > max_stem_err)
 				max_stem_err = stem_err;
@@ -306,10 +290,11 @@ static int calculate_beam(struct BEAM *bm,
 			float stem_err;
 
 			ys = a * s->xs + b - staff_tb[s->staff].y;
-			stem_err = GSTEM
-				- (s->stem > 0
-					? ys - (float) (3 * (s->pits[s->nhd] - 18))
-					: (float) (3 * (s->pits[0] - 18)) - ys);
+			stem_err = GSTEM - 2;
+			if (s->stem > 0)
+				stem_err -= ys - (float) (3 * (s->pits[s->nhd] - 18));
+			else	stem_err += ys - (float) (3 * (s->pits[0] - 18));
+			stem_err += 3 * (s->nflags - 1);
 			if (stem_err > max_stem_err)
 				max_stem_err = stem_err;
 			if (s == s2)
@@ -321,21 +306,24 @@ static int calculate_beam(struct BEAM *bm,
 		b += s1->stem * max_stem_err;
 
 	/* have room for the gracenotes, bars and clefs */
+/*fxime: test*/
+    if (!two_staves && !two_dir)
 	for (s = s1->next; ; s = s->next) {
 		struct SYMBOL *g;
 
 		if ((g = s->grace) == 0) {
-			if (s->type == CLEF || s->type == BAR) {
+			if (s->type == CLEF
+			    || (s->type == BAR && s->as.u.bar.type != B_INVIS)) {
 				y = a * s->x + b;
-				if (s->next->stem > 0) {
+				if (s1->stem > 0) {
 					y = s->ymx - y
-						+ BEAM_DEPTH + BEAM_SHIFT * (s->nflags - 1)
+						+ BEAM_DEPTH + BEAM_SHIFT * (nflags - 1)
 						+ 2;
 					if (y > 0)
 						b += y;
 				} else {
 					y = s->ymn - y
-						- BEAM_DEPTH - BEAM_SHIFT * (s->nflags - 1)
+						- BEAM_DEPTH - BEAM_SHIFT * (nflags - 1)
 						- 7;
 					if (y < 0)
 						b += y;
@@ -347,15 +335,15 @@ static int calculate_beam(struct BEAM *bm,
 		}
 		for ( ; g != 0; g = g->next) {
 			y = a * g->x + b;
-			if (s->next->stem > 0) {
+			if (s1->stem > 0) {
 				y = g->ys - y
-					+ BEAM_DEPTH + BEAM_SHIFT * (g->nflags - 1)
+					+ BEAM_DEPTH + BEAM_SHIFT * (nflags - 1)
 					+ 2;
 				if (y > 0)
 					b += y;
 			} else {
 				y = g->y - y
-					- BEAM_DEPTH - BEAM_SHIFT * (g->nflags - 1)
+					- BEAM_DEPTH - BEAM_SHIFT * (nflags - 1)
 					- 7;
 				if (y < 0)
 					b += y;
@@ -522,33 +510,48 @@ static void draw_beams(struct BEAM *bm)
 			}
 			k1 = s;
 			for (;;) {
-				if (s == s2
-				    || (s->next->len > 0 && s->next->nflags < i)
-				    || (s->sflags & S_BEAM_BREAK))
+				if (s == s2)
 					break;
 				s = s->next;
+				if ((s->len > 0 && s->nflags < i)
+				    || (s->sflags & S_BEAM_BR1)
+				    || ((s->sflags & S_BEAM_BR2) && i > 2)) {
+					s = s->prev;
+					break;
+				}
 			}
 			k2 = s;
 			while (k2->type != NOTE)
 				k2 = k2->prev;
 			x1 = k1->xs;
 			if (k1 == k2) {
-				if (k1 == s1)
+				if (k1 == s1
+				    || (k1->sflags & S_BEAM_BR1)
+				    || ((k1->sflags & S_BEAM_BR2)
+					&& i > 2)) {
 					x1 += bstub;
-				else if (k1 == s2 || (k1->sflags & S_BEAM_BREAK)) {
+				} else if (k1 == s2)
 					x1 -= bstub;
-				} else {
+				else {
 					struct SYMBOL *k;
 
-					k = k1->prev;
+					k = k1->next;
 					while (k->type != NOTE)
-						k = k->prev;
-					if ((k->sflags & S_BEAM_BREAK)
-					    || k->nflags < k1->next->nflags
-					    || (k->nflags == k1->next->nflags
-					     && k->dots < k1->next->dots))
-						x1 += bstub;
-					else	x1 -= bstub;
+						k = k->next;
+					if ((k->sflags & S_BEAM_BR1)
+					    || ((k->sflags & S_BEAM_BR2)
+						&& i > 2))
+						x1 -= bstub;
+					else {
+						k1 = k1->prev;
+						while (k1->type != NOTE)
+							k1 = k1->prev;
+						if (k1->nflags < k->nflags
+						    || (k1->nflags == k->nflags
+							&& k1->dots < k->dots))
+							x1 += bstub;
+						else	x1 -= bstub;
+					}
 				}
 			}
 			draw_beam(x1, k2->xs,
@@ -557,8 +560,7 @@ static void draw_beams(struct BEAM *bm)
 #else
 				  shift * k1->stem,	/*fixme: more complicated */
 #endif
-				  bh,
-				  bm);
+				  bh, bm);
 			if (s == s2)
 				break;
 		}
@@ -589,18 +591,31 @@ static int topbar(int nlines)
 /* -- draw the left side of the staves -- */
 static void draw_lstaff(float x)
 {
-	int i, j;
-	float xb, y, yb, ybk;
+	int i, j, brace, bracket;
+	float y, yb, x0, y0;
 
-	for (i = 0; i < nstaff; i++)
+	if (cfmt.alignbars)
+		return;
+	brace = 0;
+	for (i = 0; i < nstaff; i++) {
+		if (staff_tb[i].brace || staff_tb[i].bracket)
+			brace++;
+		if (staff_tb[i].brace_end || staff_tb[i].bracket_end)
+			brace--;
 		if (!staff_tb[i].empty
 		    && staff_tb[i].clef.stafflines != 0)
 			break;
-	for (j = nstaff; j > i; j--)
+	}
+	for (j = nstaff; j > i; j--) {
+		if (staff_tb[i].brace || staff_tb[i].bracket)
+			brace++;
+		if (staff_tb[i].brace_end || staff_tb[i].bracket_end)
+			brace--;
 		if (!staff_tb[j].empty
 		    && staff_tb[j].clef.stafflines != 0)
 			break;
-	if (i == j)
+	}
+	if (i == j && brace == 0)
 		return;
 	set_scale(-1);
 	PUT3("%.1f %.1f %.1f bar\n",
@@ -608,29 +623,55 @@ static void draw_lstaff(float x)
 		+ topbar(staff_tb[i].clef.stafflines) * staff_tb[i].clef.staffscale,
 	     x, staff_tb[j].y
 		+ botbar(staff_tb[j].clef.stafflines) * staff_tb[j].clef.staffscale);
-	xb = yb = ybk = 0;
+	x0 = y0 = yb = 0;
+	brace = bracket = 0;
 	for (i = 0; i <= nstaff; i++) {
+		if (staff_tb[i].empty
+		    || staff_tb[i].clef.stafflines == 0) {
+			if (staff_tb[i].brace)
+				brace++;
+			if (staff_tb[i].bracket)
+				bracket++;
+			if (staff_tb[i].brace_end)
+				brace--;
+			if (staff_tb[i].bracket_end)
+				bracket--;
+			continue;
+		}
 		y = staff_tb[i].y;
-		if (staff_tb[i].brace)
+		if (staff_tb[i].brace || brace) {
+			x0 = x;
+			y0 = yb;
 			yb = y + topbar(staff_tb[i].clef.stafflines)
 				* staff_tb[i].clef.staffscale;
-		if (staff_tb[i].bracket) {
-			ybk = y + topbar(staff_tb[i].clef.stafflines)
+			x -= 6;
+			if (brace)
+				brace = 0;
+		}
+		if (staff_tb[i].bracket || bracket) {
+			x0 = x;
+			y0 = yb;
+			yb = y + topbar(staff_tb[i].clef.stafflines)
 				* staff_tb[i].clef.staffscale;
-			xb = -6;
+			x -= 6;
+			if (bracket)
+				bracket = 0;
 		}
 		if (staff_tb[i].brace_end) {
+			x += 6;
 			y += botbar(staff_tb[i].clef.stafflines)
 				* staff_tb[i].clef.staffscale;
 			PUT3("%.1f %.1f %.1f brace\n",
-			     yb - y, x + xb, yb);
+			     yb - y, x, yb);
+			yb = y0;
 		}
 		if (staff_tb[i].bracket_end) {
+			x += 6;
 			y += botbar(staff_tb[i].clef.stafflines)
 				* staff_tb[i].clef.staffscale;
 			PUT3("%.1f %.1f %.1f bracket\n",
-			     ybk - y, x, ybk);
-			xb = 0;
+			     yb - y, x, yb);
+			yb = y0;
 		}
 	}
 }
@@ -676,20 +717,17 @@ static void draw_timesig(float x,
 			 struct SYMBOL *s)
 {
 	int i, staff, l;
+	char *f, meter[64];
+	float dx;
 
 	if (s->as.u.meter.nmeter == 0)
 		return;
 	staff = s->staff;
 	x = x - s->wl + 2;
 	for (i = 0; i < s->as.u.meter.nmeter; i++) {
-		char *f, meter[64];
-		float dx;
-
-		if (s->as.u.meter.meter[i].top[0] == ' ') {
-			x += 6.;			/* half-space */
-			continue;
-		}
 		l = strlen(s->as.u.meter.meter[i].top);
+		if (l > sizeof s->as.u.meter.meter[i].top)
+			l = sizeof s->as.u.meter.meter[i].top;
 		if (s->as.u.meter.meter[i].bot[0] != '\0') {
 			int l2;
 
@@ -698,6 +736,8 @@ static void draw_timesig(float x,
 				s->as.u.meter.meter[i].bot);
 			f = "tsig";
 			l2 = strlen(s->as.u.meter.meter[i].bot);
+			if (l2 > sizeof s->as.u.meter.meter[i].bot)
+				l2 = sizeof s->as.u.meter.meter[i].bot;
 			if (l2 > l)
 				l = l2;
 		} else {
@@ -718,7 +758,7 @@ static void draw_timesig(float x,
 			}
 		}
 		PUT1("%s ", meter);
-		dx = (float) (7 * l);
+		dx = (float) (13 * l);
 		putxy(x + dx * 0.5, staff_tb[staff].y);
 		PUT1("%s\n", f);
 		x += dx;
@@ -858,7 +898,7 @@ static void draw_keysig(struct VOICE_S *p_voice,
 			x += 5;
 		}
 	}
-	if (old_sf != 0 || s->as.u.key.sf != 0 || s->as.u.key.nacc != 0)
+	if (old_sf != 0 || s->as.u.key.sf != 0 || s->as.u.key.nacc >= 0)
 		PUT0("\n");
 }
 
@@ -971,10 +1011,8 @@ static char *rest_tb[NFLAGS_SZ] = {
 	"r2", "r1", "r0", "r00"
 };
 
-	x = s->x + s->shhd[0] * cur_scale;
-	y = s->y;
-
 	/* if rest alone in the measure, center */
+	x = s->x + s->shhd[0] * cur_scale;
 	if (s->len == voice_tb[s->voice].meter.wmeasure) {
 		struct SYMBOL *prev;
 
@@ -1008,6 +1046,7 @@ static char *rest_tb[NFLAGS_SZ] = {
 	}
 
 	staffb = staff_tb[s->staff].y;		/* bottom of staff */
+	y = s->y;
 
 	if (s->sflags & S_OTHER_HEAD) {
 		draw_all_deco_head(s, x, y + staffb);
@@ -1015,44 +1054,73 @@ static char *rest_tb[NFLAGS_SZ] = {
 	}
 
 	i = C_XFLAGS - s->nflags;		/* rest_tb index */
+	if (i == 7 && y == 12
+	    && staff_tb[s->staff].clef.stafflines <= 2)
+		y -= 6;				/* semibreve a bit lower */
 	putxy(x, y + staffb);
 	PUT1("%s ", rest_tb[i]);
 
-	/* add helper line(s) */
-	switch (i) {
-	case 9:					/* breve / longa */
-	case 8:
-		if (y >= 24) {
-			puty(y + 6 + staffb);
-			PUT0("hl ");
+	/* add helper line(s) when greater than minim*/
+	if (i >= 6) {
+		int yb, yt;
+
+		switch (staff_tb[s->staff].clef.stafflines) {
+		case 0:
+			yb = 12;
+			yt = 12;
+			break;
+		case 1:
+			yb = 6;
+			yt = 18;
+			break;
+		case 2:
+			yb = 0;
+			yt = 18;
+			break;
+		case 3:
+			yb = 0;
+			yt = 24;
+			break;
+		default:
+			yb = -6;
+			yt = staff_tb[s->staff].clef.stafflines * 6;
+			break;
 		}
-		if (i == 8) {
-			if (y <= -6) {
+		switch (i) {
+		case 6:					/* minim */
+			if (y <= yb || y >= yt) {
 				puty(y + staffb);
 				PUT0("hl ");
 			}
-		} else {
-			if (y <= 0) {
-				puty(y - 6 + staffb);
+			break;
+		case 7:					/* semibreve */
+			if (y < yb || y >= yt - 6) {
+				puty(y + 6 + staffb);
 				PUT0("hl ");
 			}
+			break;
+		default:
+			if (y >= yt - 6) {
+				puty(y + 6 + staffb);
+				PUT0("hl ");
+				break;
+			}
+			if (i == 8) {			/* breve */
+				if (y <= yb) {
+					puty(y + staffb);
+					PUT0("hl ");
+				}
+			} else {			/* longa */
+				if (y <= yb + 6) {
+					puty(y - 6 + staffb);
+					PUT0("hl ");
+				}
+			}
+			break;
 		}
-		break;
-	case 7:					/* semibreve */
-		if (y < -6 || y >= 24) {
-			puty(y + 6 + staffb);
-			PUT0("hl ");
-		}
-		break;
-	case 6:					/* minim */
-		if (y <= -6 || y >= 30) {
-			puty(y + staffb);
-			PUT0("hl ");
-		}
-		break;
 	}
 
-	dotx = 8.0;
+	dotx = 8;
 	for (i = 0; i < s->dots; i++) {
 		PUT1("%.1f 3 dt ", dotx);
 		dotx += 3.5;
@@ -1155,7 +1223,6 @@ static void draw_gracenotes(struct SYMBOL *s)
 	x2 = bet2 * x3 + (1 - bet2) * x0;
 	y2 = bet2 * y3 + (1 - bet2) * y0 - dy2;
 
-	PUT0(" ");
 	putxy(x1, y1 + staffb);
 	putxy(x2, y2 + staffb);
 	putxy(x3, y3 + staffb);
@@ -1163,15 +1230,53 @@ static void draw_gracenotes(struct SYMBOL *s)
 	PUT0("gsl\n");
 }
 
+/* -- set the y offset of the dots -- */
+static void setdoty(struct SYMBOL *s,
+		    signed char *y_tb)
+{
+	int m, m1, y, doty;
+
+	/* set the normal offsets */
+	doty = s->doty;
+	for (m = 0; m <= s->nhd; m++) {
+		y = 3 * (s->pits[m] - 18);	/* note height on staff */
+		if ((y % 6) == 0) {
+			if (doty != 0)
+				y -= 3;
+			else	y += 3;
+		}
+		y_tb[m] = y;
+	}
+
+	/* dispatch and recenter the dots in the staff spaces */
+	for (m = 0; m < s->nhd; m++) {
+		if (y_tb[m + 1] > y_tb[m])
+			continue;
+		m1 = m;
+		while (m1 > 0) {
+			if (y_tb[m1] > y_tb[m1 - 1] + 6)
+				break;
+			m1--;
+		}
+		if (3 * (s->pits[m1] - 18) - y_tb[m1]
+				< y_tb[m + 1] - 3 * (s->pits[m + 1] - 18)) {
+			while (m1 <= m)
+				y_tb[m1++] -= 6;
+		} else	y_tb[m + 1] = y_tb[m] + 6;
+	}
+}
+
 /* -- draw m-th head with accidentals and dots -- */
 /* (the staves are defined) */
 static void draw_basic_note(float x,
 			    struct SYMBOL *s,
-			    int m)
+			    int m,
+			    signed char *y_tb)
 {
 	int i, y, no_head, head, dots, nflags;
 	float staffb, shhd;
 	char *p;
+	char perc_hd[8];
 
 	staffb = staff_tb[s->staff].y;		/* bottom of staff */
 	y = 3 * (s->pits[m] - 18);		/* note height on staff */
@@ -1217,11 +1322,11 @@ static void draw_basic_note(float x,
 		p = "/y exch def/x exch def";
 	else if (s->as.u.note.grace)
 		p = "ghd";
-	else if (s->as.u.note.accs[m]
-		 && voice_tb[s->voice].clef.type == PERC) {
-		if (s->as.u.note.accs[m] == A_FT)
-			p = "pflhd";
-		else	p = "pshhd";
+	else if (staff_tb[s->staff].clef.type == PERC
+		 && (i = s->as.u.note.accs[m]) != 0) {
+		i &= 0x07;
+		sprintf(perc_hd, "p%shd", acc_tb[i]);
+		p = perc_hd;
 	} else {
 		switch (head) {
 		case H_OVAL:
@@ -1271,20 +1376,12 @@ static void draw_basic_note(float x,
 	/* draw the dots */
 /*fixme: to see for grace notes*/
 	if (dots) {
-		int doty;
 		float dotx;
+		int doty;
 
-		dotx = 8. + s->xmx;
-		if (y % 6)
-			doty = 0;
-		else	{
-			if ((doty = s->doty) == 0) {	/* defined when lowest voice */
-				if (m < s->nhd && s->pits[m] == s->pits[m + 1] - 1)
-					doty = -3;
-				else	doty = 3;
-			}
-		}
-		for (i = 0; i < dots; i++) {
+		dotx = (int) (8. + s->xmx);
+		doty = y_tb[m] - y;
+		while (--dots >= 0) {
 			PUT2(" %.1f %d dt", dotx - shhd, doty);
 			dotx += 3.5;
 		}
@@ -1292,7 +1389,7 @@ static void draw_basic_note(float x,
 
 	/* draw the accidental */
 	if ((i = s->as.u.note.accs[m]) != 0
-	    && voice_tb[s->voice].clef.type != PERC) {
+	    && staff_tb[s->staff].clef.type != PERC) {
 		x -= s->shac[m] * cur_scale;
 		PUT0(" ");
 		putx(x);
@@ -1311,16 +1408,18 @@ static void draw_note(float x,
 	int i, m, ma, y;
 	float staffb, slen;
 	char *hltype;
+	signed char y_tb[MAXHD];
 
-	staffb = staff_tb[s->staff].y;
+	if (s->dots)
+		setdoty(s, y_tb);
 
-	/* draw the master note - can be only the first or the last note */
+	/* draw the master note, first or last one */
 	if (cfmt.setdefl)
 		set_defl(s->stem >= 0 ? DEF_STEMUP : 0);
-	if (s->stem >= 0)
-		ma = 0;
-	else	ma = s->nhd;
-	draw_basic_note(x, s, ma);		/* draw the note head */
+	ma = s->stem >= 0 ? 0 : s->nhd;
+	draw_basic_note(x, s, ma, y_tb);
+
+	staffb = staff_tb[s->staff].y;
 	if (!s->as.u.note.invis
 	    && !s->as.u.note.stemless) {	/* add stem and flags */
 		char c, c2;
@@ -1337,8 +1436,7 @@ static void draw_note(float x,
 			slen /= voice_tb[s->voice].scale;
 			PUT3(" %.1f %c%c", slen, c2, c);
 		} else {				/* stem and flags */
-			if (voice_tb[s->voice].key.bagpipe
-			    && cfmt.straightflags)
+			if (cfmt.straightflags)
 				c = 's';		/* straight flag */
 			c2 = s->as.u.note.grace ? 'g' : 'f';
 			slen /= voice_tb[s->voice].scale;
@@ -1354,46 +1452,48 @@ static void draw_note(float x,
 		PUT1(" %.1f su", slen);
 	}
 
-	if (s->as.u.note.grace)
-		hltype = "ghl";
-	else {
-		switch (s->head) {
-		default:
-			hltype = "hl";
-			break;
-		case H_OVAL:
-			hltype = "hl1";
-			break;
-		case H_SQUARE:
-			hltype = "hl2";
-			break;
+	if (!s->as.u.note.invis) {
+		if (s->as.u.note.grace)
+			hltype = "ghl";
+		else {
+			switch (s->head) {
+			default:
+				hltype = "hl";
+				break;
+			case H_OVAL:
+				hltype = "hl1";
+				break;
+			case H_SQUARE:
+				hltype = "hl2";
+				break;
+			}
+		}
+		y = 3 * (s->pits[0] - 18);	/* lower helper lines */
+		i = -6;
+		switch (staff_tb[s->staff].clef.stafflines) {
+		case 0:
+		case 1: i = 6; break;
+		case 2:
+		case 3: i = 0; break;
+		}
+		for ( ; i >= y; i -= 6) {
+			PUT0(" ");
+			puty(i + staffb);
+			PUT1("%s", hltype);
+		}
+		y = 3 * (s->pits[s->nhd] - 18);	/* upper helper lines */
+		switch (staff_tb[s->staff].clef.stafflines) {
+		case 0:
+		case 1:
+		case 2: i = 18; break;
+		case 3: i = 24; break;
+		default: i = staff_tb[s->staff].clef.stafflines * 6; break;
+		}
+		for ( ; i <= y; i += 6) {
+			PUT0(" ");
+			puty(i + staffb);
+			PUT1("%s", hltype);
 		}
-	}
-	y = 3 * (s->pits[0] - 18);		/* lower helper lines */
-	i = -6;
-	switch (staff_tb[s->staff].clef.stafflines) {
-	case 0:
-	case 1: i = 6; break;
-	case 2:
-	case 3: i = 0; break;
-	}
-	for ( ; i >= y; i -= 6) {
-		PUT0(" ");
-		puty(i + staffb);
-		PUT1("%s", hltype);
-	}
-	y = 3 * (s->pits[s->nhd] - 18);		/* upper helper lines */
-	switch (staff_tb[s->staff].clef.stafflines) {
-	case 0:
-	case 1:
-	case 2: i = 18; break;
-	case 3: i = 24; break;
-	default: i = staff_tb[s->staff].clef.stafflines * 6; break;
-	}
-	for ( ; i <= y; i += 6) {
-		PUT0(" ");
-		puty(i + staffb);
-		PUT1("%s", hltype);
 	}
 
 	/* draw the other notes */
@@ -1401,7 +1501,7 @@ static void draw_note(float x,
 		if (m == ma)
 			continue;
 		PUT0(" ");
-		draw_basic_note(x, s, m);	/* draw the note heads */
+		draw_basic_note(x, s, m, y_tb);
 	}
 	PUT0("\n");
 }
@@ -1636,20 +1736,8 @@ if (two_staves) fprintf(stderr, "*** multi-staves slurs not treated\n");
 		x2 = k2->x;
 	else	x2 = realwidth;		/* (the slur starts on last note of the line) */
 #ifdef YSTEP
-#if 1
 	y1 = (float) (s > 0 ? k1->ymx + 2 : k1->ymn - 2);
 	y2 = (float) (s > 0 ? k2->ymx + 2 : k2->ymn - 2);
-#else
-	y1 = y_get(k1, s > 0, x1, 0, 0);
-	y2 = y_get(k2, s > 0, x2, 0, 0);
-	if (s > 0) {
-		y1 += 2;
-		y2 += 2;
-	} else {
-		y1 -= 2;
-		y2 -= 2;
-	}
-#endif
 #else
 	y1 = s > 0 ? k1->dc_top + 2 : k1->dc_bot - 2;
 	y2 = s > 0 ? k2->dc_top + 2 : k2->dc_bot - 2;
@@ -1669,7 +1757,7 @@ if (two_staves) fprintf(stderr, "*** multi-staves slurs not treated\n");
 			}
 		} else {
 			if (k1->stem < 0) {
-				x1 -= 2;
+				x1 -= 1;
 				if (k1->as.u.note.word_end
 				    && k1->nflags >= -1) {
 					if (k1->nflags > 0) {
@@ -1684,7 +1772,7 @@ if (two_staves) fprintf(stderr, "*** multi-staves slurs not treated\n");
 	if (k2->type == NOTE) {
 		if (s > 0) {
 			if (k2->stem > 0) {
-				x2 += 2;
+				x2 += 1;
 				if ((k2->sflags & S_WORD_ST)
 				    && k2->nflags >= -1)
 					y2 = k2->ys - 6;
@@ -1716,22 +1804,22 @@ if (two_staves) fprintf(stderr, "*** multi-staves slurs not treated\n");
 #ifdef YSTEP
 		if (k1->next->x < x1 + 48) {
 			if (s > 0) {
-				y = y_get(k1->next, 1, k1->next->x, 0, 0) - 2;
+				y = k1->next->ymx - 2;
 				if (y1 < y)
 					y1 = y;
 			} else {
-				y = y_get(k1->next, 0, k1->next->x, 0, 0) + 2;
+				y = k1->next->ymn + 2;
 				if (y1 > y)
 					y1 = y;
 			}
 		}
 		if (k2->prev->x > x2 - 48) {
 			if (s > 0) {
-				y = y_get(k2->prev, 1, k2->prev->x, 0, 0) - 2;
+				y = k2->prev->ymx - 2;
 				if (y2 < y)
 					y2 = y;
 			} else {
-				y = y_get(k2->prev, 0, k2->prev->x, 0, 0) + 2;
+				y = k2->prev->ymn + 2;
 				if (y2 > y)
 					y2 = y;
 			}
@@ -1774,13 +1862,14 @@ if (two_staves) fprintf(stderr, "*** multi-staves slurs not treated\n");
 	else	y2 = -6;
 
 	a = (y2 - y1) / (x2 - x1);		/* slur steepness */
-	if (a > SLUR_SLOPE)
-		a = SLUR_SLOPE;
-	else if (a < -SLUR_SLOPE)
-		a = -SLUR_SLOPE;
-	if (a * s > 0)
-		y1 = y2 - a * (x2 - x1);
-	else	y2 = y1 + a * (x2 - x1);
+	if (a > SLUR_SLOPE || a < -SLUR_SLOPE) {
+		if (a > SLUR_SLOPE)
+			a = SLUR_SLOPE;
+		else	a = -SLUR_SLOPE;
+		if (a * s > 0)
+			y1 = y2 - a * (x2 - x1);
+		else	y2 = y1 + a * (x2 - x1);
+	}
 
 	/* for big vertical jump, shift endpoints */
 	y = y2 - y1;
@@ -1807,30 +1896,11 @@ if (two_staves) fprintf(stderr, "*** multi-staves slurs not treated\n");
 	if (k2->as.u.note.grace)
 		x2 = k2->x + GSTEM_XOFF * 1.5;
 
-#ifdef YSTEP
-	/* adjust if some tuplet */
-	if (s > 0) {
-		y = y_get(k1, 1, x1 + 5, 0, 0);
-		if (y > y1)
-			y1 = y;
-		y = y_get(k2, 1, x2 - 5, 0, 0);
-		if (y > y2)
-			y2 = y;
-	} else {
-		y = y_get(k1, 0, x1 + 5, 0, 0);
-		if (y < y1)
-			y1 = y;
-		y = y_get(k2, 0, x2 - 5, 0, 0);
-		if (y < y2)
-			y2 = y;
-	}
-#endif
-
 	h = 0;
 	a = (y2 - y1) / (x2 - x1);
-	addy = y1 - a * x1;
-	if (k1 != k2)
-	    for (k = k1->next; k != 0 && k != k2 ; k = k->next) {
+	if (k1 != k2) {
+	    addy = y1 - a * x1;
+	    for (k = k1->next; k != k2 ; k = k->next) {
 		if (k->staff != upstaff)
 			continue;
 		switch (k->type) {
@@ -1870,9 +1940,10 @@ if (two_staves) fprintf(stderr, "*** multi-staves slurs not treated\n");
 			break;
 		    }
 		}
-		y1 += 0.45 * h;
-		y2 += 0.45 * h;
-		h *= 0.65;
+	    }
+	    y1 += 0.45 * h;
+	    y2 += 0.45 * h;
+	    h *= 0.65;
 	}
 
 	if (nn > 3)
@@ -2150,7 +2221,8 @@ static struct SYMBOL *draw_tuplet(struct SYMBOL *t)
 	some_slur = 0;
 	upstaff = t->staff;
 	for (s2 = t->next; s2 != 0; s2 = s2->next) {
-		if (s2->type != NOTE && s2->type != REST) {
+		if (s2->len == 0
+		    || (s2->type != NOTE && s2->type != REST)) {
 			if (s2->type == TUPLET) {
 				sy = draw_tuplet(s2);	/* nested tuplet */
 				if (sy->time > next->time)
@@ -2189,32 +2261,45 @@ static struct SYMBOL *draw_tuplet(struct SYMBOL *t)
 	} else {
 
 		/* search if a bracket is needed */
-		if ((t->u & 0x0f00) == 0x200)	/* if 'when' == always */
+		if ((t->u & 0x0f00) == 0x200	/* if 'when' == always */
+		    || s1->type != NOTE || s2->type != NOTE)
 			nb_only = 0;
 		else {
-			nb_only = s1->type == NOTE && s2->type == NOTE;
-			if (!((s1->sflags & S_WORD_ST) || (s1->prev->sflags & S_BEAM_BREAK)
-			       || s1->prev->nflags < s1->nflags)
-/*			       || s1->prev->as.u.note.word_end) */
-			    || !(s2->as.u.note.word_end || (s2->sflags & S_BEAM_BREAK)))
-				nb_only = 0;
-		}
-		if (nb_only)
-		    for (sy = s1; ; sy = sy->next) {
-			if (sy->len == 0) {	/* not a note or a rest */
-				if (sy->type != GRACE) {
+			nb_only = 1;
+			for (sy = s1; ; sy = sy->next) {
+				if (sy->type != NOTE && sy->type != REST) {
+					if (sy->type == GRACE)
+						continue;
 					nb_only = 0;
 					break;
 				}
 				if (sy == s2)
 					break;
-				continue;
+				if (sy->as.u.note.word_end) {
+					nb_only = 0;
+					break;
+				}
 			}
-			if (sy == s2)
-				break;
-			if (sy->as.u.note.word_end || (sy->sflags & S_BEAM_BREAK)) {
-				nb_only = 0;
-				break;
+			if (nb_only
+			    && !(s1->sflags & (S_WORD_ST | S_BEAM_BR1 | S_BEAM_BR2))) {
+				for (sy = s1->prev; sy != 0; sy = sy->prev) {
+					if (sy->len != 0
+					    && (sy->type == NOTE || sy->type == REST)) {
+						if (sy->nflags >= s1->nflags)
+							nb_only = 0;
+						break;
+					}
+				}
+			}
+			if (nb_only && !s2->as.u.note.word_end) {
+				for (sy = s2->next; sy != 0; sy = sy->next) {
+					if (sy->len != 0) {
+						if (!(sy->sflags & (S_BEAM_BR1 | S_BEAM_BR2))
+						    && sy->nflags >= s2->nflags)
+							nb_only = 0;
+						break;
+					}
+				}
 			}
 		}
 	}
@@ -2232,11 +2317,7 @@ static struct SYMBOL *draw_tuplet(struct SYMBOL *t)
 				for (sy = s1; ; sy = sy->next) {
 					yy = a * sy->x + b;
 #ifdef YSTEP
-#if 1
 					ym = sy->ymx;
-#else
-					ym = y_get(sy, 1, sy->x, 0, 0);
-#endif
 					if (ym > yy)
 						b += ym - yy;
 #else
@@ -2253,11 +2334,7 @@ static struct SYMBOL *draw_tuplet(struct SYMBOL *t)
 				for (sy = s1; ; sy = sy->next) {
 					yy = a * sy->x + b;
 #ifdef YSTEP
-#if 1
 					ym = sy->ymn;
-#else
-					ym = y_get(sy, 0, sy->x, 0, 0);
-#endif
 					if (ym < yy)
 						b += ym - yy;
 #else
@@ -2573,17 +2650,9 @@ static void draw_note_ties(struct SYMBOL *k1,
 			   int dotted,
 			   int job)
 {
-	int i, s0, s1, s;
-	int m1, m2, p1, p2, y1, y2;
+	int i, s, m1, m2, p1, p2, y1, y2;
 	float x1, x2, h;
 
-	s1 = 0;
-	if ((s0 = k1->multi) == 0
-	    && (s0 = k2->multi) == 0) {
-		if (k1->voice == k2->voice)
-			s1 = slur_direction(k1, k2);
-		else	s1 = 1;
-	}
 	for (i = 0; i < ntie; i++) {
 		m1 = mhead1[i];
 		p1 = k1->pits[m1];
@@ -2591,27 +2660,7 @@ static void draw_note_ties(struct SYMBOL *k1,
 		p2 = k2->pits[m2];
 		if (k1->as.u.note.ti1[m1] == SL_ABOVE)
 			s = 1;
-		else if (k1->as.u.note.ti1[m1] == SL_BELOW)
-			s = -1;
-		else if ((s = s0) == 0) {
-
-			/* try to have the same tie direction as the next one */
-			if (job != 2
-			    && k2->nhd != 0
-			    && m2 == k2->nhd
-			    && k2->as.u.note.ti1[m2])
-				s = 1;
-			else if (job == 1 && k2->nhd == 0)
-				s = k2->stem >= 0 ? -1 : 1;
-			else if (k1->nhd == 0)
-				s = s1;
-			else if (m1 == 0)	/* if bottom */
-				s = -1;
-			else if (m1 == k1->nhd)	/* if top */
-				s = 1;
-			else	s = s1;
-		}
-
+		else	s = -1;
 		x1 = k1->x;
 		h = k1->shhd[m1];		/* head shift */
 		if (s > 0) {
@@ -2704,9 +2753,6 @@ static void draw_note_ties(struct SYMBOL *k1,
 		slur_out(x1, staff_tb[k1->staff].y + y1,
 			 x2, staff_tb[k1->staff].y + y2,
 			 s, h, dotted, -1);
-#ifdef YSTEP
-		y_set(k1, s > 0, x1 + 3, x2 - x1 - 6, y1 + h);
-#endif
 	}
 }
 
@@ -2971,14 +3017,22 @@ static float draw_lyrics(struct VOICE_S *p_voice,
 		j = 0;
 /*fixme: may not be the current font*/
 		y -= cfmt.vocalfont.size;
+#if 1
+		if (y > -cfmt.vocalspace)
+#else
 		if (p_voice->staff == nstaff	/* if last staff */
 		    && y > -cfmt.vocalspace)
+#endif
 			y = -cfmt.vocalspace;
 	} else {
 		j = nly - 1;
 		nly = -1;
+#if 1
+		if (y < 24 + cfmt.vocalspace - cfmt.vocalfont.size)
+#else
 		if (p_voice->staff == nstaff	/* if last staff */
 		    && y < 24 + cfmt.vocalspace - cfmt.vocalfont.size)
+#endif
 			y = 24 + cfmt.vocalspace - cfmt.vocalfont.size;
 	}
 /*fixme: may not be the current font*/
@@ -3278,30 +3332,41 @@ void draw_sym_near(void)
 		}
 	}
 
-	draw_deco_near();
-
 #ifdef YSTEP
 	/* initialize the y offsets */
 	{
-		int staff;
+		int i, staff;
 
 		for (staff = 0; staff <= nstaff; staff++) {
-			memset(staff_tb[staff].top, 0, sizeof staff_tb[staff].top);
-			memset(staff_tb[staff].bot, 0, sizeof staff_tb[staff].bot);
+			for (i = 0; i < YSTEP; i++) {
+				staff_tb[staff].top[i] = 0;
+				staff_tb[staff].bot[i] = 24;
+			}
 		}
 	}
 #endif
 
+	set_tie_room();
+	draw_deco_near();
+
 	/* have room for the accidentals */
 	for (s = first_voice->sym; s != 0; s = s->ts_next) {
 		int y;
-
 #ifdef YSTEP
+		struct SYMBOL *g;
+
 		/* set the y offsets */
 		if (s->prev == 0)
 			continue;		/* skip the clefs */
 		if (s->len != 0 && s->as.u.note.invis)
 			continue;
+		if ((g = s->grace) != 0) {
+			for ( ; g != 0; g = g->next) {
+				y_set(s, 1, g->x - g->wl, g->wl + g->wr, g->ymx);
+				y_set(s, 0, g->x - g->wl, g->wl + g->wr, g->ymn);
+			}
+			continue;
+		}
 		y_set(s, 1, s->x - s->wl, s->wl + s->wr, s->ymx);
 		y_set(s, 0, s->x - s->wl, s->wl + s->wr, s->ymn);
 #endif
@@ -3382,42 +3447,6 @@ void draw_sym_near(void)
 	}
 #endif
 	draw_deco_note();
-
-	/* have room for the ties out of the staves */
-	for (p_voice = first_voice; p_voice; p_voice = p_voice->next) {
-		for (s = p_voice->sym->next; s != 0; s = s->ts_next) {
-			struct SYMBOL *s2;
-			float dx, dy;
-
-			if (!(s->sflags & S_TI1))
-				continue;
-			if (s->pits[0] < 20 && s->as.u.note.ti1[0])
-				;
-			else if (s->pits[s->nhd] > 24 && s->as.u.note.ti1[s->nhd])
-				;
-			else	continue;
-			s2 = s->next;
-			while (s2 != 0 && s2->type != NOTE)
-				s2 = s2->next;
-			if (s2 != 0) {
-				if (s2->staff != s->staff)
-					continue;
-				dx = s2->x - s->x - 10;
-			} else	dx = realwidth - s->x - 10;
-			if (dx < 100)
-				dy = 9;
-			else if (dx < 300)
-				dy = 12;
-			else	dy = 16;
-/*fixme:should check if tie above or below*/
-			if (s->pits[0] < 20)
-				y_set(s, 0, s->x + 5, dx,
-				      3 * (s->pits[0] - 18) - dy);
-			if (s->pits[s->nhd] > 24)
-				y_set(s, 1, s->x + 5, dx,
-				      3 * (s->pits[s->nhd] - 18) + dy);
-		}
-	}
 	draw_all_lyrics();
 	draw_deco_staff();
 }
@@ -3502,6 +3531,8 @@ void draw_symbols(struct VOICE_S *p_voice)
 			       sizeof p_voice->meter);
 			if (p_voice->second || staff_tb[s->staff].empty)
 				break;
+			if (cfmt.alignbars && s->staff != 0)
+				break;
 			set_sscale(s->staff);
 			draw_timesig(x, s);
 			break;
@@ -3514,16 +3545,12 @@ void draw_symbols(struct VOICE_S *p_voice)
 			draw_keysig(p_voice, x, s);
 			break;
 		case MREST:
-			if (p_voice->second)
-				break;
 			set_scale(s->voice);
 			PUT1("(%d)\n", s->as.u.bar.len);
 			putxy(x, staff_tb[s->staff].y);
 			PUT0("mrest\n");
 			break;
 		case MREP:
-			if (p_voice->second)
-				break;
 			set_scale(s->voice);
 			if (s->as.u.bar.len == 1) {
 				x = (s->prev->x + s->next->x) * 0.5;
@@ -3622,10 +3649,12 @@ void draw_vname(int first_line,
 	for (p_voice = first_voice; p_voice; p_voice = p_voice->next) {
 		scale = p_voice->scale;
 		if (scale != 1) {
-			PUT2("/scvo%d{gsave %.1f dup scale}def\n",
+			PUT2("/scvo%d{gsave %.2f dup scale}def\n",
 			     p_voice - voice_tb, scale);
 		}
 		staff = p_voice->staff;
+		if (staff_tb[staff].empty)
+			continue;
 		if (staff_tb[staff].brace_end)
 			staff--;
 		staff_p = &staff_d[staff];
@@ -3779,6 +3808,190 @@ static void set_sscale(int staff)
 		PUT1("scst%d ", staff);
 }
 
+/* -- set the tie directions for one voice -- */
+static void set_tie_dir(struct SYMBOL *sym)
+{
+	struct SYMBOL *s;
+	int i, ntie, dir, sec, pit;
+
+	for (s = sym; s != 0; s = s->next) {
+		if (!(s->sflags & S_TI1))
+			continue;
+
+		/* if other voice, set the ties in opposite direction */
+		if (s->multi != 0) {
+/*			struct SYMBOL *s2;
+
+			s2 = s->ts_next;
+			if (s2->time == s->time && s2->staff == s->staff) { */
+				dir = s->multi > 0 ? SL_ABOVE : SL_BELOW;
+				for (i = 0; i <= s->nhd; i++) {
+					if (s->as.u.note.ti1[i] == SL_AUTO)
+						s->as.u.note.ti1[i] = dir;
+				}
+				continue;
+/*			} */
+		}
+
+		/* if one note, set the direction according to the stem */
+		sec = ntie = 0;
+		pit = 128;
+		for (i = 0; i <= s->nhd; i++) {
+			if (s->as.u.note.ti1[i]) {
+				ntie++;
+				if (pit < 128
+				    && s->as.u.note.pits[i] <= pit + 1)
+					sec++;
+				pit = s->as.u.note.pits[i];
+			}
+		}
+		if (ntie <= 1) {
+			dir = s->stem < 0 ? SL_ABOVE : SL_BELOW;
+			for (i = 0; i <= s->nhd; i++) {
+				if (s->as.u.note.ti1[i]) {
+					if (s->as.u.note.ti1[i] == SL_AUTO)
+						s->as.u.note.ti1[i] = dir;
+					break;
+				}
+			}
+			continue;
+		}
+		if (sec == 0) {
+			if (ntie & 1) {
+/* in chords with an odd number of notes, the outer noteheads are paired off
+ * center notes are tied according to their position in relation to the
+ * center line */
+				ntie = ntie / 2 + 1;
+				dir = SL_BELOW;
+				for (i = 0; i <= s->nhd; i++) {
+					if (s->as.u.note.ti1[i]) {
+						if (--ntie == 0) {	/* central tie */
+							if (s->as.u.note.pits[i] >= 22)
+								dir = SL_ABOVE;
+						}
+						if (s->as.u.note.ti1[i] == SL_AUTO)
+							s->as.u.note.ti1[i] = dir;
+						if (ntie == 0)
+							dir = SL_ABOVE;
+					}
+				}
+				continue;
+			} else {
+/* even number of notes, ties divided in opposite directions */
+				ntie /= 2;
+				dir = SL_BELOW;
+				for (i = 0; i <= s->nhd; i++) {
+					if (s->as.u.note.ti1[i]) {
+						if (s->as.u.note.ti1[i] == SL_AUTO)
+							s->as.u.note.ti1[i] = dir;
+						if (--ntie == 0)
+							dir = SL_ABOVE;
+					}
+				}
+				continue;
+			}
+		}
+/*fixme: treat more than one second */
+/*		if (nsec == 1) {	*/
+/* When a chord contains the interval of a second, tie those two notes in
+ * opposition; then fill in the remaining notes of the chord accordingly */
+			pit = 128;
+			for (i = 0; i <= s->nhd; i++) {
+				if (s->as.u.note.ti1[i]) {
+					if (pit < 128
+					    && s->as.u.note.pits[i] <= pit + 1) {
+						ntie = i;
+						break;
+					}
+					pit = s->as.u.note.pits[i];
+				}
+			}
+			dir = SL_BELOW;
+			for (i = 0; i <= s->nhd; i++) {
+				if (s->as.u.note.ti1[i]) {
+					if (ntie == i)
+						dir = SL_ABOVE;
+					if (s->as.u.note.ti1[i] == SL_AUTO)
+						s->as.u.note.ti1[i] = dir;
+				}
+			}
+/*fixme..
+			continue;
+		}
+..*/
+/* if a chord contains more than one pair of seconds, the pair farthest
+ * from the center line receives the ties drawn in opposition */
+	}
+}
+
+/* -- have room for the ties out of the staves -- */
+static void set_tie_room(void)
+{
+	struct VOICE_S *p_voice;
+	struct SYMBOL *s, *s2;
+
+	for (p_voice = first_voice; p_voice; p_voice = p_voice->next) {
+		if ((s = p_voice->sym->next) == 0)
+			continue;
+		set_tie_dir(s);
+		for ( ; s != 0; s = s->next) {
+			float dx, y, dy;
+
+			if (!(s->sflags & S_TI1))
+				continue;
+			if (s->pits[0] < 20 && s->as.u.note.ti1[0] == SL_BELOW)
+				;
+			else if (s->pits[s->nhd] > 24
+				 && s->as.u.note.ti1[s->nhd] == SL_ABOVE)
+				;
+			else	continue;
+			s2 = s->next;
+			while (s2 != 0 && s2->type != NOTE)
+				s2 = s2->next;
+			if (s2 != 0) {
+				if (s2->staff != s->staff)
+					continue;
+				dx = s2->x - s->x - 10;
+			} else	dx = realwidth - s->x - 10;
+			if (dx < 100)
+				dy = 9;
+			else if (dx < 300)
+				dy = 12;
+			else	dy = 16;
+			if (s->pits[s->nhd] > 24) {
+				y = 3 * (s->pits[s->nhd] - 18) + dy;
+#ifdef YSTEP
+				if (s->ymx < y)
+					s->ymx = y;
+				if (s2 != 0 && s2->ymx < y)
+					s2->ymx = y;
+				y_set(s, 1, s->x + 5, dx, y);
+#else
+				if (s->dc_top < y)
+					s->dc_top = y;
+				if (s2 != 0 && s2->dc_top < y)
+					s2->dc_top = y;
+#endif
+			}
+			if (s->pits[0] < 20) {
+				y = 3 * (s->pits[0] - 18) - dy;
+#ifdef YSTEP
+				if (s->ymn > y)
+					s->ymn = y;
+				if (s2 != 0 && s2->ymn > y)
+					s2->ymn = y;
+				y_set(s, 0, s->x + 5, dx, y);
+#else
+				if (s->dc_bot > y)
+					s->dc_bot = y;
+				if (s2 != 0 && s2->dc_bot > y)
+					s2->dc_bot = y;
+#endif
+			}
+		}
+	}
+}
+
 /* -- draw the tin whistle tablature -- */
 void draw_whistle(void)
 {
