@@ -3,7 +3,7 @@
  *
  * This file is part of abcm2ps.
  *
- * Copyright (C) 1998-2005 Jean-Franois Moine
+ * Copyright (C) 1998-2006 Jean-Franois Moine
  * Adapted from abc2ps, Copyright (C) 1996,1997 Michael Methfessel
  *
  * This program is free software; you can redistribute it and/or modify
@@ -42,115 +42,119 @@ static float staffwidth;
 /* format table */
 static struct format {
 	char *name;
-	short type;
+	void *v;
+	char type;
 #define FORMAT_I 0	/* int */
 #define FORMAT_R 1	/* float */
 #define FORMAT_F 2	/* font spec */
 #define FORMAT_U 3	/* float with unit */
 #define FORMAT_B 4	/* boolean */
 #define FORMAT_S 5	/* string */
-	short subtype;		/* special cases - see code */
-	void *v;
+	char subtype;		/* special cases - see code */
+	short lock;
 } format_tb[] = {
-	{"aligncomposer", FORMAT_I, 0, &cfmt.aligncomposer},
-	{"autoclef", FORMAT_B, 0, &cfmt.autoclef},
-	{"annotationfont", FORMAT_F, 0, &cfmt.annotationfont},
-	{"barnumbers", FORMAT_I, 0, &cfmt.measurenb},
-	{"barsperstaff", FORMAT_I, 0, &cfmt.barsperstaff},
-	{"botmargin", FORMAT_U, 0, &cfmt.botmargin},
-	{"bstemdown", FORMAT_B, 0, &cfmt.bstemdown},
-	{"combinevoices", FORMAT_B, 0, &cfmt.combinevoices},
-	{"composerfont", FORMAT_F, 0, &cfmt.composerfont},
-	{"composerspace", FORMAT_U, 0, &cfmt.composerspace},
-	{"contbarnb", FORMAT_B, 0, &cfmt.contbarnb},
-	{"continueall", FORMAT_B, 0, &cfmt.continueall},
-	{"dateformat", FORMAT_S, 0, &cfmt.dateformat},
-	{"dynalign", FORMAT_B, 0, &cfmt.dynalign},
-	{"encoding", FORMAT_I, 1, &cfmt.encoding},
-	{"exprabove", FORMAT_B, 0, &cfmt.exprabove},
-	{"exprbelow", FORMAT_B, 0, &cfmt.exprbelow},
-	{"footer", FORMAT_S, 0, &cfmt.footer},
-	{"footerfont", FORMAT_F, 0, &cfmt.footerfont},
-	{"freegchord", FORMAT_B, 0, &cfmt.freegchord},
-	{"flatbeams", FORMAT_B, 0, &cfmt.flatbeams},
-	{"gchordbox", FORMAT_B, 0, &cfmt.gchordbox},
-	{"gchordfont", FORMAT_F, 3, &cfmt.gchordfont},
-	{"graceslurs", FORMAT_B, 0, &cfmt.graceslurs},
-	{"halfbeam", FORMAT_B, 0, &cfmt.halfbeam},
-	{"header", FORMAT_S, 0, &cfmt.header},
-	{"headerfont", FORMAT_F, 0, &cfmt.headerfont},
-	{"historyfont", FORMAT_F, 0, &cfmt.historyfont},
-	{"hyphencont", FORMAT_B, 0, &cfmt.hyphencont},
-	{"indent", FORMAT_U, 0, &cfmt.indent},
-	{"infofont", FORMAT_F, 0, &cfmt.infofont},
-	{"infoline", FORMAT_B, 0, &cfmt.infoline},
-	{"infospace", FORMAT_U, 0, &cfmt.infospace},
-	{"landscape", FORMAT_B, 0, &cfmt.landscape},
-	{"leftmargin", FORMAT_U, 0, &cfmt.leftmargin},
-	{"lineskipfac", FORMAT_R, 0, &cfmt.lineskipfac},
-	{"maxshrink", FORMAT_R, 0, &cfmt.maxshrink},
-	{"maxstaffsep", FORMAT_U, 0, &cfmt.maxstaffsep},
-	{"maxsysstaffsep", FORMAT_U, 0, &cfmt.maxsysstaffsep},
-	{"measurebox", FORMAT_B, 0, &cfmt.measurebox},
-	{"measurefirst", FORMAT_I, 2, &cfmt.measurefirst},
-	{"measurefont", FORMAT_F, 2, &cfmt.measurefont},
-	{"measurenb", FORMAT_I, 0, &cfmt.measurenb},
-	{"musiconly", FORMAT_B, 0, &cfmt.musiconly},
-	{"musicspace", FORMAT_U, 0, &cfmt.musicspace},
-	{"notespacingfactor", FORMAT_R, 1, &cfmt.notespacingfactor},
-	{"oneperpage", FORMAT_B, 0, &cfmt.oneperpage},
-	{"pageheight", FORMAT_U, 0, &cfmt.pageheight},
-	{"pagewidth", FORMAT_U, 0, &cfmt.pagewidth},
-	{"parskipfac", FORMAT_R, 0, &cfmt.parskipfac},
-	{"partsbox", FORMAT_B, 0, &cfmt.partsbox},
-	{"partsfont", FORMAT_F, 1, &cfmt.partsfont},
-	{"partsspace", FORMAT_U, 0, &cfmt.partsspace},
-	{"printparts", FORMAT_B, 0, &cfmt.printparts},
-	{"printtempo", FORMAT_B, 0, &cfmt.printtempo},
-	{"pslevel", FORMAT_I, 0, &cfmt.pslevel},
-	{"repeatfont", FORMAT_F, 0, &cfmt.repeatfont},
-	{"rightmargin", FORMAT_U, 0, &cfmt.rightmargin},
-	{"scale", FORMAT_R, 0, &cfmt.scale},
-	{"setdefl", FORMAT_B, 0, &cfmt.setdefl},
-	{"setfont-1", FORMAT_F, 0, &cfmt.font_tb[1]},
-	{"setfont-2", FORMAT_F, 0, &cfmt.font_tb[2]},
-	{"setfont-3", FORMAT_F, 0, &cfmt.font_tb[3]},
-	{"setfont-4", FORMAT_F, 0, &cfmt.font_tb[4]},
+	{"alignbars", &cfmt.alignbars, FORMAT_I, 0},
+	{"aligncomposer", &cfmt.aligncomposer, FORMAT_I, 0},
+	{"autoclef", &cfmt.autoclef, FORMAT_B, 0},
+	{"annotationfont", &cfmt.annotationfont, FORMAT_F, 0},
+	{"barnumbers", &cfmt.measurenb, FORMAT_I, 0},
+	{"barsperstaff", &cfmt.barsperstaff, FORMAT_I, 0},
+	{"botmargin", &cfmt.botmargin, FORMAT_U, 0},
+	{"bstemdown", &cfmt.bstemdown, FORMAT_B, 0},
+	{"combinevoices", &cfmt.combinevoices, FORMAT_B, 0},
+	{"comball", &cfmt.comball, FORMAT_B, 0},
+	{"composerfont", &cfmt.composerfont, FORMAT_F, 0},
+	{"composerspace", &cfmt.composerspace, FORMAT_U, 0},
+	{"contbarnb", &cfmt.contbarnb, FORMAT_B, 0},
+	{"continueall", &cfmt.continueall, FORMAT_B, 0},
+	{"dateformat", &cfmt.dateformat, FORMAT_S, 0},
+	{"dynalign", &cfmt.dynalign, FORMAT_B, 0},
+	{"encoding", &cfmt.encoding, FORMAT_I, 1},
+	{"exprabove", &cfmt.exprabove, FORMAT_B, 0},
+	{"exprbelow", &cfmt.exprbelow, FORMAT_B, 0},
+	{"footer", &cfmt.footer, FORMAT_S, 0},
+	{"footerfont", &cfmt.footerfont, FORMAT_F, 0},
+	{"freegchord", &cfmt.freegchord, FORMAT_B, 0},
+	{"flatbeams", &cfmt.flatbeams, FORMAT_B, 0},
+	{"gchordbox", &cfmt.gchordbox, FORMAT_B, 0},
+	{"gchordfont", &cfmt.gchordfont, FORMAT_F, 3},
+	{"graceslurs", &cfmt.graceslurs, FORMAT_B, 0},
+	{"header", &cfmt.header, FORMAT_S, 0},
+	{"headerfont", &cfmt.headerfont, FORMAT_F, 0},
+	{"historyfont", &cfmt.historyfont, FORMAT_F, 0},
+	{"hyphencont", &cfmt.hyphencont, FORMAT_B, 0},
+	{"indent", &cfmt.indent, FORMAT_U, 0},
+	{"infofont", &cfmt.infofont, FORMAT_F, 0},
+	{"infoline", &cfmt.infoline, FORMAT_B, 0},
+	{"infospace", &cfmt.infospace, FORMAT_U, 0},
+	{"landscape", &cfmt.landscape, FORMAT_B, 0},
+	{"leftmargin", &cfmt.leftmargin, FORMAT_U, 0},
+	{"lineskipfac", &cfmt.lineskipfac, FORMAT_R, 0},
+	{"maxshrink", &cfmt.maxshrink, FORMAT_R, 0},
+	{"maxstaffsep", &cfmt.maxstaffsep, FORMAT_U, 0},
+	{"maxsysstaffsep", &cfmt.maxsysstaffsep, FORMAT_U, 0},
+	{"measurebox", &cfmt.measurebox, FORMAT_B, 0},
+	{"measurefirst", &cfmt.measurefirst, FORMAT_I, 2},
+	{"measurefont", &cfmt.measurefont, FORMAT_F, 2},
+	{"measurenb", &cfmt.measurenb, FORMAT_I, 0},
+	{"musiconly", &cfmt.musiconly, FORMAT_B, 0},
+	{"musicspace", &cfmt.musicspace, FORMAT_U, 0},
+	{"notespacingfactor", &cfmt.notespacingfactor, FORMAT_R, 1},
+	{"oneperpage", &cfmt.oneperpage, FORMAT_B, 0},
+	{"pageheight", &cfmt.pageheight, FORMAT_U, 0},
+	{"pagewidth", &cfmt.pagewidth, FORMAT_U, 0},
+	{"parskipfac", &cfmt.parskipfac, FORMAT_R, 0},
+	{"partsbox", &cfmt.partsbox, FORMAT_B, 0},
+	{"partsfont", &cfmt.partsfont, FORMAT_F, 1},
+	{"partsspace", &cfmt.partsspace, FORMAT_U, 0},
+	{"printparts", &cfmt.printparts, FORMAT_B, 0},
+	{"printtempo", &cfmt.printtempo, FORMAT_B, 0},
+	{"pslevel", &cfmt.pslevel, FORMAT_I, 0},
+	{"repeatfont", &cfmt.repeatfont, FORMAT_F, 0},
+	{"rightmargin", &cfmt.rightmargin, FORMAT_U, 0},
+	{"scale", &cfmt.scale, FORMAT_R, 0},
+	{"setdefl", &cfmt.setdefl, FORMAT_B, 0},
+	{"setfont-1", &cfmt.font_tb[1], FORMAT_F, 0},
+	{"setfont-2", &cfmt.font_tb[2], FORMAT_F, 0},
+	{"setfont-3", &cfmt.font_tb[3], FORMAT_F, 0},
+	{"setfont-4", &cfmt.font_tb[4], FORMAT_F, 0},
 #if DFONT_MIN!= 5
 #	error Bad start of dynamic fonts
 #endif
-	{"shifthnote", FORMAT_B, 0, &cfmt.shifthnote},
-	{"slurheight", FORMAT_R, 0, &cfmt.slurheight},
-	{"splittune", FORMAT_B, 0, &cfmt.splittune},
-	{"squarebreve", FORMAT_B, 0, &cfmt.squarebreve},
-	{"staffsep", FORMAT_U, 0, &cfmt.staffsep},
-	{"staffwidth", FORMAT_U, 1, &staffwidth},
-	{"stemheight", FORMAT_R, 0, &cfmt.stemheight},
-	{"straightflags", FORMAT_B, 0, &cfmt.straightflags},
-	{"stretchlast", FORMAT_B, 0, &cfmt.stretchlast},
-	{"stretchstaff", FORMAT_B, 0, &cfmt.stretchstaff},
-	{"subtitlefont", FORMAT_F, 0, &cfmt.subtitlefont},
-	{"subtitlespace", FORMAT_U, 0, &cfmt.subtitlespace},
-	{"sysstaffsep", FORMAT_U, 0, &cfmt.sysstaffsep},
-	{"tempofont", FORMAT_F, 0, &cfmt.tempofont},
-	{"textfont", FORMAT_F, 0, &cfmt.textfont},
-	{"textoption", FORMAT_I, 4, &cfmt.textoption},
-	{"textspace", FORMAT_U, 0, &cfmt.textspace},
-	{"titlecaps", FORMAT_B, 0, &cfmt.titlecaps},
-	{"titlefont", FORMAT_F, 0, &cfmt.titlefont},
-	{"titleleft", FORMAT_B, 0, &cfmt.titleleft},
-	{"titlespace", FORMAT_U, 0, &cfmt.titlespace},
-	{"topmargin", FORMAT_U, 0, &cfmt.topmargin},
-	{"topspace", FORMAT_U, 0, &cfmt.topspace},
-	{"tuplets", FORMAT_I, 3, &cfmt.tuplets},
-	{"vocalabove", FORMAT_B, 0, &cfmt.vocalabove},
-	{"vocalfont", FORMAT_F, 0, &cfmt.vocalfont},
-	{"vocalspace", FORMAT_U, 0, &cfmt.vocalspace},
-	{"voicefont", FORMAT_F, 0, &cfmt.voicefont},
-	{"withxrefs", FORMAT_B, 0, &cfmt.withxrefs},
-	{"wordsfont", FORMAT_F, 0, &cfmt.wordsfont},
-	{"wordsspace", FORMAT_U, 0, &cfmt.wordsspace},
-	{"writehistory", FORMAT_B, 0, &cfmt.writehistory},
+	{"shifthnote", &cfmt.shifthnote, FORMAT_B, 0},
+	{"slurheight", &cfmt.slurheight, FORMAT_R, 0},
+	{"splittune", &cfmt.splittune, FORMAT_B, 0},
+	{"squarebreve", &cfmt.squarebreve, FORMAT_B, 0},
+	{"staffsep", &cfmt.staffsep, FORMAT_U, 0},
+	{"staffwidth", &staffwidth, FORMAT_U, 1},
+	{"stemheight", &cfmt.stemheight, FORMAT_R, 0},
+	{"straightflags", &cfmt.straightflags, FORMAT_B, 0},
+	{"stretchlast", &cfmt.stretchlast, FORMAT_B, 0},
+	{"stretchstaff", &cfmt.stretchstaff, FORMAT_B, 0},
+	{"subtitlefont", &cfmt.subtitlefont, FORMAT_F, 0},
+	{"subtitlespace", &cfmt.subtitlespace, FORMAT_U, 0},
+	{"sysstaffsep", &cfmt.sysstaffsep, FORMAT_U, 0},
+	{"tempofont", &cfmt.tempofont, FORMAT_F, 0},
+	{"textfont", &cfmt.textfont, FORMAT_F, 0},
+	{"textoption", &cfmt.textoption, FORMAT_I, 4},
+	{"textspace", &cfmt.textspace, FORMAT_U, 0},
+	{"titlecaps", &cfmt.titlecaps, FORMAT_B, 0},
+	{"titlefont", &cfmt.titlefont, FORMAT_F, 0},
+	{"titleformat", &cfmt.titleformat, FORMAT_S, 0},
+	{"titleleft", &cfmt.titleleft, FORMAT_B, 0},
+	{"titlespace", &cfmt.titlespace, FORMAT_U, 0},
+	{"timewarn", &cfmt.timewarn, FORMAT_B, 0},
+	{"topmargin", &cfmt.topmargin, FORMAT_U, 0},
+	{"topspace", &cfmt.topspace, FORMAT_U, 0},
+	{"tuplets", &cfmt.tuplets, FORMAT_I, 3},
+	{"vocalabove", &cfmt.vocalabove, FORMAT_B, 0},
+	{"vocalfont", &cfmt.vocalfont, FORMAT_F, 0},
+	{"vocalspace", &cfmt.vocalspace, FORMAT_U, 0},
+	{"voicefont", &cfmt.voicefont, FORMAT_F, 0},
+	{"withxrefs", &cfmt.withxrefs, FORMAT_B, 0},
+	{"wordsfont", &cfmt.wordsfont, FORMAT_F, 0},
+	{"wordsspace", &cfmt.wordsspace, FORMAT_U, 0},
+	{"writehistory", &cfmt.writehistory, FORMAT_B, 0},
 	{0, 0, 0, 0}		/* end of table */
 };
 
@@ -319,7 +323,6 @@ void set_format(void)
 	f->maxshrink = 0.65;
 	f->stretchstaff = 1;
 	f->graceslurs = 1;
-	f->halfbeam = 1;
 	f->lineskipfac = 1.1;
 	f->parskipfac = 0.4;
 	f->measurenb = -1;
@@ -528,17 +531,15 @@ static void g_fspc(char *p,
 }
 
 /* -- parse a format line -- */
-/* return:
- *	0: command line format may be modified
- *	1: format not modified */
-int interpret_format_line(char *w,		/* keyword */
-			  char *p)		/* argument */
+void interpret_fmt_line(char *w,		/* keyword */
+			char *p,		/* argument */
+			int lock)
 {
 	struct format *fd;
 
 	if (strcmp(w, "deco") == 0) {
 		deco_add(p);
-		return 1;
+		return;
 	}
 	if (!strcmp(w, "font")) {
 		int fnum, encoding;
@@ -550,23 +551,26 @@ int interpret_format_line(char *w,		/* keyword */
 		else	encoding = get_encoding(p);
 		fnum = add_font(fname, encoding);
 		used_font[fnum] = 1;
-		return 0;
+		return;
 	}
 	if (strcmp(w, "format") == 0) {
 		if (read_fmt_file(p) < 0)
 			error(1, 0, "No such format file '%s'", p);
-		return 1;
+		return;
 	}
 	if (strcmp(w, "postscript") == 0) {
 		if (!file_initialized)
 			user_ps_add(p);
-		return 1;
+		return;
 	}
 	for (fd = format_tb; fd->name; fd++)
 		if (strcmp(w, fd->name) == 0)
 			break;
 	if (fd->name == 0)
-		return 1;
+		return;
+	if (fd->lock && !lock)
+		return;
+	fd->lock |= lock;
 	switch (fd->type) {
 	case FORMAT_I:
 		if (fd->subtype == 3) {
@@ -577,7 +581,7 @@ int interpret_format_line(char *w,		/* keyword */
 				error(1, 0,
 				      "Bad 'tuplets' value '%s' - ignored",
 				      p);
-				break;
+				return;
 			}
 			cfmt.tuplets = (i1 << 8) | (i2 << 4) | i3;
 			break;
@@ -626,6 +630,7 @@ int interpret_format_line(char *w,		/* keyword */
 			if (cfmt.notespacingfactor <= 0) {
 				fprintf(stderr,
 					"Bad value for 'notespacingfactor'\n");
+				cfmt.notespacingfactor = 1;
 				break;
 			}
 			dot_space = sqrt(cfmt.notespacingfactor);
@@ -689,7 +694,6 @@ int interpret_format_line(char *w,		/* keyword */
 		break;
 	    }
 	}
-	return 0;
 }
 
 /* -- remove the spaces and comments of a format line -- */
@@ -723,6 +727,19 @@ static char *clean_line(char *p)
 	return q;
 }
 
+/* -- lock a format -- */
+void lock_fmt(void *fmt)
+{
+	struct format *fd;
+
+	for (fd = format_tb; fd->name; fd++)
+		if (fd->v == fmt)
+			break;
+	if (fd->name == 0)
+		return;
+	fd->lock = 1;
+}
+
 /* -- read a format file -- */
 int read_fmt_file(char *filename)
 {
@@ -752,12 +769,21 @@ int read_fmt_file(char *filename)
 				p = line;
 				if (!fgets(p, sizeof line, fp))
 					break;
+#if 1
+				p[strlen(p) - 1] = '\0';
+#else
 				p = clean_line(p);
+#endif
 				if (*p == '\0')
 					continue;
 				if (strcmp(p, "endps") == 0)
 					break;
-				interpret_format_line("postscript", p);
+#if 1
+				if (!file_initialized)
+					user_ps_add(p);
+#else
+				interpret_fmt_line("postscript", p, 0);
+#endif
 			}
 			continue;
 		}
@@ -771,7 +797,7 @@ int read_fmt_file(char *filename)
 			while (isspace((unsigned char) *p))
 				p++;
 		}
-		interpret_format_line(q, p);
+		interpret_fmt_line(q, p, 0);
 	}
 	fclose(fp);
 	return 0;
