@@ -3,7 +3,7 @@
  *
  * This file is part of abcm2ps.
  *
- * Copyright (C) 1998-2005 Jean-Franois Moine
+ * Copyright (C) 1998-2006 Jean-Franois Moine
  * Adapted from abc2ps, Copyright (C) 1996,1997 Michael Methfessel
  *
  * This program is free software; you can redistribute it and/or modify
@@ -67,7 +67,7 @@ static struct {
 static void set_head_directions(struct SYMBOL *s)
 {
 	int i, i1, i2, i3, n, sig, d, shift;
-	int p1, p2, p3, m, nac;
+	int p1, p2, p3, ps, m, nac;
 	float dx, dx1, dx2, dx3, shmin, shmax;
 	unsigned char ax_tb[MAXHD], ac_tb[MAXHD];
 	static float dx_tb[4] = {
@@ -143,15 +143,15 @@ static void set_head_directions(struct SYMBOL *s)
 	m = n;
 	p2 = i2 = 0;			/* (compiler warning) */
 	dx2 = 0;
-	p3 = 255;
+	ps = 255;
 	for (i = 0; i < nac; i++) {
 		i1 = ax_tb[i];
 		p1 = s->pits[i1];
 		if (m >= 0) {		/* see if any head shift */
-			if (p3 - s->pits[i1] >= 4) {
+			if (ps - s->pits[i1] >= 4) {
 				for (m--; m >= 0; m--) {
 					if (s->shhd[m] < 0) {
-						p3 = s->pits[m];
+						ps = s->pits[m];
 						break;
 					}
 				}
@@ -159,7 +159,7 @@ static void set_head_directions(struct SYMBOL *s)
 		}
 		dx1 = dx;
 		if (m >= 0 && s->shhd[m] < 0
-		    && p3 - p1 < 4 && p3 - p1 > -4)
+		    && ps - p1 < 4 && ps - p1 > -4)
 			dx1 -= s->shhd[m];
 		if (s->as.u.note.grace)
 			dx1 *= 0.7;
@@ -237,59 +237,196 @@ static struct SYMBOL *delsym(struct SYMBOL *s)
 	return s->next;
 }
 
-/* -- try to combine voices */
-static int combine_voices(struct SYMBOL *s)
+/* -- check if voice combine may occur -- */
+static int may_combine(struct SYMBOL *s)
 {
 	struct SYMBOL *s2;
+	int nhd2;
 
-	s2 = s->ts_next;
-	if (s->type != s2->type)	/* keep note + rest */
+	if ((s2 = s->ts_next) == 0)
 		return 0;
-	if (s->time != s2->time
-	    || s->len != s2->len	/* (select note or rest) */
+	if (s2->voice == s->voice
+	    || s2->staff != s->staff
+	    || s2->time != s->time
+	    || s2->len != s->len
 	    || s2->ly != 0
+	    || ((s2->sflags ^ s->sflags) & S_WORD_ST)
+	    || s2->as.u.note.word_end != s->as.u.note.word_end
 	    || (s2->sflags & (S_SL1 | S_SL2))
 	    || s2->as.u.note.slur_st != 0
-	    || s2->as.u.note.slur_end != 0
-	    || s->pits[0] <= s2->pits[s2->nhd] + 1
-	    || s->nhd + s2->nhd + 1 >= MAXHD)
+	    || s2->as.u.note.slur_end != 0)
+		return 0;
+	nhd2 = s2->nhd;
+	if (s->nhd + nhd2 + 1 >= MAXHD)
+		return 0;
+	if (!cfmt.comball && s->pits[0] <= s2->pits[nhd2] + 1)
 		return 0;
-
 	if (s2->as.u.note.dc.n != 0) {
 		if (s2->as.u.note.dc.h != s2->as.u.note.dc.h
 		    || memcmp(&s->as.u.note.dc, &s2->as.u.note.dc,
 				sizeof s->as.u.note.dc) != 0)
 			return 0;
 	}
+	return 1;
+}
+
+/* -- combine 2 voices -- */
+static void do_combine(struct SYMBOL *s)
+{
+	struct SYMBOL *s2;
+	int nhd, nhd2;
+
+	nhd = s->nhd;
+	s2 = s->ts_next;
+	nhd2 = s2->nhd;
+	if (s->type != s2->type) {	/* if note and rest */
+		if (s2->type == REST) {
+			delsym(s2);	/* remove the rest */
+			return;
+		}
+		s->type = NOTE;		/* copy the note into the rest */
+		nhd = -1;
+		s->pits[0] = 127;
+	}
 
 	/* combine the voices */
-	memcpy(&s->pits[s2->nhd + 1], s->pits,
-		sizeof s->pits[0] * (s->nhd + 1));
-	memcpy(s->pits, s2->pits,
-		sizeof s->pits[0] * (s2->nhd + 1));
+	if (s->pits[0] >= s2->pits[nhd2]) {	/* standard case */
+		if (s->pits[0] == s2->pits[nhd2])
+			nhd2--;
+		memcpy(&s->pits[nhd2 + 1], s->pits,
+			sizeof s->pits[0] * (nhd + 1));
+		memcpy(s->pits, s2->pits,
+			sizeof s->pits[0] * (nhd2 + 1));
+
 #define COMBINEV(f)\
-    memcpy(&s->as.u.note.f[s2->nhd + 1], s->as.u.note.f,\
-	sizeof s->as.u.note.f[0] * (s->nhd + 1));\
+    memcpy(&s->as.u.note.f[nhd2 + 1], s->as.u.note.f,\
+	sizeof s->as.u.note.f[0] * (nhd + 1));\
     memcpy(s->as.u.note.f, s2->as.u.note.f,\
-	sizeof s->as.u.note.f[0] * (s2->nhd + 1))
-	COMBINEV(pits);
-	COMBINEV(lens);
-	COMBINEV(accs);
-	COMBINEV(sl1);
-	COMBINEV(sl2);
-	COMBINEV(ti1);
-	COMBINEV(decs);
+	sizeof s->as.u.note.f[0] * (nhd2 + 1))
+
+		COMBINEV(pits);
+		COMBINEV(lens);
+		COMBINEV(accs);
+		COMBINEV(sl1);
+		COMBINEV(sl2);
+		COMBINEV(ti1);
+		COMBINEV(decs);
 #undef COMBINEV
-	s->nhd += s2->nhd + 1;
-	s->ymn = 3 * (s->pits[0] - 18) - 2;
+		nhd += nhd2 + 1;
+		s->nhd = nhd;
+		s->ymn = 3 * (s->pits[0] - 18) - 2;
 /*fixme:should recalculate yav*/
+	} else {				/* voice inverted */
+/*fixme:KO if imbricated chords*/
+		memcpy(&s->pits[nhd + 1], s2->pits,
+			sizeof s->pits[0] * (nhd2 + 1));
+
+#define COMBINEV(f)\
+    memcpy(&s->as.u.note.f[nhd + 1], s2->as.u.note.f,\
+	sizeof s->as.u.note.f[0] * (nhd2 + 1));\
+
+		COMBINEV(pits);
+		COMBINEV(lens);
+		COMBINEV(accs);
+		COMBINEV(sl1);
+		COMBINEV(sl2);
+		COMBINEV(ti1);
+		COMBINEV(decs);
+#undef COMBINEV
+		nhd += nhd2 + 1;
+		s->nhd = nhd;
+/*fixme:should recalculate yav*/
+		s->ymx = 3 * (s->pits[nhd] - 18) + 2;
+	}
 	if (s->as.u.note.ti1[0] == SL_AUTO)	/* force the tie directions */
 		s->as.u.note.ti1[0] = SL_BELOW;
-	if (s->as.u.note.ti1[s->nhd] == SL_AUTO)
-		s->as.u.note.ti1[s->nhd] = SL_ABOVE;
+	if (s->as.u.note.ti1[nhd] == SL_AUTO)
+		s->as.u.note.ti1[nhd] = SL_ABOVE;
 
-	delsym(s2);			/* remove the symbol */
-	return 1;
+	delsym(s2);			/* remove the next symbol */
+}
+
+/* -- try to combine voices */
+static void combine_voices(void)
+{
+	struct SYMBOL *s, *s2;
+	int r;
+
+	for (s = first_voice->sym; s != 0; s = s->ts_next) {
+		switch (s->type) {
+		case TUPLET:
+			if ((s2 = s->ts_next) == 0)
+				break;
+			r = s->as.u.tuplet.r_plet;
+			if (s2->type != TUPLET
+			    || s2->as.u.tuplet.r_plet != r
+			    || s2->as.u.tuplet.p_plet != s->as.u.tuplet.p_plet
+			    || s2->as.u.tuplet.q_plet != s->as.u.tuplet.q_plet)
+				continue;
+			s2 = s;
+			for (;;) {
+				s2 = s2->next;
+				while (s2->type != NOTE && s2->type != REST)
+					s2 = s2->next;
+				if (!may_combine(s2))
+					break;
+				if (--r <= 0)
+					break;
+			}
+			if (r > 0)
+				continue;
+			delsym(s->ts_next);	/* remove the tuplet */
+			r = s->as.u.tuplet.r_plet;
+			s2 = s;
+			for (;;) {
+				s2 = s2->next;
+				while (s2->type != NOTE && s2->type != REST)
+					s2 = s2->next;
+				do_combine(s2);
+				if (--r <= 0)
+					break;
+			}
+			continue;
+		default:
+			continue;
+		case NOTE:
+		case REST:
+			break;
+		}
+		if (s->ts_next == 0)
+			break;
+		if ((s->sflags & S_IN_TUPLET) || s->prev->type == TUPLET)
+			continue;
+		if (!(s->sflags & S_WORD_ST))
+			continue;
+		if (!s->as.u.note.word_end) {
+			s2 = s;
+			for (;;) {
+				if (!may_combine(s2)) {
+					s2 = 0;
+					break;
+				}
+				if (s2->as.u.note.word_end)
+					break;
+				do {
+					s2 = s2->next;
+				} while (s2->type != NOTE && s2->type != REST);
+			}
+			if (s2 == 0)
+				continue;
+			s2 = s;
+			for (;;) {
+				do_combine(s2);
+				if (s2->as.u.note.word_end)
+					break;
+				do {
+					s2 = s2->next;
+				} while (s2->type != NOTE && s2->type != REST);
+			}
+		}
+		if (may_combine(s))
+			do_combine(s);
+	}
 }
 
 /* -- insert a clef change (treble or bass) before a symbol -- */
@@ -393,12 +530,14 @@ static void set_clef(int staff)
 			s2 = s->ts_prev;
 			if (s2->time == time
 			    && s2->staff == staff
+			    && s2->type == NOTE
 			    && s2->pits[0] >= 19)	/* F */
 				continue;
 			s2 = s->ts_next;
 			if (s2 != 0
 			    && s2->staff == staff
 			    && s2->time == time
+			    && s2->type == NOTE
 			    && s2->pits[0] >= 19)	/* F */
 				continue;
 		} else {
@@ -408,12 +547,14 @@ static void set_clef(int staff)
 			s2 = s->ts_prev;
 			if (s2->time == time
 			    && s2->staff == staff
+			    && s2->type == NOTE
 			    && s2->pits[0] <= 13)	/* G, */
 				continue;
 			s2 = s->ts_next;
 			if (s2 != 0
 			    && s2->staff == staff
 			    && s2->time == time
+			    && s2->type == NOTE
 			    && s2->pits[0] <= 13)	/* G, */
 				continue;
 		}
@@ -533,7 +674,7 @@ static void set_clef(int staff)
 static void def_tssym(void)
 {
 	struct SYMBOL *s, *t, *prev_sym;
-	int time, bars, seq;
+	int time, bars, seq, fl;
 	struct VOICE_S *p_voice;
 
 	for (p_voice = first_voice; p_voice; p_voice = p_voice->next) {
@@ -562,40 +703,49 @@ static void def_tssym(void)
 			if ((s = p_voice->s_anc) == 0
 			    || s->time > time)
 				continue;
-			if (s->time < time
-			    || s->seq < seq) {
+			if (s->time < time) {
 				time = s->time;
 				seq = s->seq;
-			}
+			} else if (s->seq < seq)
+				seq = s->seq;
 		}
 		if (seq < 0)
 			break;		/* echu (finished) */
 
 		/* warn about incorrect number of notes / measures */
-		t = 0;
+		fl = 0;
 		for (p_voice = first_voice; p_voice; p_voice = p_voice->next) {
 			if ((s = p_voice->s_anc) != 0
 			    && s->time == time
 			    && s->seq == seq) {
 				p_voice->selected = 1;
-				if (s->type == BAR
-				    && s->as.u.bar.type != B_INVIS
-				    && t == 0)
-					t = s;
+				switch (s->type) {
+				case BAR:
+					if (s->as.u.bar.type != B_INVIS)
+						fl |= 1;
+					break;
+				case MREST:
+					fl |= 2;
+					break;
+				}
+				
 			} else	p_voice->selected = 0;
 		}
 
-		if (t != 0) {		/* if any bar */
+		if (fl & 1) {		/* if any bar */
 			int ko = 0;
 
 			bars++;
 			for (p_voice = first_voice; p_voice; p_voice = p_voice->next) {
-				if ((s = p_voice->s_anc) == 0
-				    || s->type == BAR)
+				if ((s = p_voice->s_anc) == 0)
 					continue;
 				if (s->time != time) {	/* bad time */
+					if (s->prev == 0
+					    || s->prev->time + s->prev->len
+						!=  s->time)
+						continue;	/* normal time skip */
 					error(1, s,
-					      "Too many notes in measure %d for voice %s",
+					      "Bad bar at measure %d for voice %s",
 					      bars, p_voice->name);
 					ko = 1;
 					break;
@@ -618,6 +768,65 @@ static void def_tssym(void)
 				continue;
 			}
 		}
+		if ((fl & 2)		/* if any mrest alone in a staff */
+		    && first_voice->next != 0) {	/* and many voices */
+			fl = 0;
+			for (p_voice = first_voice; p_voice; p_voice = p_voice->next) {
+				if ((s = p_voice->s_anc) == 0)
+					continue;
+				if (s->type != MREST) {
+					fl = 1;
+					break;
+				}
+			}
+			if (fl) {	/* if not only multi measure rests */
+				for (p_voice = first_voice; p_voice; p_voice = p_voice->next) {
+					int i, dt, time2;
+
+					if ((s = p_voice->s_anc) == 0
+					    || s->type != MREST)
+						continue;
+					p_voice->last_symbol = s->prev;
+					i = s->as.u.bar.len;
+					dt = s->len / i;
+					time2 = s->time;
+					t = add_sym(p_voice, REST);
+					t->as.type = ABC_T_REST;
+					t->as.linenum = s->as.linenum;
+					t->as.colnum = s->as.colnum;
+					t->len = t->as.u.note.lens[0] = dt;
+					t->head = H_FULL;
+					t->nflags = -2;
+					t->time = time2;
+					if (s->as.u.bar.dc.n > 0)
+						memcpy(&t->as.u.note.dc,
+							&s->as.u.bar.dc,
+							sizeof t->as.u.note.dc);
+					p_voice->s_anc = t;
+					time2 += dt;
+					while (--i > 0) {
+						t = add_sym(p_voice, BAR);
+						t->as.type = ABC_T_BAR;
+						t->as.u.bar.type = B_SINGLE;
+						t->as.linenum = s->as.linenum;
+						t->as.colnum = s->as.colnum;
+						t->time = time2;
+						t = add_sym(p_voice, REST);
+						t->as.type = ABC_T_REST;
+						t->as.linenum = s->as.linenum;
+						t->as.colnum = s->as.colnum;
+						t->len = t->as.u.note.lens[0] = dt;
+						t->head = H_FULL;
+						t->nflags = -2;
+						t->time = time2;
+						time2 += dt;
+					}
+					t = p_voice->last_symbol;
+					if ((t->next = s->next) != 0)
+						s->next->prev = t;
+				}
+			}
+		}
 
 		/* set the time linkage */
 		for (p_voice = first_voice; p_voice; p_voice = p_voice->next) {
@@ -657,7 +866,7 @@ static void set_float(void)
 				staff_chg = 0;
 				continue;
 			}
-			if (s->pits[s->nhd] <= 13) {	/* G, */
+			if (s->pits[s->nhd] <= 12) {	/* F, */
 				staff_chg = 1;
 				s->staff++;
 				continue;
@@ -667,6 +876,8 @@ static void set_float(void)
 				if (s1->staff != staff
 				    || s1->voice == s->voice)
 					break;
+/*fixme:test again*/
+if (s1->type == NOTE)
 /*fixme:test*/
 				if (/*s1->type == NOTE
 				    &&*/ s1->pits[0] < up)
@@ -686,6 +897,8 @@ static void set_float(void)
 				if (s1->staff != staff + 1
 				    || s1->voice == s->voice)
 					break;
+/*fixme:test again*/
+if (s1->type == NOTE)
 /*fixme:test*/
 				if (/*s1->type == NOTE
 				    &&*/ s1->pits[s1->nhd] > down)
@@ -817,13 +1030,11 @@ static void set_pitch(void)
 					for (m = g->nhd; m >= 0; m--)
 						g->pits[m] += delta;
 				}
-				g->ymn = 3 * (g->pits[0] - 18);
-				g->ymx = 3 * (g->pits[g->nhd] - 18);
+				g->ymn = 3 * (g->pits[0] - 18) - 2;
+				g->ymx = 3 * (g->pits[g->nhd] - 18) + 2;
 			}
 			continue;
 		case MREST:
-			if (voice_tb[s->voice].second)
-				continue;	/* not displayed */
 			s->ymx = 24 + 15;
 			s->ymn = -2;
 #ifndef YSTEP
@@ -902,24 +1113,15 @@ static void set_multi(void)
 			vtb[i].st1 = vtb[i].st2 = -1;
 
 		/* go to the next bar and get the max/min offsets */
+/*fixme: the stem height is not calculated yet*/
 		for (t = s;
 		     t != 0 && t->type != BAR;
 		     t = t->ts_next) {
 			if (t->len == 0		/* not a note or a rest */
 			    || t->as.u.note.invis)
 				continue;
-
-			/* try to combine the voices if asked */
 			staff = t->staff;
 			voice = t->voice;
-			if (cfmt.combinevoices
-			    && t->ts_next != 0
-			    && t->ts_next->voice != voice
-			    && t->ts_next->staff == staff) {
-				if (combine_voices(t))
-					continue;
-			}
-			
 			if (vtb[voice].st1 < 0)
 				vtb[voice].st1 = staff;
 			else if (vtb[voice].st1 == staff)
@@ -978,9 +1180,11 @@ static void set_multi(void)
 		     s = s->ts_next) {
 			int us, ls;
 
+			if (s->len == 0)	/* if not note or rest */
+				continue;
 			staff = s->staff;
 			voice = s->voice;
-			if (vtb[voice].st2 >= 0) {
+			if (s->multi == 0 && vtb[voice].st2 >= 0) {
 				if (staff == vtb[voice].st1)
 					s->multi = -1;
 				else if (staff == vtb[voice].st2)
@@ -999,11 +1203,11 @@ static void set_multi(void)
 				continue;
 			}
 			rvoice = vtb[voice].range;
-			for (i = 0; i < 4; i++) {
+			for (i = stb[staff].nvoice; i >= 0; i--) {
 				if (stb[staff].st[i].voice == rvoice)
 					break;
 			}
-			if (i == 4)
+			if (i < 0)
 				continue;		/* voice ignored */
 			if (s->multi == 0) {
 				if (i == stb[staff].nvoice)
@@ -1032,7 +1236,8 @@ static void set_multi(void)
 					}
 				}
 			}
-			if (s->type != REST || s->len == 0)
+			if (s->type != REST || s->len == 0
+			    || s->as.u.note.invis)
 				continue;
 
 			/* set the rest vertical offset */
@@ -1042,23 +1247,20 @@ static void set_multi(void)
 			ls = rest_sp[C_XFLAGS - s->nflags].l;
 
 			if (i == 0) {			/* first voice */
-				int nrest, ymx;
+				int alone, ymx;
 
-				t = s;
-				nrest = ((stb[staff].nvoice + 1) << 1) + 1;
-				do {
-					if (t->as.u.note.invis)
-						nrest -= 2;
-					else	nrest -= 3;
-					if (nrest == 0)
+				alone = 1;
+				for (t = s->ts_next; t != 0; t = t->ts_next) {
+					if (t->staff != s->staff
+					    || t->time != s->time)
 						break;
-					if (t->ts_next == 0)
+					if (t->type != REST
+					    || !t->as.u.note.invis) {
+						alone = 0;
 						break;
-					t = t->ts_next;
-				} while (t->type == REST
-					 && t->staff == s->staff
-					 && t->time == s->time);
-				if (nrest == 0) {
+					}
+				}
+				if (alone) {
 					s = t;
 					continue;
 				}
@@ -1098,6 +1300,7 @@ static void set_multi(void)
 					&& !s->ts_prev->as.u.note.invis
 					&& s->ts_prev->ymn < s->y + us))
 					s->shhd[0] = 10;
+					s->xmx = 10;
 			}
 			s->ymx = s->y + us;
 			if (s->ymx > stb[staff].st[i].ymx)
@@ -1242,6 +1445,10 @@ static void set_global(void)
 				break;
 			}
 		}
+		while (sym != s) {
+			sym->pits[0] = pitch;
+			sym = sym->next;
+		}
 		start_flag = 1;
 		lastnote = 0;
 		for (s = sym; s != 0; s = s->next) {
@@ -1466,12 +1673,15 @@ static float set_staff(void)
 					p_delta->mtop = v;
 			}
 		}
-		if (!staff_tb[nstaff].empty) {
-			p_delta = &delta_tb[nstaff];
-			for (i = 0; i < YSTEP; i++) {
-				v = staff_tb[nstaff].bot[i];
-				if (mbot > v)
-					mbot = v;
+		for (staff = nstaff; staff >= 0; staff--) {
+			if (!staff_tb[staff].empty) {
+				p_delta = &delta_tb[staff];
+				for (i = 0; i < YSTEP; i++) {
+					v = staff_tb[staff].bot[i];
+					if (mbot > v)
+						mbot = v;
+				}
+				break;
 			}
 		}
 	}
@@ -1615,37 +1825,44 @@ static float set_staff(void)
 static void set_beams(struct SYMBOL *sym)
 {
 	struct SYMBOL *s, *t, *prev_note;
-	int beam, laststem, lasty, bar_time;
+	int beam, laststem, lasty;
+#if 0
+	int bar_time = 0;
+#endif
 
 	beam = 0;
 	laststem = -1;
 	lasty = 0;
 	prev_note = 0;
-	bar_time = 0;
 	for (s = sym; s != 0; s = s->next) {
 		if (s->type != NOTE) {
 			struct SYMBOL *g;
 
 			if ((g = s->grace) != 0) {
-				if (s->multi == 0)
-					s->multi = 1;
-				s->stem = s->multi;
+				if (s->stem == 0
+				    && (s->stem = s->multi) == 0)
+					s->stem = 1;
 				for (; g != 0; g = g->next)
-					g->multi = g->stem = s->multi;
-			} else if (s->type == BAR)
+					g->multi = g->stem = s->stem;
+			}
+#if 0
+			  else if (s->type == BAR)
 				bar_time = s->time;
+#endif
 			continue;
 		}
 
+#if 0
 		/* set beam breaks on demi-semi-quaver sequences */
 		if (beam && cfmt.halfbeam
 		    && s->len <= SEMIQUAVER / 2
 		    && prev_note->len <= SEMIQUAVER / 2
 		    && (s->time - bar_time) % QUAVER == 0)
 			prev_note->sflags |= S_BEAM_BREAK;
+#endif
 
-		if ((s->stem = s->multi) == 0) { /* not set by set_multi() */
-						/* (voice alone on the staff) */
+		if (s->stem == 0		/* if not explicitly set */
+		    && (s->stem = s->multi) == 0) { /* and alone on the staff */
 
 			/* notes in a beam have the same stem direction */
 			if (beam)
@@ -1808,7 +2025,7 @@ static void set_overlap(void)
 		}
 
 		/* if there may be a stem clash, force a shift */
-		if (s1->stem < 0 || s2->stem >= 0) {
+		if (d != 0 && (s1->stem < 0 || s2->stem >= 0)) {
 			if (s1->ymn > s2->ymx
 			    || s1->ymx < s2->ymn)
 				continue;
@@ -1838,12 +2055,12 @@ static void set_overlap(void)
 				s2->doty = -3;
 				continue;
 			}
-			if ((l1 = s1->len) >= SEMIBREVE
-			    || (l2 = s2->len) >= SEMIBREVE)
+			if ((l1 = s1->len) >= SEMIBREVE)
+				break;
+			if ((l2 = s2->len) >= SEMIBREVE)
 				break;
 			if (cfmt.shifthnote
-			    && ((l1 = s1->len) >= MINIM
-				|| (l2 = s2->len) >= MINIM))
+			    && (l1 >= MINIM || l2 >= MINIM))
 				break;
 			if (s1->stem == s2->stem
 			    || s1->as.u.note.stemless
@@ -1859,37 +2076,30 @@ static void set_overlap(void)
 				l1 = l2;
 				l2 = s1->len;
 			}
-			if (l1 == l2 * 2
-			    || l1 == l2 * 4
-			    || l1 == l2 * 8
-			    || l1 == l2 * 16
-			    || l1 == l2 * 3
-			    || l1 * 2 == l2 * 3
-			    || l1 * 3 == l2 * 4) {
-				if (l1 < MINIM) {
-					if (s1->stem < 0) {
-						if (s1->dots > 0)
-							dy1 = -3;
-					} else if (s2->dots > 0) {
-						s2->sflags &= ~S_NO_HEAD;
-						s1->sflags |= S_NO_HEAD;
-						s2->as.u.note.accs[nhd2] =
-							s1->as.u.note.accs[0];
-						s1->as.u.note.accs[0] = 0;
-						dy2 = -3;
-					}
-					goto do_shift;
+			if (l1 < MINIM) {
+				if (s1->stem < 0) {
+					if (s1->dots > 0)
+						dy1 = -3;
+				} else if (s2->dots > 0) {
+					s2->sflags &= ~S_NO_HEAD;
+					s1->sflags |= S_NO_HEAD;
+					s2->as.u.note.accs[nhd2] =
+						s1->as.u.note.accs[0];
+					s1->as.u.note.accs[0] = 0;
+					dy2 = -3;
 				}
-				if (l2 < CROTCHET) {	/* (l1 == MINIM) */
-					if (s2->len == MINIM) {
-						s2->sflags &= ~S_NO_HEAD;
-						s1->sflags |= S_NO_HEAD;
-						s2->as.u.note.accs[nhd2] =
-							s1->as.u.note.accs[0];
-						s1->as.u.note.accs[0] = 0;
-					}
-					goto do_shift;
+				goto do_shift;
+			}
+			if (l2 < CROTCHET) {	/* (l1 >= MINIM) */
+				if (s2->len >= MINIM) {
+					s2->sflags &= ~S_NO_HEAD;
+					s1->sflags |= S_NO_HEAD;
+					s2->as.u.note.accs[nhd2] =
+						s1->as.u.note.accs[0];
+					s1->as.u.note.accs[0] = 0;
+					dy2 = -3;
 				}
+				goto do_shift;
 			}
 			s2->sflags &= ~S_NO_HEAD;
 			break;
@@ -1923,8 +2133,13 @@ static void set_overlap(void)
 		dy2 = -3;
 
 		/* if the 1st voice is below the 2nd one,
+		 * and there is no chord overlap,
 		 * shift the 1st voice */
 		if (d < 0) {
+			if (s1->stem > 0 && s1->pits[s1->nhd] >= s2->pits[0]) {
+				d2 = noteshift + 2; /* have room for the stem */
+				goto do_shift;
+			}
 			d1 = noteshift;
 			if (d > -7
 			    && s2->len < CROTCHET
@@ -1932,7 +2147,7 @@ static void set_overlap(void)
 			    && s2->as.u.note.word_end) {
 /*fixme: have higher stem??*/
 				if (s2->stem >= 0)
-					d1 += 1; /* have space for the flag */
+					d1 += 1.5; /* have room for the flag */
 			} else if (s1->pits[s1->nhd] < s2->pits[0] - 1)
 				d1 *= 0.5;
 /*fixme: check if overlap in chord*/
@@ -1945,7 +2160,7 @@ static void set_overlap(void)
 			dy1 = -3;
 			dy2 = 0;
 
-		/* if the upper note is SEMIBREVE or higher, shift the 1st voice */
+		/* if the upper note is SEMIBREVE or higher, shift it */
 		} else if (s1->len >= SEMIBREVE
 			   && s1->len > s2->len)
 			d1 = noteshift;
@@ -1954,7 +2169,7 @@ static void set_overlap(void)
 		else {
 			d2 = noteshift;
 			if (s1->dots > 0
-			    && (d != 1 || (s->pits[0] & 1)))
+			    && (d != 1 || (s1->pits[0] & 1)))
 				sd1 = 1;	/* and the dot of the 1st voice */
 		}
 
@@ -2002,15 +2217,16 @@ static void set_overlap(void)
 static void set_stems(void)
 {
 	struct SYMBOL *s, *g;
-	float slen, ymin, ymax, scale;
+	float slen, scale;
 	int ymn, ymx;
 
 	for (s = first_voice->sym; s != 0; s = s->ts_next) {
 		if (s->type != NOTE) {
+			int ymin, ymax;
+
 			if ((g = s->grace) == 0)
 				continue;
-			ymin = 0;
-			ymax = 24;
+			ymin = ymax = 12;
 			for (; g != 0; g = g->next) {
 				slen = GSTEM;
 				if (g->nflags > 1)
@@ -2019,29 +2235,31 @@ static void set_stems(void)
 				ymx = 3 * (g->pits[g->nhd] - 18);
 				if (s->stem >= 0) {
 					g->y = ymn;
-					ymx += slen;
-					g->ys = ymx;
+					g->ys = ymx + slen;
+					ymx = (int) (g->ys + 0.5);
 				} else {
 					g->y = ymx;
-					ymn -= slen;
-					g->ys = ymn;
+					g->ys = ymn - slen;
+					ymn = (int) (g->ys - 0.5);
 				}
+				ymx += 2;
+				ymn -= 2;
 				if (ymn < ymin)
-					ymin = (float) ymn;
+					ymin = ymn;
 				else if (ymx > ymax)
-					ymax = (float) ymx;
-				g->ymx = ymx + 1;
-				g->ymn = ymn - 1;
+					ymax = ymx;
+				g->ymx = ymx;
+				g->ymn = ymn;
 #ifndef YSTEP
 				g->dc_top = ymx;
 				g->dc_bot = ymn;
 #endif
 			}
-			s->ymx = (short) (ymax + 2.5);
-			s->ymn = (short) (ymin - 2.5);
+			s->ymx = ymax;
+			s->ymn = ymin;
 #ifndef YSTEP
-			s->dc_top = ymax + 2;
-			s->dc_bot = ymin - 2;
+			s->dc_top = s->ymx;
+			s->dc_bot = s->ymn;
 #endif
 			continue;
 		}
@@ -2059,10 +2277,11 @@ static void set_stems(void)
 		}
 		if ((scale = voice_tb[s->voice].scale) != 1)
 			slen *= (scale + 1) * 0.5;
-		if (s->nhd > 0)
-			slen -= 2;
 		ymn = 3 * (s->pits[0] - 18);
-		ymx = 3 * (s->pits[s->nhd] - 18);
+		if (s->nhd > 0) {
+			slen -= 2;
+			ymx = 3 * (s->pits[s->nhd] - 18);
+		} else	ymx = ymn;
 		if (s->as.u.note.stemless) {
 			if (s->stem >= 0) {
 				s->y = ymn;
@@ -2071,8 +2290,14 @@ static void set_stems(void)
 				s->ys = (float) ymn;
 				s->y = ymx;
 			}
-			ymin = (float) (ymn - 4);
-			ymax = (float) (ymx + 4);
+			if (s->nflags == -4)	/* if longa */
+				ymn -= 6;
+			s->ymx = ymx + 4;
+			s->ymn = ymn - 4;
+#ifndef YSTEP
+			s->dc_top = s->ymx;
+			s->dc_bot = s->ymn;
+#endif
 		} else if (s->stem >= 0) {
 			if (s->nflags >= 2)
 				slen -= 1;
@@ -2085,16 +2310,22 @@ static void set_stems(void)
 					slen -= 2;
 			}
 			s->y = ymn;
-			ymin = (float) (ymn - 4);
-			s->ys = ymx + slen;
-			if (s->ys < 12)
-				s->ys = 12;
-			ymax = s->ys + 2;
 			if (s->as.u.note.ti1[0] != 0)
 /*fixme
  *			    || s->as.u.note.ti2[0] != 0) */
-				ymin -= 3;
-		} else {
+				ymn -= 3;
+			s->ymn = ymn - 4;
+#ifndef YSTEP
+			s->dc_bot = s->ymn;
+#endif
+			s->ys = ymx + slen;
+			if (s->ys < 12)
+				s->ys = 12;
+			s->ymx = (int) (s->ys + 2.5);
+#ifndef YSTEP
+			s->dc_top = s->ys + 2;
+#endif
+		} else {			/* stem down */
 			if (s->pits[0] < 18
 			    && (s->nflags <= 0
 				|| !((s->sflags & S_WORD_ST)
@@ -2106,21 +2337,20 @@ static void set_stems(void)
 			s->ys = ymn - slen;
 			if (s->ys > 12)
 				s->ys = 12;
-			ymin = s->ys - 2.;
+			s->ymn = (int) (s->ys - 2.5);
+#ifndef YSTEP
+			s->dc_bot = s->ys - 2;
+#endif
 			s->y = ymx;
-			ymax = (float) (ymx + 4);
 			if (s->as.u.note.ti1[s->nhd] != 0)
 /*fixme
  *			    || s->as.u.note.ti2[s->nhd] != 0)*/
-				ymax += 3.;
-		}
-			 
-		s->ymx = (int) (ymax + 0.5);
-		s->ymn = (int) (ymin - 0.5);
+				ymx += 3;
+			s->ymx = ymx + 4;
 #ifndef YSTEP
-		s->dc_top = ymax;
-		s->dc_bot = ymin;
+			s->dc_top = s->ymx;
 #endif
+		}
 	}
 }
 
@@ -2473,9 +2703,6 @@ static void set_width(struct SYMBOL *s)
 		if (s2->type == GRACE) {
 			s->wl = wlnote - 4.5;
 			s->pl = 0;
-			wlw -= s->wl;
-			if (s2->wl < wlw - s2->wr)
-				s2->wl = wlw - s2->wr;
 		} else	s->wl = wlw;
 		break;
 	case BAR:
@@ -2534,9 +2761,15 @@ static void set_width(struct SYMBOL *s)
 		/* have room for the repeat numbers / guitar chord */
 		if (s->as.text == 0)
 			break;
-		str_font(&cfmt.font_tb[s->gcf]);
-		xx = tex_str(s->as.text)
-			+ cwid(' ') * cfmt.font_tb[s->gcf].swfac * 1.5;
+		{
+			struct FONTSPEC *f;
+
+			f = s->as.u.bar.repeat_bar ?
+				&cfmt.repeatfont : &cfmt.font_tb[s->gcf];
+			str_font(f);
+			xx = tex_str(s->as.text)
+				+ cwid(' ') * f->swfac * 1.5;
+		}
 		if (!s->as.u.bar.repeat_bar) {
 			if (s->prev->as.text != 0) {
 				float spc;
@@ -2549,12 +2782,12 @@ static void set_width(struct SYMBOL *s)
 				xx -= spc;
 			}
 		}
-/*fixme: may have gchord a bit further..*/
 		for (s2 = s->next; s2 != 0; s2 = s2->next) {
 			switch (s2->type) {
 			case PART:
 			case TEMPO:
 			case TUPLET:
+			case GRACE:
 				continue;
 			case NOTE:
 			case REST:
@@ -2568,12 +2801,6 @@ static void set_width(struct SYMBOL *s)
 			}
 			break;
 		}
-		if (s->next != 0
-		    && s->next->as.text != 0
-		    && s->next->len > 0) {	/*fixme: what if 2 bars?*/
-			AT_LEAST(s->wr, xx);
-			s->pr = s->wr;
-		}
 		break;
 	case CLEF:
 		if (s->as.u.clef.type < 0)
@@ -2631,10 +2858,6 @@ static void set_width(struct SYMBOL *s)
 		for (i = 0; i < s->as.u.meter.nmeter; i++) {
 			int l;
 
-			if (s->as.u.meter.meter[i].top[0] == ' ') {
-				w += 3;
-				continue;
-			}
 			l = strlen(s->as.u.meter.meter[i].top);
 			if (l > sizeof s->as.u.meter.meter[i].top)
 				l = sizeof s->as.u.meter.meter[i].top;
@@ -2647,14 +2870,12 @@ static void set_width(struct SYMBOL *s)
 				if (l2 > l)
 					l = l2;
 			}
-			w += 3.5 * l;
+			w += 6.5 * l;
 		}
 		s->wl = w;
 		s->wr = w + 7;
 		break;
 	case MREST:
-		if (voice_tb[s->voice].second)
-			break;			/* not displayed */
 		s->wl = 40 / 2 + 16;
 		s->wr = 40 / 2 + 16;
 		s->pl = s->wl + 16;
@@ -2674,7 +2895,7 @@ static void set_width(struct SYMBOL *s)
 		break;
 	case GRACE:
 		s->wl = set_graceoffs(s) + GSPACE * 0.8;
-		s->prev->pr -= s->wl - 10.;
+		s->prev->pr -= s->wl - 10;
 		w = GSPACE0;
 		if ((s2 = s->next) != 0
 		    && s2->type == NOTE) {
@@ -2684,10 +2905,10 @@ static void set_width(struct SYMBOL *s)
 			while (g->next != 0)
 				g = g->next;
 			if (g->y >= (float) (3 * (s2->pits[s2->nhd] - 18)))
-				w -= 1.;	/* above, a bit closer */
+				w -= 1;	/* above, a bit closer */
 			else if ((g->sflags & S_WORD_ST)
 				 && g->y < (float) (3 * (s2->pits[0] - 18) - 7))
-				w += 2.;	/* below with flag, a bit further */
+				w += 2;	/* below with flag, a bit further */
 		}
 		s->wr = w;
 		break;
@@ -2722,7 +2943,7 @@ static void check_bar(struct SYMBOL *s)
 	p_voice = &voice_tb[s->voice];
 
 	/* search the last bar */
-	while (s->type == CLEF || s->type == KEYSIG) {
+	while (s->type == CLEF || s->type == KEYSIG || s->type == TIMESIG) {
 		if ((s = s->prev) == 0)
 			return;
 	}
@@ -2776,11 +2997,14 @@ static void set_piece(struct SYMBOL *s)
 	if ((tsnext = s->ts_next) == 0)
 		return;
 
-	/* if the clef or the key signature changes on the next line,
-	 * put it at the end of the current line */
+	/* if clef key or time signature change on the next line,
+	 * put them at the end of the current line */
 	for (;;) {
 		type = tsnext->type;
 		switch (type) {
+		case TIMESIG:
+			if (!cfmt.timewarn)
+				break;
 		case CLEF:
 		case KEYSIG:
 			for (s = tsnext; s->ts_next != 0; s = s->ts_next)
@@ -2788,6 +3012,8 @@ static void set_piece(struct SYMBOL *s)
 					break;
 			if ((tsnext = s->ts_next) == 0)
 				return;
+			if (type == TIMESIG)
+				insert_meter = 1;
 			continue;
 		case FMTCHG:		/* remove the staffbreaks */
 			if (tsnext->u != STBRK)
@@ -2885,8 +3111,6 @@ static void set_xoffset(struct SYMBOL *s,
 			space *= fnnp;
 		if (s->sflags & S_IN_TUPLET)	/* reduce spacing in n-plet */
 			space *= gnnp;
-		/* fall thru */
-	case GRACE:
 		stretch = s->pl;
 		break;
 	case BAR:
@@ -2900,9 +3124,10 @@ static void set_xoffset(struct SYMBOL *s,
 
 	switch (prev->type) {
 	case NOTE:
-	case GRACE:
 	case REST:
-		stretch += prev->pr;
+	case GRACE:
+		if (last->type != GRACE)
+			stretch += prev->pr;
 		break;
 	case BAR:
 	case MREST:
@@ -2945,11 +3170,14 @@ static void set_xoffset(struct SYMBOL *s,
 /*fixme: add the dots*/
 			}
 #endif
+/*fixme: should check if at the same height*/
 #if 1
-			s->shrink = last->shrink + 5;
+			shrink = last->shrink + 5;
 #else
-			s->shrink = last->shrink + s->wl;
+			shrink = last->shrink + s->wl;
 #endif
+			if (s->shrink < shrink)
+				s->shrink = shrink;
 		}
 	}
 
@@ -3016,10 +3244,10 @@ static void set_sym_glue(float width)
 
 		/* get the notes at this time, set spacing
 		 * and get the min shrinking */
+		new_stretch = 0;
 		time = s->time;
 		seq = s->seq;
 		s4 = s;
-		new_stretch = 0;
 		s2 = s;
 		for (;;) {
 
@@ -3059,19 +3287,27 @@ static void set_sym_glue(float width)
 			/* remove some double bars */
 			if (s->next != 0
 			    && s->next->type == BAR
-			    && s->next->as.text == 0
-			    && s->next->as.u.bar.dc.n == 0
 			    && s->next->next != 0
 			    && s->next->next->type == BAR
-			    && s->next->next->as.text == 0) {
+			    && !s->next->next->as.u.bar.repeat_bar
+			    && (s->next->as.text == 0
+				|| s->next->next->as.text == 0)
+			    && (s->next->as.u.bar.dc.n == 0
+				|| s->next->next->as.u.bar.dc.n == 0)) {
 				s3 = 0;
 				if ((s->next->as.u.bar.type == B_SINGLE
 				     || s->next->as.u.bar.type == B_DOUBLE)
 				    && (s->next->next->as.u.bar.type & 0xf0)) {
 					s3 = s->next->next;
-					memcpy(&s3->prev->as.u.bar,
+					if (s->next->as.u.bar.dc.n != 0)
+						memcpy(&s3->as.u.bar.dc,
+							&s->next->as.u.bar.dc,
+							sizeof s3->as.u.bar.dc);
+					memcpy(&s->next->as.u.bar,
 						&s3->as.u.bar,
-						sizeof s3->prev->as.u.bar);
+						sizeof s->next->as.u.bar);
+					if (s3->as.text != 0)
+						s->next->as.text = s3->as.text;
 				}
 #if 0
 				if ((s->next->as.u.bar.type & 0xf0)
@@ -3080,9 +3316,9 @@ static void set_sym_glue(float width)
 					s3 = s->next->next;
 #endif
 				if (s3 != 0) {
-					delsym(s3);
 					if (s3 == s2)
 						s2 = s3->ts_next;
+					delsym(s3);
 				}
 			}
 		}
@@ -3099,14 +3335,14 @@ static void set_sym_glue(float width)
 		} else if (shrink < width)
 			continue;
 
-		/* may have a clef, key sig or staffbreak at end of line */
-/*fixme: may also have a meter change*/
+		/* may have a clef, key/time sig or staffbreak at EOL */
 		switch (s->type) {
 		case FMTCHG:
 			if (s->u != STBRK)
 				break;
-		case KEYSIG:
 		case CLEF:
+		case KEYSIG:
+		case TIMESIG:
 			continue;
 		}
 		s = s4->ts_prev;
@@ -3525,8 +3761,11 @@ void output_music(void)
 	check_buffer();	/* dump buffer if not enough space for a music line */
 
 	set_global();			/* set global characteristics */
-	if (first_voice->next != 0)	/* when multi-voices */
+	if (first_voice->next != 0) {	/* if many voices */
+		if (cfmt.combinevoices)
+			combine_voices();
 		set_multi();		/* set the stems direction in 'multi' */
+	}
 	for (p_voice = first_voice; p_voice; p_voice = p_voice->next)
 		set_beams(p_voice->sym);	/* decide on beams */
 	set_stems();			/* set the stem lengths */
@@ -3540,7 +3779,7 @@ void output_music(void)
 			/ cfmt.scale;
 	if (lwidth < 1 CM) {
 		error(1, 0, "Bad page width %.1f", lwidth);
-		lwidth = 1 CM;
+		lwidth = 20 CM;
 	}
 	for (;;) {		/* loop over pieces of line for output */
 		float indent, line_height;
@@ -3578,6 +3817,7 @@ void output_music(void)
 	for (voice = MAXVOICE; --voice >= 0; ) {
 		voice_tb[voice].sym = 0;
 		voice_tb[voice].time = 0;
+		voice_tb[voice].have_ly = 0;
 	}
 }
 
