@@ -3,7 +3,7 @@
  *
  * This file is part of abcm2ps.
  *
- * Copyright (C) 1998-2005 Jean-Franois Moine
+ * Copyright (C) 1998-2006 Jean-Franois Moine
  * Adapted from abc2ps, Copyright (C) 1996,1997 Michael Methfessel
  *
  * This program is free software; you can redistribute it and/or modify
@@ -37,7 +37,8 @@ struct VOICE_S voice_tb[MAXVOICE];	/* voice table */
 static struct VOICE_S *curvoice;	/* current voice while parsing */
 struct VOICE_S *first_voice;		/* first voice */
 
-static struct FORMAT dfmt;		/* format at start of tune */
+struct FORMAT dfmt;		/* current global format */
+unsigned short *micro_tb;	/* ptr to the microtone table of the tune */
 
 static int lyric_nb;			/* current number of lyric lines */
 static struct SYMBOL *lyric_start;	/* 1st note of the line for w: */
@@ -214,7 +215,7 @@ static void gchord_adjust(struct SYMBOL *s)
 static char *get_lyric(char *p)
 {
 	struct SYMBOL *s;
-	char word[81], *q;
+	char word[128], *q;
 	int ln;
 	struct FONTSPEC *f;
 
@@ -233,7 +234,7 @@ static char *get_lyric(char *p)
 
 	/* scan the lyric line */
 	while (*p != '\0') {
-		while (isspace((unsigned) *p))
+		while (isspace((unsigned char) *p))
 			p++;
 		if (*p == '\0')
 			break;
@@ -350,21 +351,28 @@ static char *get_lyric(char *p)
 static void get_over(struct SYMBOL *s)
 {
 	struct VOICE_S *p_voice, *p_voice2;
+#if 0
 	struct SYMBOL *s1, *s2;
-	int ctime, linenum, colnum;
+	int linenum, colnum;
+#endif
+	int ctime;
 
 	/* treat the end of overlay */
 	p_voice = curvoice;
+#if 0
 	linenum = s != 0 ? s->as.linenum : 0;
 	colnum = s != 0 ? s->as.colnum : 0;
-#if 0
+#endif
+#if 1
 	if (over_bar) {
 		struct SYMBOL *s2;
 
 		s2 = add_sym(p_voice, BAR);
 		s2->as.type = ABC_T_BAR;
-		s2->as.linenum = linenum;
-		s2->as.colnum = colnum;
+		if (s != 0) {
+			s2->as.linenum = s->as.linenum;
+			s2->as.colnum = s->as.colnum;
+		}
 	}
 #endif
 	if (s == 0
@@ -396,8 +404,8 @@ static void get_over(struct SYMBOL *s)
 	if (p_voice2->name == 0) {
 		p_voice2->name = "&";
 		p_voice2->second = 1;
-		p_voice2->staff = p_voice->staff;
-		p_voice2->cstaff = p_voice2->staff;
+		p_voice2->cstaff = p_voice2->staff = p_voice->staff;
+		p_voice2->scale = p_voice->scale;
 #if 0
 		memcpy(&p_voice2->clef, &p_voice->clef,
 			sizeof p_voice2->clef);
@@ -410,6 +418,29 @@ static void get_over(struct SYMBOL *s)
 		p_voice2->prev = p_voice;
 	}
 
+#if 1
+	if (voice_over == 0) {
+		voice_over = s;
+		over_bar = 1;
+		ctime = p_voice2->time;
+		for (s = p_voice->last_symbol; /*s != 0*/; s = s->prev)
+			if (s->type == BAR || s->time <= ctime)
+				break;
+	} else {
+		struct SYMBOL *tmp;
+
+		tmp = s;
+		s = (struct SYMBOL *) voice_over->as.next;
+/*fixme: what if this symbol is not in the voice?*/
+		if (s->voice != curvoice - voice_tb) {
+			error(1, s, "Voice overlay not closed");
+			voice_over = 0;
+			return;
+		}
+		voice_over = tmp;
+	}
+	ctime = s->time;
+#else
 	/* search the start of sequence */
 	ctime = p_voice2->time;
 	if (voice_over == 0) {
@@ -431,7 +462,6 @@ static void get_over(struct SYMBOL *s)
 		}
 		voice_over = tmp;
 	}
-
 	/* search the last common sequence */
 	for (s1 = s; /*s1 != 0*/; s1 = s1->prev)
 		if (s1->time <= ctime)
@@ -480,6 +510,7 @@ static void get_over(struct SYMBOL *s)
 				break;
 		}
 	}
+#endif
 	p_voice2->time = ctime;
 	curvoice = p_voice2;
 }
@@ -628,10 +659,7 @@ static void get_staves(struct SYMBOL *s)
 			}
 		}
 	}
-	if (staff < 0)
-		staff = 0;
-	nstaff = staff;
-	return;
+	goto ok;
 
 	/* when error, let one voice per staff */
 err:
@@ -640,9 +668,12 @@ err:
 	     p_voice != 0;
 	     p_voice = p_voice->next, staff++)
 		p_voice->staff = p_voice->cstaff = staff;
+ok:
 	if (staff < 0)
 		staff = 0;
 	nstaff = staff;
+	if (cfmt.alignbars)
+		cfmt.alignbars = nstaff + 1;
 }
 
 /* -- re-initialize all potential voices -- */
@@ -690,18 +721,15 @@ static char *state_txt[4] = {
 };
 static void get_info(struct SYMBOL *s,
 		     int info_type,
-		     char *p,
 		     struct abctune *t)
 {
 	struct ISTRUCT *inf;
+	char *p;
 
 	/* change global or local */
 	inf = s->as.state == ABC_S_GLOBAL ? &default_info : &info;
 	lvlarena(s->as.state != ABC_S_GLOBAL);
 
-	while (isspace((unsigned) *p))
-	       p++;
-
 	switch (info_type) {
 	case 'A':
 		add_info(s, &inf->area);
@@ -837,6 +865,9 @@ static void get_info(struct SYMBOL *s,
 			break;
 		if (lyric_start == 0)
 			break;
+		p = &s->as.text[2];
+		while (isspace((unsigned char) *p))
+		       p++;
 		if ((p = get_lyric(p)) != 0)
 			error(1, s, "%s", p);
 		curvoice->have_ly = 1;
@@ -852,6 +883,9 @@ newtune:
 			write_buffer();	/* flush stuff left from %% lines */
 		dfmt = cfmt;		/* save the format at start of tune */
 		memcpy(&info, &default_info, sizeof info);
+		p = &s->as.text[2];
+		while (isspace((unsigned char) *p))
+		       p++;
 		info.xref = p;
 		if (info_type == 'T')
 			info.title = s;
@@ -1024,7 +1058,6 @@ void do_tune(struct abctune *t,
 	nstaff = 0;
 	memset(staff_tb, 0, sizeof staff_tb);
 	staves_found = 0;
-
 	for (i = MAXVOICE; --i >= 0; ) {
 		voice_tb[i].clef.line = 2;	/* treble clef on 2nd line */
 		voice_tb[i].clef.stafflines = 5;
@@ -1039,6 +1072,7 @@ void do_tune(struct abctune *t,
 	voice_init();		/* initialize all the voices */
 	for (i = 0; i < nwhistle; i++)
 		voice_tb[whistle_tb[i].voice].whistle = 1;
+	micro_tb = t->micro_tb;	/* microtone values */
 
 	curvoice = first_voice = voice_tb;
 	if (cfmt.oneperpage) {
@@ -1068,22 +1102,19 @@ void do_tune(struct abctune *t,
 		switch (as->type) {
 		case ABC_T_INFO: {
 			int info_type;
-			char *p;
 
 			if (header_only
 			    && (as->text[0] == 'X'
 				|| as->text[0] == 'T'))
 				continue;
 			info_type = as->text[0];
-			p = &as->text[2];
 			for (;;) {
-				get_info(s, info_type, p, t);
+				get_info(s, info_type, t);
 				if (as->next == 0
 				    || as->next->type != ABC_T_INFO2)
 					break;
 				as = as->next;
 				s = (struct SYMBOL *) as;
-				p = &as->text[0];
 			}
 			break;
 		    }
@@ -1111,12 +1142,25 @@ void do_tune(struct abctune *t,
 		case ABC_T_EOLN:
 			if (curvoice->last_symbol != 0)
 				curvoice->last_symbol->sflags |= S_EOLN;
+			if (!cfmt.alignbars)
+				continue;
+			i = (curvoice - voice_tb) + 1;
+			if (i < cfmt.alignbars) {
+				curvoice = &voice_tb[i];
+				continue;
+			}
+			output_music();
+			curvoice = &voice_tb[0];
 			continue;
 		case ABC_T_MREST:
 		case ABC_T_MREP: {
 			int len;
 
 			len = curvoice->wmeasure * as->u.bar.len;
+			if (curvoice->second) {
+				curvoice->time += len;
+				break;
+			}
 			if (as->type == ABC_T_MREP
 			    && as->u.bar.len > 1) {
 
@@ -1176,8 +1220,10 @@ void do_tune(struct abctune *t,
 			write_eps();
 	} else	write_buffer();
 
-	if (info.xref != 0)
+	if (info.xref != 0) {
 		cfmt = dfmt;	/* restore the format at start of tune */
+		info.xref = 0;
+	}
 }
 
 /* -- get a clef definition (in K: or V:) -- */
@@ -1404,6 +1450,9 @@ static void get_voice(struct SYMBOL *s)
 	if (p_voice->prev == 0 && p_voice != first_voice) {	/* new voice */
 		struct VOICE_S *p_voice2;
 
+		if (cfmt.alignbars) {
+			error(1, s, "V: does not work with %%%%alignbars");
+		}
 		if (!staves_found) {
 			if (!s->as.u.voice.merge) {
 #if MAXSTAFF < MAXVOICE
@@ -1466,7 +1515,7 @@ static void get_note(struct SYMBOL *s)
 		if (grace_head != 0)
 			grace_head = 0;
 		sym_link(s, type);
-		s->multi = curvoice->stem;
+		s->stem = curvoice->stem;
 	} else {			/* in a grace note sequence */
 		int div;
 
@@ -1478,7 +1527,7 @@ static void get_note(struct SYMBOL *s)
 			s2->as.colnum = s->as.colnum;
 			grace_head = s2;
 			grace_head->grace = grace_tail = s;
-			s2->multi = curvoice->gstem;
+			s2->stem = curvoice->gstem;
 		} else {
 			grace_tail->next = s;
 			s->prev = grace_tail;
@@ -1620,7 +1669,7 @@ static void ps_def(struct SYMBOL *s,
 		PUT1("%s\n", p);
 		return;
 	}
-	interpret_format_line("postscript", p);
+	interpret_fmt_line("postscript", p, 0);
 }
 
 /* -- process a pseudo-comment (%% or I:) -- */
@@ -1631,11 +1680,11 @@ static struct abcsym *process_pscomment(struct abcsym *as)
 	struct SYMBOL *s = (struct SYMBOL *) as;
 
 	p = as->text + 2;		/* skip '%%' */
-	while (isspace((unsigned) *p))
+	while (isspace((unsigned char) *p))
 		p++;
 	if (strncasecmp(p, "fmt ", 4) == 0) {
 		p += 4;			/* skip 'fmt' */
-		while (isspace((unsigned) *p))
+		while (isspace((unsigned char) *p))
 			p++;
 	}
 	p = get_str(w, p, sizeof w);
@@ -1649,11 +1698,11 @@ static struct abcsym *process_pscomment(struct abcsym *as)
 				p = as->text;
 				if (*p == '%' && p[1] == '%') {
 					p += 2;
-					while (isspace((unsigned) *p))
+					while (isspace((unsigned char) *p))
 						p++;
 					if (strncasecmp(p, "fmt ", 4) == 0) {
 						p += 4;
-						while (isspace((unsigned) *p))
+						while (isspace((unsigned char) *p))
 							p++;
 					}
 					if (strncmp(p, "endps", 5) == 0)
@@ -1690,11 +1739,11 @@ static struct abcsym *process_pscomment(struct abcsym *as)
 				p = as->text;
 				if (*p == '%' && p[1] == '%') {
 					p += 2;
-					while (isspace((unsigned) *p))
+					while (isspace((unsigned char) *p))
 						p++;
 					if (strncasecmp(p, "fmt ", 4) == 0) {
 						p += 4;
-						while (isspace((unsigned) *p))
+						while (isspace((unsigned char) *p))
 							p++;
 					}
 					if (strncmp(p, "endtext", 7) == 0) {
@@ -1730,21 +1779,15 @@ static struct abcsym *process_pscomment(struct abcsym *as)
 						break;
 				}
 			}
+			fclose(epsf);
 			if (strncmp(line, "%%BoundingBox:", 14) != 0) {
 				error(1, s,
 				      "No bounding box in '%s'", p);
 				return as;
 			}
+			PUT0("\001");
 			abskip((y2 - y1) * cfmt.scale);
-			PUT3("%%start EPS file '%s'\nsave\n"
-			     "/showpage {} def /setpagedevice {pop} def\n"
-			     "%.2f %.2f T\n", 
-			     p, -x1, -y1);
-			rewind(epsf);
-			while (fgets(line, sizeof line, epsf))	/* copy the file */
-				PUT1("%s", line);
-			fclose(epsf);
-			PUT0("restore\n%%end EPS\n");
+			PUT3("%.2f %.2f %s\n", x1, y1, p);
 			buffer_eob();
 			return as;
 		}
@@ -1756,10 +1799,9 @@ static struct abcsym *process_pscomment(struct abcsym *as)
 			output_music();
 			if (strncmp(p, "start", 5) == 0) {
 				buffer_eob();
-				if ((bposy = get_bposy()) == 0) {
+				if (!in_page)
 					PUT0("%%");	/* initialize the output */
-					bposy = get_bposy();
-				}
+				bposy = get_bposy();
 				multicol_max = multicol_start = bposy;
 				lmarg = cfmt.leftmargin;
 				rmarg = cfmt.rightmargin;
@@ -1803,7 +1845,7 @@ static struct abcsym *process_pscomment(struct abcsym *as)
 			output_music();
 			write_buffer();
 			use_buffer = 0;
-			if (isdigit((unsigned) *p))
+			if (isdigit((unsigned char) *p))
 				pagenum = atoi(p);
 			close_page();
 			return as;
@@ -1842,16 +1884,16 @@ static struct abcsym *process_pscomment(struct abcsym *as)
 			h1 = h2 = len = 0;
 			if (*p != '\0') {
 				h1 = scan_u(p);
-				while (*p != '\0' && !isspace((unsigned) *p))
+				while (*p != '\0' && !isspace((unsigned char) *p))
 					p++;
-				while (isspace((unsigned) *p))
+				while (isspace((unsigned char) *p))
 					p++;
 			}
 			if (*p != '\0') {
 				h2 = scan_u(p);
-				while (*p != '\0' && !isspace((unsigned) *p))
+				while (*p != '\0' && !isspace((unsigned char) *p))
 					p++;
-				while (isspace((unsigned) *p))
+				while (isspace((unsigned char) *p))
 					p++;
 			}
 			if (*p != '\0')
@@ -1876,7 +1918,11 @@ static struct abcsym *process_pscomment(struct abcsym *as)
 			if (as->state != ABC_S_TUNE
 			    && as->state != ABC_S_EMBED)
 				return as;
-			staff = atoi(p) - 1;
+			if (*p == '+')
+				staff = curvoice->cstaff + atoi(p + 1);
+			else if (*p == '-')
+				staff = curvoice->cstaff - atoi(p + 1);
+			else	staff = atoi(p) - 1;
 			if ((unsigned) staff > nstaff) {
 				error(1, s, "Bad staff in %%%%staff");
 				return as;
@@ -1944,8 +1990,34 @@ static struct abcsym *process_pscomment(struct abcsym *as)
 			buffer_eob();
 		}
 	}
-	if (interpret_format_line(w, p) == 0)
-		ops_into_fmt();
+	interpret_fmt_line(w, p, 0);
+	if (cfmt.alignbars && strcmp(w, "alignbars") == 0) {
+		struct VOICE_S *p_voice;
+		int i;
+
+		output_music();
+		if ((unsigned) cfmt.alignbars > MAXSTAFF) {
+			error(1, s, "Too big value in %%%%alignbars");
+			cfmt.alignbars = MAXSTAFF;
+		}
+		if (staves_found)
+			cfmt.alignbars = nstaff + 1;
+		first_voice = curvoice = p_voice = &voice_tb[0];
+		for (i = 0; i < cfmt.alignbars; i++) {
+			staff_tb[i].stop_bar = 1;
+			p_voice->staff = p_voice->cstaff = i;
+			if (i > 0)
+				p_voice->prev = p_voice - 1;
+			else	p_voice->prev = 0;
+			if (i >= cfmt.alignbars - 1) {
+				p_voice->next = 0;
+				break;
+			}
+			p_voice->next = p_voice + 1;
+			p_voice++;
+		}
+		nstaff = i;
+	}
 	return as;
 }
 
@@ -2008,8 +2080,10 @@ static void set_tuplet(struct SYMBOL *t)
 		oldlen = s->len;
 		s->len = (oldlen * lplet) / l;
 		if (--r <= 0) {
+#if 0
 			if (oldlen < QUAVER)
-				s->sflags |= S_BEAM_BREAK;
+				s->sflags |= S_BEAM_BR1;
+#endif
 			break;
 		}
 		l -= oldlen;
