@@ -3,7 +3,7 @@
  *
  * This file is part of abcm2ps.
  *
- * Copyright (C) 1998-2005 Jean-Franois Moine
+ * Copyright (C) 1998-2006 Jean-Franois Moine
  * Adapted from abc2ps, Copyright (C) 1996,1997 Michael Methfessel
  *
  * This program is free software; you can redistribute it and/or modify
@@ -200,6 +200,8 @@ static struct u_ps {
 	char text[2];
 } *user_ps;
 
+static char *trim_title(char *p);
+
 /* -- print message for internal error and maybe stop -- */
 void bug(char *msg, int fatal)
 {
@@ -293,6 +295,14 @@ float cwid(unsigned char c)
 	return (float) w[c] / 1000.;
 }
 
+/* -- memorize the current font in case of page break -- */
+static void set_font_init(void)
+{
+	if (strcf < 0)
+		return;
+	font_init = &cfmt.font_tb[strcf];
+}
+
 /* -- change string taking care of some tex-style codes -- */
 /* Puts \ in front of ( and ) in case brackets are not balanced,
  * interprets all ISOLatin1..6 escape sequences as defined in rfc1345.
@@ -422,12 +432,14 @@ void str_font(struct FONTSPEC *font)
 static void str_ft_out1(char *p, int l)
 {
 	if (strfc) {
+		strfc = 0;
 		if (strtx) {
 			PUT1(")%s ", strop);
 			strtx = 0;
 		}
 		set_font(&cfmt.font_tb[strcf]);
-		strfc = 0;
+		if (font_init != 0)
+			set_font_init();
 	}
 	if (!strtx) {
 		PUT0("(");
@@ -559,19 +571,15 @@ static void put_inf2r(struct SYMBOL *s1,
 		s1 = s2;
 		s2 = 0;
 	}
-	if (s2 == 0) {
-		p = s1->as.text;
-		if (p[1] == ':')
-			p += 2;
-		while (isspace((unsigned char) *p))
-			p++;
-	} else {
+	p = s1->as.text;
+	if (p[1] == ':')
+		p += 2;
+	while (isspace((unsigned char) *p))
+		p++;
+	if (s1->as.text[0] == 'T' && s1->as.text[1] == ':')
+		p = trim_title(p);
+	if (s2 != 0) {
 		buf[sizeof buf - 1] = '\0';
-		p = s1->as.text;
-		if (p[1] == ':')
-			p += 2;
-		while (isspace((unsigned char) *p))
-			p++;
 		strncpy(buf, p, sizeof buf - 1);
 		q = buf + strlen(buf);
 		if (q < buf + sizeof buf - 4) {
@@ -592,14 +600,6 @@ static void put_inf2r(struct SYMBOL *s1,
 	put_str(p, action);
 }
 
-/* -- memorize the current font in case of page break -- */
-static void set_font_init(void)
-{
-	if (strcf < 0)
-		return;
-	font_init = &cfmt.font_tb[strcf];
-}
-
 /* -- add text to a block -- */
 void add_to_text_block(char *s, int job)
 {
@@ -884,7 +884,7 @@ void put_words(struct SYMBOL *words)
 	font_init = 0;
 }
 
-/* -- output text lines -- */
+/* -- output history lines -- */
 static void put_text(char *head,
 		     struct SYMBOL *s)
 {
@@ -892,12 +892,13 @@ static void put_text(char *head,
 
 	if (s == 0)
 		return;
+	if (strcf != 0) {
+		set_font(&cfmt.historyfont);
+		strcf = 0;
+		set_font_init();
+	}
 	h = font_init->size * cfmt.lineskipfac;
 	bskip(h * 1.2);
-	if (strcf != 0) {
-		set_font(font_init);
-		strcf = 0;
-	}
 	PUT1("w%s ", head);
 	bskip(h);
 	do {
@@ -912,11 +913,7 @@ static void put_text(char *head,
 void put_history(void)
 {
 	bskip(cfmt.textspace);
-	font_init = &cfmt.historyfont;
 	str_font(&cfmt.historyfont);
-	set_font(&cfmt.historyfont);
-	strcf = 0;
-	set_font_init();
 	if (!cfmt.infoline)
 		put_text("rhythm", info.rhythm);
 	put_text("book", info.book);
@@ -929,41 +926,48 @@ void put_history(void)
 }
 
 /* -- trim_title: move trailing "The" to front -- */
-static void trim_title(char *p)
+static char *trim_title(char *p)
 {
 	char *q;
 	int l;
+static char buf[256];
 
 	l = strlen(p);
 	q = p + l - 3;
-	if (strcmp(q, "The") != 0)
-		return;
-	q--;
-	while (isspace((unsigned char) *q))
+	if (strcmp(q, "The") == 0) {
 		q--;
-	if (*q != ',')
-		return;
-	l = q - p;
-	memmove(p + 4, p, l);
-	memcpy(p, "The ", 4);
-	p[l + 4] = '\0';
+		while (isspace((unsigned char) *q))
+			q--;
+		if (*q == ',') {
+			strcpy(buf, "The ");
+			l = q - p;
+			if (l > sizeof buf - 4 - 1)
+				l = sizeof buf - 4 - 1;
+			strncpy(&buf[4], p, l);
+			buf[l + 4] = '\0';
+			return buf;
+		}
+	}
+	if (cfmt.titlecaps) {
+		strncpy(buf, p, sizeof buf - 1);
+		p = buf;
+		p[sizeof buf - 1] = '\0';
+		cap_str(p);
+	}
+	return p;
 }
 
 /* -- write a title -- */
 void write_title(struct SYMBOL *s)
 {
-	char *p, t[200];
+	char *p;
 
 	p = &s->as.text[2];
 	while (isspace((unsigned char) *p))
 		p++;
 	if (*p == '\0' || strcmp(p, "(notitle)") == 0)
 		return;
-	strncpy(t, p, sizeof t - 1);
-	t[sizeof t - 1] = '\0';
-	trim_title(t);
-	if (cfmt.titlecaps)
-		cap_str(t);
+	p = trim_title(p);
 	if (s == info.title) {
 		bskip(cfmt.titlespace + cfmt.titlefont.size);
 		set_font(&cfmt.titlefont);
@@ -978,10 +982,141 @@ void write_title(struct SYMBOL *s)
 		     - cfmt.leftmargin - cfmt.rightmargin) / cfmt.scale);
 	if (s == info.title && cfmt.withxrefs)
 		PUT1("%s.  ", info.xref);
-	tex_str(t);
+	tex_str(p);
 	PUT2("%s)show%s\n", tex_buf, cfmt.titleleft ? "" : "c");
 }
 
+/* -- write_heading with format -- */
+static void write_headform(float lwidth)
+{
+	char *p;
+	struct SYMBOL *s, *title, xref_sym;
+	struct FONTSPEC *f;
+	int align;
+	float x, y, yl, yc, yr;
+
+	title = info.title;
+	yl = yc = yr = 0;
+	p = cfmt.titleformat;
+	for (;;) {
+		f = &cfmt.historyfont;
+		while (isspace((unsigned char) *p))
+			p++;
+		switch (*p++) {
+		case '\0':
+			if (yc > yl)
+				yl = yc;
+			if (yr > yl)
+				yl = yr;
+			yc = yr = yl;
+			bskip(yl);
+			return;
+		case 'A':
+			s = info.area;
+			f = &cfmt.infofont;
+			break;
+		case 'B':
+			s = info.book;
+			break;
+		case 'C':
+			s = info.comp;
+			f = &cfmt.composerfont;
+			break;
+		case 'D':
+			s = info.disco;
+			break;
+		case 'H':
+			s = info.histo;
+			break;
+		case 'N':
+			s = info.notes;
+			break;
+		case 'O':
+			s = info.orig;
+			f = &cfmt.composerfont;
+			break;
+		case 'P':
+			s = info.parts;
+			f = &cfmt.partsfont;
+			break;
+		case 'R':
+			s = info.rhythm;
+			f = &cfmt.infofont;
+			break;
+		case 'S':
+			s = info.src;
+			break;
+		case 'T':
+			s = title;
+			f = (s == info.title) ? &cfmt.titlefont
+					 : &cfmt.subtitlefont;
+			break;
+		case 'X':
+			s = &xref_sym;
+			s->as.text = info.xref;
+			s->next = 0;
+			f = &cfmt.titlefont;
+			break;
+		case 'Z':
+			s = info.trans;
+			break;
+		case ',':
+			if (yc > yl)
+				yl = yc;
+			if (yr > yl)
+				yl = yr;
+			yc = yr = yl;
+			continue;
+		default:
+			continue;
+		}
+		if (s == 0)
+			continue;
+		switch (*p) {
+		default:
+			align = A_CENTER;
+			x = lwidth * 0.5;
+			y = yc;
+			break;
+		case '1':
+			align = A_RIGHT;
+			x = lwidth;
+			y = yr;
+			p++;
+			break;
+		case '-':
+			align = A_LEFT;
+			x = 0;
+			y = yl;
+			p++;
+			break;
+		}
+		do {
+			str_font(f);
+			PUT2("%.1f %.1f M ", x, -y);
+			put_inf2r(s, 0, align);
+			y += f->size * 1.1;
+			if (s == title) {
+				title = title->next;
+				break;
+			}
+			s = s->next;
+		} while (s != 0);
+		switch (align) {
+		case A_LEFT:
+			yl = y;
+			break;
+		case A_RIGHT:
+			yr = y;
+			break;
+		default:
+			yc = y;
+			break;
+		}
+	}
+	/*not reached*/
+}
+
 /* -- write_heading -- */
 void write_heading(struct abctune *t)
 {
@@ -991,6 +1126,11 @@ void write_heading(struct abctune *t)
 	lwidth = ((cfmt.landscape ? cfmt.pageheight : cfmt.pagewidth)
 		- cfmt.leftmargin - cfmt.rightmargin) / cfmt.scale;
 
+	if (cfmt.titleformat != 0) {
+		write_headform(lwidth);
+		return;
+	}
+
 	/* titles */
 	for (s = info.title; s != 0; s = s->next)
 		write_title(s);
