@@ -3,7 +3,7 @@
  *
  * This file is part of abcm2ps.
  *
- * Copyright (C) 1998-2005 Jean-Franois Moine
+ * Copyright (C) 1998-2006 Jean-Franois Moine
  * Adapted from abc2ps, Copyright (C) 1996,1997 Michael Methfessel
  *
  * This program is free software; you can redistribute it and/or modify
@@ -131,9 +131,11 @@ static char ps_head[] =
 
 	/* str x y bnum - tuplet number / ratio */
 	"/bnum{M/Times-Italic 12 selectfont showc}!\n"
-	"/bnumb{	currentgray 3 index stringwidth pop 4 add\n"
-	"        dup .5 mul neg 4 index add 3 index 3 -1 roll 8\n"
-	"        1.0 setgray rectfill setgray bnum}!\n"
+	/* same with clearing below the number */
+	"/bnumb{	currentgray /Times-Italic 12 selectfont\n"
+	"	3 index stringwidth pop 4 add\n"
+	"	dup .5 mul neg 4 index add 3 index 3 -1 roll 8\n"
+	"	1.0 setgray rectfill setgray M showc}!\n"
 
 	/* dx dy x y tubr - tuplet bracket */
 	"/tubr{3 sub M 0 3 RL RL 0 -3 RL dlw stroke}!\n"
@@ -559,7 +561,7 @@ static char ps_head[] =
 
 	/* (meter) x y stsig - single time signature */
 	"/stsig{	M gsave/Times-Bold 18 selectfont 1.2 1 scale\n"
-	"	0 6 RM showc grestore}!\n"
+	"	0 7 RM showc grestore}!\n"
 
 	/* l y n staff - staff with n lines*/
 	"/staff{	dlw 1 1 3 -1 roll{\n"
@@ -610,10 +612,10 @@ static char ps_head[] =
 	"	-7 4 RM 14 10 RL -14 -4 RM 14 10 RL\n"
 	"	stroke}!\n"
 
-	/* str bracket_type dx x y repbra - repeat bracket */
+	/* str dy bracket_type dx x y repbra - repeat bracket */
 	"/repbra{gsave dlw T 0 -20 M\n"
 	"	0 20 3 index 1 ne{RL}{RM}ifelse 0 RL 0 ne{0 -20 RL}if stroke\n"
-	"	4 -13 M show grestore}!\n"
+	"	4 exch M show grestore}!\n"
 
 	/* pp2x pp1x p1 pp1 pp2 p2 p1 SL - slur / tie */
 	"/SL{M curveto RL curveto closepath fill}!\n"
@@ -628,7 +630,7 @@ static char ps_head[] =
 	"/jshow{w 0 32 4 -1 roll widthshow}!\n"
 
 	/* texts which may be translated */
-	"/wbook{0 0 M (Book:)show}!\n"
+	"/wbook{0 0 M(Book:)show}!\n"
 	"/wdisco{0 0 M(Discography:)show}!\n"
 	"/whisto{0 0 M(History:)show}!\n"
 	"/wnotes{0 0 M(Notes:)show}!\n"
@@ -682,8 +684,8 @@ static char ps_head[] =
 	/* -- default percussion heads -- */
 	/* x y pshhd - percussion sharp head */
 	"/pshhd{/x 2 index def/y 1 index def dsh0}!\n"
-	/* x y pflhd - percussion flat head */
-	"/pflhd{/x 2 index def/y 1 index def dsh0\n"
+	/* x y pfthd - percussion flat head */
+	"/pfthd{/x 2 index def/y 1 index def dsh0\n"
 	"	.7 SLW x y 4 0 360 arc stroke}!\n"
 
 	/* x y ghd - grace note head */
@@ -853,7 +855,7 @@ void define_encoding(int enc,		/* index */
 	}
 	fprintf(fout, "/mkfontext%d{\n"
 		"	findfont dup length dict begin\n"
-                "		{1 index/FID ne{def}{pop pop}ifelse}forall\n"
+		"		{1 index/FID ne{def}{pop pop}ifelse}forall\n"
 		"		/Encoding %s def\n"
 		"		currentdict\n"
 		"	end\n"
@@ -909,8 +911,7 @@ void define_symbols(void)
 		"		1.2 -3.2 9.6 -5.7 5.6 -14.6 RC\n"
 		"		1.6 5.4 -1 10.2 -5.6 11.4 RC\n"
 		"		fill\n"
-		"	  }\n"
-		"	ifelse}!\n",
+		"	  }ifelse}!\n",
 		STEM_XOFF, STEM_YOFF, STEM_YOFF);
 
 	/* n len sfd - stem and n flags down */
@@ -932,21 +933,34 @@ void define_symbols(void)
 		"		1.2 3.2 9.6 5.7 5.6 14.6 RC\n"
 		"		1.6 -5.4 -1 -10.2 -5.6 -11.4 RC\n"
 		"		fill\n"
-		"	  }\n"
-		"	ifelse}!\n",
+		"	  }ifelse}!\n",
 		-STEM_XOFF, -STEM_YOFF, STEM_YOFF);
 
 	/* n len sfs - stem and n straight flag down */
-	fprintf(fout, "/sfs{	dlw x y M %.1f %.1f RM\n"
-		"	%.1f add 0 exch RL currentpoint stroke\n"
-		"	M 1 1 3 -1 roll{\n"
-		"		pop currentpoint\n"
-		"		7 %.1f RL\n"
-		"		0 %.1f RL\n"
-		"		-7 -%.1f RL\n"
-		"		fill 5.4 add M\n"
-		"	}for}!\n",
-		-STEM_XOFF, -STEM_YOFF, STEM_YOFF,
+	fprintf(fout, "/sfs{	dup 0 lt{\n"
+		"		dlw x y M -%.1f -%.1f RM\n"
+		"		%.1f add 0 exch RL currentpoint stroke\n"
+		"		M 1 1 3 -1 roll{\n"
+		"			pop currentpoint\n"
+		"			7 %.1f RL\n"
+		"			0 %.1f RL\n"
+		"			-7 -%.1f RL\n"
+		"			fill 5.4 add M\n"
+		"		}for\n"
+		"	}{\n"
+		"		dlw x y M %.1f %.1f RM\n"
+		"		-%.1f add 0 exch RL currentpoint stroke\n"
+		"		M 1 1 3 -1 roll{\n"
+		"			pop currentpoint\n"
+		"			7 -%.1f RL\n"
+		"			0 -%.1f RL\n"
+		"			-7 %.1f RL\n"
+		"			fill -5.4 add M\n"
+		"		}for\n"
+		"	}ifelse}!\n",
+		STEM_XOFF, STEM_YOFF, STEM_YOFF,
+		BEAM_DEPTH, BEAM_DEPTH, BEAM_DEPTH,
+		STEM_XOFF, STEM_YOFF, STEM_YOFF,
 		BEAM_DEPTH, BEAM_DEPTH, BEAM_DEPTH);
 
 	/* len gu - grace note stem up */
